/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.param;

import org.apache.hivemind.ClassResolver;
import org.apache.hivemind.util.PropertyUtils;
import org.apache.tapestry.IBinding;
import org.apache.tapestry.IComponent;
import org.apache.tapestry.IForm;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.form.Form;
import org.apache.tapestry.form.IFormComponent;
import org.apache.tapestry.spec.Direction;
import org.apache.tapestry.spec.IParameterSpecification;

/**
 * Standard implementation of {@link IParameterConnector}. Subclasses add in the ability to clear
 * parameters.
 * 
 * @author Howard Lewis Ship
 * @since 2.0.3
 */

public abstract class AbstractParameterConnector implements IParameterConnector
{public static com.cortexeb.tools.clover.d __CLOVER_203_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1097439627296L);
    private String _parameterName;

    private String _propertyName;

    private IBinding _binding;

    private IComponent _component;

    private boolean _required;

    private Object _clearValue;

    private Direction _direction;

    /**
     * Creates a connector. In addition, obtains the current value of the component property; this
     * value will be used to restore the component property.
     */

    protected AbstractParameterConnector(IComponent component, String parameterName,
            IBinding binding)
    {try { __CLOVER_203_0.M[3101]++;
        __CLOVER_203_0.S[12879]++;_component = component;
        __CLOVER_203_0.S[12880]++;_parameterName = parameterName;
        __CLOVER_203_0.S[12881]++;_binding = binding;

        __CLOVER_203_0.S[12882]++;IParameterSpecification pspec = _component.getSpecification().getParameter(_parameterName);
        __CLOVER_203_0.S[12883]++;_required = pspec.isRequired();
        __CLOVER_203_0.S[12884]++;_propertyName = pspec.getPropertyName();
        __CLOVER_203_0.S[12885]++;_direction = pspec.getDirection();

        __CLOVER_203_0.S[12886]++;_clearValue = readCurrentPropertyValue();
    } finally { }}

    /** @since 2.2 */

    private Object readCurrentPropertyValue()
    {try { __CLOVER_203_0.M[3102]++;
        __CLOVER_203_0.S[12887]++;return PropertyUtils.read(_component, _propertyName);
    } finally { }}

    /**
     * Sets the property of the component to the specified value.
     */

    protected void setPropertyValue(Object value)
    {try { __CLOVER_203_0.M[3103]++;
        __CLOVER_203_0.S[12888]++;PropertyUtils.write(_component, _propertyName, value);
    } finally { }}

    /**
     * Gets the value of the binding.
     * 
     * @param requiredType
     *            if not null, the expected type of the value object.
     * @see IBinding#getObject()
     * @see IBinding#getObject(String, Class)
     */

    protected Object getBindingValue(Class requiredType)
    {try { __CLOVER_203_0.M[3104]++;
        __CLOVER_203_0.S[12889]++;Object result;

        __CLOVER_203_0.S[12890]++;if ((((requiredType == null) && (++__CLOVER_203_0.CT[2191] != 0)) || (++__CLOVER_203_0.CF[2191] == 0))){
            __CLOVER_203_0.S[12891]++;result = _binding.getObject();}
        else{
            __CLOVER_203_0.S[12892]++;result = _binding.getObject(_parameterName, requiredType);}

        __CLOVER_203_0.S[12893]++;return result;
    } finally { }}

    protected IBinding getBinding()
    {try { __CLOVER_203_0.M[3105]++;
        __CLOVER_203_0.S[12894]++;return _binding;
    } finally { }}

    public String toString()
    {try { __CLOVER_203_0.M[3106]++;
        __CLOVER_203_0.S[12895]++;StringBuffer buffer = new StringBuffer(super.toString());
        __CLOVER_203_0.S[12896]++;buffer.append('[');
        __CLOVER_203_0.S[12897]++;buffer.append(_component.getExtendedId());
        __CLOVER_203_0.S[12898]++;buffer.append(' ');
        __CLOVER_203_0.S[12899]++;buffer.append(_parameterName);
        __CLOVER_203_0.S[12900]++;buffer.append(' ');
        __CLOVER_203_0.S[12901]++;buffer.append(_binding);

        __CLOVER_203_0.S[12902]++;buffer.append(' ');
        __CLOVER_203_0.S[12903]++;buffer.append(_direction.getName());

        __CLOVER_203_0.S[12904]++;if ((((_required) && (++__CLOVER_203_0.CT[2192] != 0)) || (++__CLOVER_203_0.CF[2192] == 0))){
            __CLOVER_203_0.S[12905]++;buffer.append(" required");}

        __CLOVER_203_0.S[12906]++;buffer.append(']');

        __CLOVER_203_0.S[12907]++;return buffer.toString();
    } finally { }}

    /**
     * Restores the property to its default value. For {@link Direction#FORM}parameters, extracts
     * the property value and sets the binding form it (when appropriate).
     */

    public void resetParameter(IRequestCycle cycle)
    {try { __CLOVER_203_0.M[3107]++;
        __CLOVER_203_0.S[12908]++;if ((((_direction == Direction.FORM && cycle.isRewinding()) && (++__CLOVER_203_0.CT[2193] != 0)) || (++__CLOVER_203_0.CF[2193] == 0))){
        {
            __CLOVER_203_0.S[12909]++;IFormComponent component = (IFormComponent) _component;

            __CLOVER_203_0.S[12910]++;if ((((!component.isDisabled()) && (++__CLOVER_203_0.CT[2194] != 0)) || (++__CLOVER_203_0.CF[2194] == 0))){
            {
                __CLOVER_203_0.S[12911]++;IForm form = Form.get(cycle);

                __CLOVER_203_0.S[12912]++;if ((((form != null && form.isRewinding()) && (++__CLOVER_203_0.CT[2195] != 0)) || (++__CLOVER_203_0.CF[2195] == 0))){
                {
                    __CLOVER_203_0.S[12913]++;Object value = readCurrentPropertyValue();

                    __CLOVER_203_0.S[12914]++;_binding.setObject(value);
                }}
            }}
        }}

        // Either way, clear the value.

        __CLOVER_203_0.S[12915]++;setPropertyValue(_clearValue);
    } finally { }}

    /**
     * Returns true if the connector should update the property value from the binding. For
     * {@link org.apache.tapestry.spec.Direction#IN}, this always returns true. For
     * {@link org.apache.tapestry.spec.Direction#FORM}, this returns true only if the request cycle
     * and the active form are rewinding.
     * 
     * @since 2.2
     */

    protected boolean shouldSetPropertyValue(IRequestCycle cycle)
    {try { __CLOVER_203_0.M[3108]++;
        __CLOVER_203_0.S[12916]++;if ((((_direction == Direction.IN) && (++__CLOVER_203_0.CT[2196] != 0)) || (++__CLOVER_203_0.CF[2196] == 0))){
            __CLOVER_203_0.S[12917]++;return true;}

        // Must be FORM

        __CLOVER_203_0.S[12918]++;return !cycle.isRewinding();
    } finally { }}

}