/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.script;

import org.apache.hivemind.ClassResolver;
import org.apache.hivemind.Resource;
import org.apache.tapestry.IScript;
import org.apache.tapestry.services.ExpressionEvaluator;
import org.apache.tapestry.util.xml.DocumentParseException;
import org.apache.tapestry.util.xml.RuleDirectedParser;

/**
 * Parses a Tapestry Script, an XML file defined by one of the following public identifiers:
 * <ul>
 * <li><code>-//Primix Solutions//Tapestry Script 1.0//EN</code></li>
 * <li><code>-//Howard Ship//Tapestry Script 1.1//EN</code></li>
 * <li><code>-//Howard Lewis Ship//Tapestry Script 1.2//EN</code></li>.
 * <p>
 * The version 1.1, is largely backwards compatible to the old script, but adds a number of new
 * features (if, if-not, foreach and the use of property paths with insert).
 * <p>
 * Version 1.2 removes the &lt;insert&gt; element, using an Ant-like syntax (
 * <code>${<i>expression</i>}</code>). It also replaces the attribute name
 * <code>property-path</code> with <code>expression</code> (because OGNL is used).
 * <p>
 * A Tapestry Script is used, in association with the {@link org.apache.tapestry.html.Body}and/or
 * {@link org.apache.tapestry.html.Script}components, to generate JavaScript for use with a
 * Tapestry component. Two seperate pieces of JavaScript can be generated. The body section
 * (associated with the <code>body</code> element of the XML document) is typically used to define
 * JavaScript functions (most often, event handlers). The initialization section (associated with
 * the <code>initialization</code> element of the XML document) is used to add JavaScript that
 * will be evaluated when the page finishes loading (i.e., from the HTML &lt;body&gt; element's
 * onLoad event handler).
 * 
 * @author Howard Lewis Ship
 */

public class ScriptParser
{public static com.cortexeb.tools.clover.d __CLOVER_271_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1097439627296L);
    private RuleDirectedParser _parser;

    public static final String SCRIPT_DTD_1_0_PUBLIC_ID = "-//Primix Solutions//Tapestry Script 1.0//EN";

    public static final String SCRIPT_DTD_1_1_PUBLIC_ID = "-//Howard Ship//Tapestry Script 1.1//EN";

    public static final String SCRIPT_DTD_1_2_PUBLIC_ID = "-//Howard Lewis Ship//Tapestry Script 1.2//EN";

    /** @since 3.0 */
    public static final String SCRIPT_DTD_3_0_PUBLIC_ID = "-//Apache Software Foundation//Tapestry Script Specification 3.0//EN";

    public ScriptParser(ClassResolver resolver, ExpressionEvaluator evaluator)
    {try { __CLOVER_271_0.M[3258]++;
        __CLOVER_271_0.S[13893]++;_parser = new RuleDirectedParser();

        __CLOVER_271_0.S[13894]++;_parser.registerEntity(
                SCRIPT_DTD_1_0_PUBLIC_ID,
                "/org/apache/tapestry/script/Script_1_0.dtd");
        __CLOVER_271_0.S[13895]++;_parser.registerEntity(
                SCRIPT_DTD_1_1_PUBLIC_ID,
                "/org/apache/tapestry/script/Script_1_1.dtd");
        __CLOVER_271_0.S[13896]++;_parser.registerEntity(
                SCRIPT_DTD_1_2_PUBLIC_ID,
                "/org/apache/tapestry/script/Script_1_2.dtd");
        __CLOVER_271_0.S[13897]++;_parser.registerEntity(
                SCRIPT_DTD_3_0_PUBLIC_ID,
                "/org/apache/tapestry/script/Script_3_0.dtd");

        __CLOVER_271_0.S[13898]++;_parser.addRule("script", new ScriptRule(evaluator));
        __CLOVER_271_0.S[13899]++;_parser.addRule("let", new LetRule());
        __CLOVER_271_0.S[13900]++;_parser.addRule("set", new SetRule());
        __CLOVER_271_0.S[13901]++;_parser.addRule("include-script", new IncludeScriptRule());
        __CLOVER_271_0.S[13902]++;_parser.addRule("input-symbol", new InputSymbolRule(resolver));
        __CLOVER_271_0.S[13903]++;_parser.addRule("body", new BodyRule());
        __CLOVER_271_0.S[13904]++;_parser.addRule("initialization", new InitRule());
        __CLOVER_271_0.S[13905]++;_parser.addRule("if", new IfRule(true));
        __CLOVER_271_0.S[13906]++;_parser.addRule("if-not", new IfRule(false));
        __CLOVER_271_0.S[13907]++;_parser.addRule("foreach", new ForeachRule());
        __CLOVER_271_0.S[13908]++;_parser.addRule("unique", new UniqueRule());

        // This will go away when the 1.1 and earler DTDs are retired.
        __CLOVER_271_0.S[13909]++;_parser.addRule("insert", new InsertRule());

    } finally { }}

    /**
     * Parses the given input stream to produce a parsed script, ready to execute.
     */

    public IScript parse(Resource resourceLocation) throws DocumentParseException
    {try { __CLOVER_271_0.M[3259]++;
        __CLOVER_271_0.S[13910]++;return (IScript) _parser.parse(resourceLocation);
    } finally { }}

}