/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.spec;

import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.hivemind.ClassResolver;
import org.apache.tapestry.Tapestry;
import org.apache.tapestry.services.ExpressionEvaluator;

/**
 * Defines an "extension", which is much like a helper bean, but is part of a library or application
 * specification (and has the same lifecycle as the application).
 * 
 * @author Howard Lewis Ship
 * @since 2.2
 */

public class ExtensionSpecification extends LocatablePropertyHolder implements
        IExtensionSpecification
{public static com.cortexeb.tools.clover.d __CLOVER_349_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1097439627296L);
    private static final Log LOG = LogFactory.getLog(ExtensionSpecification.class);

    private String _className;

    protected Map _configuration = new HashMap();

    private boolean _immediate;

    /** @since 3.1 */
    private ExpressionEvaluator _evaluator;

    /** @since 3.1 */
    public ExtensionSpecification(ExpressionEvaluator evaluator)
    {try { __CLOVER_349_0.M[3388]++;
        __CLOVER_349_0.S[14234]++;_evaluator = evaluator;
    } finally { }}

    public String getClassName()
    {try { __CLOVER_349_0.M[3389]++;
        __CLOVER_349_0.S[14235]++;return _className;
    } finally { }}

    public void setClassName(String className)
    {try { __CLOVER_349_0.M[3390]++;
        __CLOVER_349_0.S[14236]++;_className = className;
    } finally { }}

    public void addConfiguration(String propertyName, String value)
    {try { __CLOVER_349_0.M[3391]++;
        __CLOVER_349_0.S[14237]++;if ((((_configuration.containsKey(propertyName)) && (++__CLOVER_349_0.CT[2369] != 0)) || (++__CLOVER_349_0.CF[2369] == 0))){
            __CLOVER_349_0.S[14238]++;throw new IllegalArgumentException(Tapestry.format(
                    "ExtensionSpecification.duplicate-property",
                    this,
                    propertyName));}

        __CLOVER_349_0.S[14239]++;_configuration.put(propertyName, value);
    } finally { }}

    /**
     * Returns an immutable Map of the configuration; keyed on property name, with values as
     * properties to assign.
     */

    public Map getConfiguration()
    {try { __CLOVER_349_0.M[3392]++;
        __CLOVER_349_0.S[14240]++;return Collections.unmodifiableMap(_configuration);
    } finally { }}

    /**
     * Invoked to instantiate an instance of the extension and return it. It also configures
     * properties of the extension.
     */

    public Object instantiateExtension(ClassResolver resolver)
    {try { __CLOVER_349_0.M[3393]++;
        __CLOVER_349_0.S[14241]++;if ((((LOG.isDebugEnabled()) && (++__CLOVER_349_0.CT[2370] != 0)) || (++__CLOVER_349_0.CF[2370] == 0))){
            __CLOVER_349_0.S[14242]++;LOG.debug("Instantiating extension class " + _className + ".");}
        __CLOVER_349_0.S[14243]++;Class extensionClass = null;
        __CLOVER_349_0.S[14244]++;Object result = null;

        __CLOVER_349_0.S[14245]++;try
        {
            __CLOVER_349_0.S[14246]++;extensionClass = resolver.findClass(_className);
        }
        catch (Exception ex)
        {
            __CLOVER_349_0.S[14247]++;throw new ApplicationRuntimeException(Tapestry.format(
                    "ExtensionSpecification.bad-class",
                    _className), getLocation(), ex);
        }

        __CLOVER_349_0.S[14248]++;result = instantiateInstance(extensionClass, result);

        __CLOVER_349_0.S[14249]++;initializeProperties(result);

        __CLOVER_349_0.S[14250]++;return result;
    } finally { }}

    private void initializeProperties(Object extension)
    {try { __CLOVER_349_0.M[3394]++;
        __CLOVER_349_0.S[14251]++;if ((((_configuration.isEmpty()) && (++__CLOVER_349_0.CT[2371] != 0)) || (++__CLOVER_349_0.CF[2371] == 0))){
            __CLOVER_349_0.S[14252]++;return;}

        __CLOVER_349_0.S[14253]++;Iterator i = _configuration.entrySet().iterator();
        __CLOVER_349_0.S[14254]++;while ((((i.hasNext()) && (++__CLOVER_349_0.CT[2372] != 0)) || (++__CLOVER_349_0.CF[2372] == 0))){
        {
            __CLOVER_349_0.S[14255]++;Map.Entry entry = (Map.Entry) i.next();

            __CLOVER_349_0.S[14256]++;String propertyName = (String) entry.getKey();
            __CLOVER_349_0.S[14257]++;String textValue = (String) entry.getValue();

            __CLOVER_349_0.S[14258]++;try
            {
                __CLOVER_349_0.S[14259]++;_evaluator.write(extension, propertyName, textValue);
            }
            catch (Exception ex)
            {
                __CLOVER_349_0.S[14260]++;throw new ApplicationRuntimeException(ex.getMessage(), getLocation(), ex);
            }
        }}
    } finally { }}

    private Object instantiateInstance(Class extensionClass, Object result)
    {try { __CLOVER_349_0.M[3395]++;
        __CLOVER_349_0.S[14261]++;try
        {
            __CLOVER_349_0.S[14262]++;result = extensionClass.newInstance();
        }
        catch (Exception ex)
        {
            __CLOVER_349_0.S[14263]++;throw new ApplicationRuntimeException(ex.getMessage(), getLocation(), ex);
        }

        __CLOVER_349_0.S[14264]++;return result;
    } finally { }}

    public String toString()
    {try { __CLOVER_349_0.M[3396]++;
        __CLOVER_349_0.S[14265]++;StringBuffer buffer = new StringBuffer("ExtensionSpecification@");
        __CLOVER_349_0.S[14266]++;buffer.append(Integer.toHexString(hashCode()));
        __CLOVER_349_0.S[14267]++;buffer.append('[');
        __CLOVER_349_0.S[14268]++;buffer.append(_className);

        __CLOVER_349_0.S[14269]++;if ((((_configuration != null) && (++__CLOVER_349_0.CT[2373] != 0)) || (++__CLOVER_349_0.CF[2373] == 0))){
        {
            __CLOVER_349_0.S[14270]++;buffer.append(' ');
            __CLOVER_349_0.S[14271]++;buffer.append(_configuration);
        }}

        __CLOVER_349_0.S[14272]++;buffer.append(']');

        __CLOVER_349_0.S[14273]++;return buffer.toString();
    } finally { }}

    /**
     * Returns true if the extensions should be instantiated immediately after the containing
     * {@link org.apache.tapestry.spec.LibrarySpecification}if parsed. Non-immediate extensions are
     * instantiated only as needed.
     */

    public boolean isImmediate()
    {try { __CLOVER_349_0.M[3397]++;
        __CLOVER_349_0.S[14274]++;return _immediate;
    } finally { }}

    public void setImmediate(boolean immediate)
    {try { __CLOVER_349_0.M[3398]++;
        __CLOVER_349_0.S[14275]++;_immediate = immediate;
    } finally { }}

}