/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry;

import org.apache.hivemind.ApplicationRuntimeException;

/**
 *  Exception thrown by an {@link org.apache.tapestry.engine.IEngineService} when it discovers that
 *  the an action link was for an out-of-date version of the page.
 *
 *  <p>The application should redirect to the StaleLink page.
 *
 *
 *  @author Howard Lewis Ship
 * 
 **/

public class StaleLinkException extends ApplicationRuntimeException
{public static com.cortexeb.tools.clover.d __CLOVER_31_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private transient IPage _page;
    private String _pageName;
    private String _targetIdPath;
    private String _targetActionId;

    public StaleLinkException()
    {
        super(null, null, null, null);__CLOVER_31_0.S[561]++;try { __CLOVER_31_0.M[162]++;
    } finally { }}

    /**
     *  Constructor used when the action id is found, but the target id path
     *  did not match the actual id path.
     *
     **/

    public StaleLinkException(IComponent component, String targetActionId, String targetIdPath)
    {
        super(
            Tapestry.format(
                "StaleLinkException.action-mismatch",
                new String[] { targetActionId, component.getIdPath(), targetIdPath }),
            component,
            null,
            null);__CLOVER_31_0.S[562]++;try { __CLOVER_31_0.M[163]++;

        __CLOVER_31_0.S[563]++;_page = component.getPage();
        __CLOVER_31_0.S[564]++;_pageName = _page.getPageName();

        __CLOVER_31_0.S[565]++;_targetActionId = targetActionId;
        __CLOVER_31_0.S[566]++;_targetIdPath = targetIdPath;
    } finally { }}

    /**
     *  Constructor used when the target action id is not found.
     *
     **/

    public StaleLinkException(IPage page, String targetActionId, String targetIdPath)
    {
        this(
            Tapestry.format(
                "StaleLinkException.component-mismatch",
                targetActionId,
                targetIdPath),
            page);__CLOVER_31_0.S[567]++;try { __CLOVER_31_0.M[164]++;

        __CLOVER_31_0.S[568]++;_targetActionId = targetActionId;
        __CLOVER_31_0.S[569]++;_targetIdPath = targetIdPath;
    } finally { }}

    public StaleLinkException(String message, IComponent component)
    {
        super(message, component, null, null);__CLOVER_31_0.S[570]++;try { __CLOVER_31_0.M[165]++;
    } finally { }}



    public String getPageName()
    {try { __CLOVER_31_0.M[166]++;
        __CLOVER_31_0.S[571]++;return _pageName;
    } finally { }}

    /**
     *  Returns the page referenced by the service URL, if known, 
     *  or null otherwise.
     *
     **/

    public IPage getPage()
    {try { __CLOVER_31_0.M[167]++;
        __CLOVER_31_0.S[572]++;return _page;
    } finally { }}
    
    public String getTargetActionId()
    {try { __CLOVER_31_0.M[168]++;
        __CLOVER_31_0.S[573]++;return _targetActionId;
    } finally { }}

    public String getTargetIdPath()
    {try { __CLOVER_31_0.M[169]++;
        __CLOVER_31_0.S[574]++;return _targetIdPath;
    } finally { }}

}