/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.bean;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.hivemind.ClassResolver;
import org.apache.tapestry.IBeanProvider;
import org.apache.tapestry.IComponent;
import org.apache.tapestry.IEngine;
import org.apache.tapestry.Tapestry;
import org.apache.tapestry.event.PageDetachListener;
import org.apache.tapestry.event.PageEvent;
import org.apache.tapestry.event.PageRenderListener;
import org.apache.tapestry.spec.BeanLifecycle;
import org.apache.tapestry.spec.IBeanSpecification;
import org.apache.tapestry.spec.IComponentSpecification;

/**
 *  Basic implementation of the {@link IBeanProvider} interface.
 *
 *  @author Howard Lewis Ship
 *  @since 1.0.4
 **/

public class BeanProvider implements IBeanProvider, PageDetachListener, PageRenderListener
{public static com.cortexeb.tools.clover.d __CLOVER_42_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private static final Log LOG = LogFactory.getLog(BeanProvider.class);

    /**
     *  Indicates whether this instance has been registered with its
     *  page as a PageDetachListener.  Registration only occurs
     *  the first time a bean with lifecycle REQUEST is instantiated.
     *
     **/

    private boolean _registeredForDetach = false;

    /**
     *  Indicates whether this instance has been registered as a render
     *  listener with the page.
     * 
     **/

    private boolean _registeredForRender = false;

    /**
     *  The component for which beans are being created and tracked.
     *
     **/

    private IComponent _component;

    /**
     *  Used for instantiating classes.
     *
     **/

    private ClassResolver _resolver;

    /**
     *  Map of beans, keyed on name.
     *
     **/

    private Map _beans;

    /**
     *  Set of bean names provided by this provider.
     * 
     *  @since 2.2
     * 
     **/

    private Set _beanNames;

    public BeanProvider(IComponent component)
    {try { __CLOVER_42_0.M[258]++;
        __CLOVER_42_0.S[1012]++;this._component = component;
        __CLOVER_42_0.S[1013]++;IEngine engine = component.getPage().getEngine();
        __CLOVER_42_0.S[1014]++;_resolver = engine.getClassResolver();

        __CLOVER_42_0.S[1015]++;if ((((LOG.isDebugEnabled()) && (++__CLOVER_42_0.CT[194] != 0)) || (++__CLOVER_42_0.CF[194] == 0))){
            __CLOVER_42_0.S[1016]++;LOG.debug("Created BeanProvider for " + component);}

    } finally { }}

    /** @since 1.0.6 **/

    public Collection getBeanNames()
    {try { __CLOVER_42_0.M[259]++;
        __CLOVER_42_0.S[1017]++;if ((((_beanNames == null) && (++__CLOVER_42_0.CT[195] != 0)) || (++__CLOVER_42_0.CF[195] == 0))){
        {
            __CLOVER_42_0.S[1018]++;Collection c = _component.getSpecification().getBeanNames();

            __CLOVER_42_0.S[1019]++;if ((((c == null || c.isEmpty()) && (++__CLOVER_42_0.CT[196] != 0)) || (++__CLOVER_42_0.CF[196] == 0))){
                __CLOVER_42_0.S[1020]++;_beanNames = Collections.EMPTY_SET;}
            else{
                __CLOVER_42_0.S[1021]++;_beanNames = Collections.unmodifiableSet(new HashSet(c));}
        }}

        __CLOVER_42_0.S[1022]++;return _beanNames;
    } finally { }}

    /**
     *  @since 1.0.5
     *
     **/

    public IComponent getComponent()
    {try { __CLOVER_42_0.M[260]++;
        __CLOVER_42_0.S[1023]++;return _component;
    } finally { }}

    public Object getBean(String name)
    {try { __CLOVER_42_0.M[261]++;
        __CLOVER_42_0.S[1024]++;Object bean = null;

        __CLOVER_42_0.S[1025]++;if ((((_beans != null) && (++__CLOVER_42_0.CT[197] != 0)) || (++__CLOVER_42_0.CF[197] == 0))){
            __CLOVER_42_0.S[1026]++;bean = _beans.get(name);}

        __CLOVER_42_0.S[1027]++;if ((((bean != null) && (++__CLOVER_42_0.CT[198] != 0)) || (++__CLOVER_42_0.CF[198] == 0))){
            __CLOVER_42_0.S[1028]++;return bean;}

        __CLOVER_42_0.S[1029]++;IBeanSpecification spec = _component.getSpecification().getBeanSpecification(name);

        __CLOVER_42_0.S[1030]++;if ((((spec == null) && (++__CLOVER_42_0.CT[199] != 0)) || (++__CLOVER_42_0.CF[199] == 0))){
            __CLOVER_42_0.S[1031]++;throw new ApplicationRuntimeException(
                Tapestry.format(
                    "BeanProvider.bean-not-defined",
                    _component.getExtendedId(),
                    name));}

        __CLOVER_42_0.S[1032]++;bean = instantiateBean(name, spec);

        __CLOVER_42_0.S[1033]++;BeanLifecycle lifecycle = spec.getLifecycle();

        __CLOVER_42_0.S[1034]++;if ((((lifecycle == BeanLifecycle.NONE) && (++__CLOVER_42_0.CT[200] != 0)) || (++__CLOVER_42_0.CF[200] == 0))){
            __CLOVER_42_0.S[1035]++;return bean;}

        __CLOVER_42_0.S[1036]++;if ((((_beans == null) && (++__CLOVER_42_0.CT[201] != 0)) || (++__CLOVER_42_0.CF[201] == 0))){
            __CLOVER_42_0.S[1037]++;_beans = new HashMap();}

        __CLOVER_42_0.S[1038]++;_beans.put(name, bean);

        // The first time in a request that a REQUEST lifecycle bean is created,
        // register with the page to be notified at the end of the
        // request cycle.

        __CLOVER_42_0.S[1039]++;if ((((lifecycle == BeanLifecycle.REQUEST && !_registeredForDetach) && (++__CLOVER_42_0.CT[202] != 0)) || (++__CLOVER_42_0.CF[202] == 0))){
        {
            __CLOVER_42_0.S[1040]++;_component.getPage().addPageDetachListener(this);
            __CLOVER_42_0.S[1041]++;_registeredForDetach = true;
        }}

        __CLOVER_42_0.S[1042]++;if ((((lifecycle == BeanLifecycle.RENDER && !_registeredForRender) && (++__CLOVER_42_0.CT[203] != 0)) || (++__CLOVER_42_0.CF[203] == 0))){
        {
            __CLOVER_42_0.S[1043]++;_component.getPage().addPageRenderListener(this);
            __CLOVER_42_0.S[1044]++;_registeredForRender = true;
        }}

        // No need to register if a PAGE lifecycle bean; those can stick around
        // forever.

        __CLOVER_42_0.S[1045]++;return bean;
    } finally { }}

    private Object instantiateBean(String beanName, IBeanSpecification spec)
    {try { __CLOVER_42_0.M[262]++;
        __CLOVER_42_0.S[1046]++;String className = spec.getClassName();
        __CLOVER_42_0.S[1047]++;Object bean = null;

        __CLOVER_42_0.S[1048]++;if ((((LOG.isDebugEnabled()) && (++__CLOVER_42_0.CT[204] != 0)) || (++__CLOVER_42_0.CF[204] == 0))){
            __CLOVER_42_0.S[1049]++;LOG.debug("Instantiating instance of " + className);}

        // Do it the hard way!

        __CLOVER_42_0.S[1050]++;try
        {
            __CLOVER_42_0.S[1051]++;Class beanClass = _resolver.findClass(className);

            __CLOVER_42_0.S[1052]++;bean = beanClass.newInstance();
        }
        catch (Exception ex)
        {

            __CLOVER_42_0.S[1053]++;throw new ApplicationRuntimeException(
                Tapestry.format(
                    "BeanProvider.instantiation-error",
                    new Object[] {
                        beanName,
                        _component.getExtendedId(),
                        className,
                        ex.getMessage()}),
                spec.getLocation(),
                ex);
        }

        // OK, have the bean, have to initialize it.

        __CLOVER_42_0.S[1054]++;List initializers = spec.getInitializers();

        __CLOVER_42_0.S[1055]++;if ((((initializers == null) && (++__CLOVER_42_0.CT[205] != 0)) || (++__CLOVER_42_0.CF[205] == 0))){
            __CLOVER_42_0.S[1056]++;return bean;}

        __CLOVER_42_0.S[1057]++;Iterator i = initializers.iterator();
        __CLOVER_42_0.S[1058]++;while ((((i.hasNext()) && (++__CLOVER_42_0.CT[206] != 0)) || (++__CLOVER_42_0.CF[206] == 0))){
        {
            __CLOVER_42_0.S[1059]++;IBeanInitializer iz = (IBeanInitializer) i.next();

            __CLOVER_42_0.S[1060]++;if ((((LOG.isDebugEnabled()) && (++__CLOVER_42_0.CT[207] != 0)) || (++__CLOVER_42_0.CF[207] == 0))){
                __CLOVER_42_0.S[1061]++;LOG.debug("Initializing property " + iz.getPropertyName());}

            __CLOVER_42_0.S[1062]++;iz.setBeanProperty(this, bean);
        }}

        __CLOVER_42_0.S[1063]++;return bean;
    } finally { }}

    /**
     *  Removes all beans with the REQUEST lifecycle.  Beans with
     *  the PAGE lifecycle stick around, and beans with no lifecycle
     *  were never stored in the first place.
     *
     **/

    public void pageDetached(PageEvent event)
    {try { __CLOVER_42_0.M[263]++;
        __CLOVER_42_0.S[1064]++;removeBeans(BeanLifecycle.REQUEST);
    } finally { }}

    /**
     *  Removes any beans with the specified lifecycle.
     * 
     *  @since 2.2
     * 
     **/

    private void removeBeans(BeanLifecycle lifecycle)
    {try { __CLOVER_42_0.M[264]++;
        __CLOVER_42_0.S[1065]++;if ((((_beans == null) && (++__CLOVER_42_0.CT[208] != 0)) || (++__CLOVER_42_0.CF[208] == 0))){
            __CLOVER_42_0.S[1066]++;return;}

        __CLOVER_42_0.S[1067]++;IComponentSpecification spec = null;

        __CLOVER_42_0.S[1068]++;Iterator i = _beans.entrySet().iterator();
        __CLOVER_42_0.S[1069]++;while ((((i.hasNext()) && (++__CLOVER_42_0.CT[209] != 0)) || (++__CLOVER_42_0.CF[209] == 0))){
        {
            __CLOVER_42_0.S[1070]++;Map.Entry e = (Map.Entry) i.next();
            __CLOVER_42_0.S[1071]++;String name = (String) e.getKey();

            __CLOVER_42_0.S[1072]++;if ((((spec == null) && (++__CLOVER_42_0.CT[210] != 0)) || (++__CLOVER_42_0.CF[210] == 0))){
                __CLOVER_42_0.S[1073]++;spec = _component.getSpecification();}

            __CLOVER_42_0.S[1074]++;IBeanSpecification s = spec.getBeanSpecification(name);

            __CLOVER_42_0.S[1075]++;if ((((s.getLifecycle() == lifecycle) && (++__CLOVER_42_0.CT[211] != 0)) || (++__CLOVER_42_0.CF[211] == 0))){
            {
                __CLOVER_42_0.S[1076]++;Object bean = e.getValue();

                __CLOVER_42_0.S[1077]++;if ((((LOG.isDebugEnabled()) && (++__CLOVER_42_0.CT[212] != 0)) || (++__CLOVER_42_0.CF[212] == 0))){
                    __CLOVER_42_0.S[1078]++;LOG.debug("Removing " + lifecycle.getName() + " bean " + name + ": " + bean);}

                __CLOVER_42_0.S[1079]++;i.remove();
            }}
        }}
    } finally { }}

    /** @since 1.0.8 **/

    public ClassResolver getClassResolver()
    {try { __CLOVER_42_0.M[265]++;
        __CLOVER_42_0.S[1080]++;return _resolver;
    } finally { }}

    /** @since 2.2 **/

    public void pageBeginRender(PageEvent event)
    {try { __CLOVER_42_0.M[266]++;
    } finally { }}

    /** @since 2.2 **/

    public void pageEndRender(PageEvent event)
    {try { __CLOVER_42_0.M[267]++;
        __CLOVER_42_0.S[1081]++;removeBeans(BeanLifecycle.RENDER);
    } finally { }}

    /** @since 2.2 **/

    public boolean canProvideBean(String name)
    {try { __CLOVER_42_0.M[268]++;
        __CLOVER_42_0.S[1082]++;return getBeanNames().contains(name);
    } finally { }}

}