/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.components;

import java.util.Iterator;

import org.apache.tapestry.AbstractComponent;
import org.apache.tapestry.IBinding;
import org.apache.tapestry.IMarkupWriter;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.Tapestry;

/**
 *  Repeatedly renders its wrapped contents while iterating through
 *  a list of values.
 *
 *  [<a href="../../../../../ComponentReference/Foreach.html">Component Reference</a>]
 *
 *  <p>
 *  While the component is rendering, the property
 *  {@link #getValue() value} (accessed as
 *  <code>components.<i>foreach</i>.value</code>
 *  is set to each successive value from the source,
 *  and the property
 *  {@link #getIndex() index} is set to each successive index
 *  into the source (starting with zero).
 * 
 *  @author Howard Lewis Ship
 * 
 **/

public abstract class Foreach extends AbstractComponent
{public static com.cortexeb.tools.clover.d __CLOVER_65_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private Object _value;
    private int _index;
    private boolean _rendering;

    public abstract IBinding getIndexBinding();


    /**
     *  Gets the source binding and returns an {@link Iterator}
     *  representing
     *  the values identified by the source.  Returns an empty {@link Iterator}
     *  if the binding, or the binding value, is null.
     *
     *  <p>Invokes {@link Tapestry#coerceToIterator(Object)} to perform
     *  the actual conversion.
     *
     **/

    protected Iterator getSourceData()
    {try { __CLOVER_65_0.M[364]++;
    	__CLOVER_65_0.S[1470]++;Object source = getSource();
    	
 		__CLOVER_65_0.S[1471]++;if ((((source == null) && (++__CLOVER_65_0.CT[281] != 0)) || (++__CLOVER_65_0.CF[281] == 0))){
 			__CLOVER_65_0.S[1472]++;return null;}
 		
        __CLOVER_65_0.S[1473]++;return Tapestry.coerceToIterator(source);
    } finally { }}

    public abstract IBinding getValueBinding();

    /**
     *  Gets the source binding and iterates through
     *  its values.  For each, it updates the value binding and render's its wrapped elements.
     *
     **/

    protected void renderComponent(IMarkupWriter writer, IRequestCycle cycle)
    {try { __CLOVER_65_0.M[365]++;
        __CLOVER_65_0.S[1474]++;Iterator dataSource = getSourceData();

        // The dataSource was either not convertable, or was empty.

        __CLOVER_65_0.S[1475]++;if ((((dataSource == null) && (++__CLOVER_65_0.CT[282] != 0)) || (++__CLOVER_65_0.CF[282] == 0))){
            __CLOVER_65_0.S[1476]++;return;}


        __CLOVER_65_0.S[1477]++;try
        {
            __CLOVER_65_0.S[1478]++;_rendering = true;
            __CLOVER_65_0.S[1479]++;_value = null;
            __CLOVER_65_0.S[1480]++;_index = 0;
            
            __CLOVER_65_0.S[1481]++;IBinding indexBinding = getIndexBinding();
            __CLOVER_65_0.S[1482]++;IBinding valueBinding = getValueBinding();
            __CLOVER_65_0.S[1483]++;String element = getElement();

            __CLOVER_65_0.S[1484]++;boolean hasNext = dataSource.hasNext();

            __CLOVER_65_0.S[1485]++;while ((((hasNext) && (++__CLOVER_65_0.CT[283] != 0)) || (++__CLOVER_65_0.CF[283] == 0))){
            {
                __CLOVER_65_0.S[1486]++;_value = dataSource.next();
                __CLOVER_65_0.S[1487]++;hasNext = dataSource.hasNext();

                __CLOVER_65_0.S[1488]++;if ((((indexBinding != null) && (++__CLOVER_65_0.CT[284] != 0)) || (++__CLOVER_65_0.CF[284] == 0))){
                    __CLOVER_65_0.S[1489]++;indexBinding.setInt(_index);}

                __CLOVER_65_0.S[1490]++;if ((((valueBinding != null) && (++__CLOVER_65_0.CT[285] != 0)) || (++__CLOVER_65_0.CF[285] == 0))){
                    __CLOVER_65_0.S[1491]++;valueBinding.setObject(_value);}

                __CLOVER_65_0.S[1492]++;if ((((element != null) && (++__CLOVER_65_0.CT[286] != 0)) || (++__CLOVER_65_0.CF[286] == 0))){
                {
                    __CLOVER_65_0.S[1493]++;writer.begin(element);
                    __CLOVER_65_0.S[1494]++;renderInformalParameters(writer, cycle);
                }}

                __CLOVER_65_0.S[1495]++;renderBody(writer, cycle);

                __CLOVER_65_0.S[1496]++;if ((((element != null) && (++__CLOVER_65_0.CT[287] != 0)) || (++__CLOVER_65_0.CF[287] == 0))){
                    __CLOVER_65_0.S[1497]++;writer.end();}

                __CLOVER_65_0.S[1498]++;_index++;
            }}
        }
        finally
        {
            __CLOVER_65_0.S[1499]++;_value = null;
            __CLOVER_65_0.S[1500]++;_rendering = false;
        }
    } finally { }}

    /**
     *  Returns the most recent value extracted from the source parameter.
     *
     *  @throws org.apache.tapestry.ApplicationRuntimeException if the Foreach is not currently rendering.
     *
     **/

    public Object getValue()
    {try { __CLOVER_65_0.M[366]++;
        __CLOVER_65_0.S[1501]++;if ((((!_rendering) && (++__CLOVER_65_0.CT[288] != 0)) || (++__CLOVER_65_0.CF[288] == 0))){
            __CLOVER_65_0.S[1502]++;throw Tapestry.createRenderOnlyPropertyException(this, "value");}
  
        __CLOVER_65_0.S[1503]++;return _value;
    } finally { }}

    public abstract String getElement();

    public abstract Object getSource();

    /**
     *  The index number, within the {@link #getSource() source}, of the
     *  the current value.
     * 
     *  @throws org.apache.tapestry.ApplicationRuntimeException if the Foreach is not currently rendering.
     *
     *  @since 2.2
     * 
     **/
    
    public int getIndex()
    {try { __CLOVER_65_0.M[367]++;
        __CLOVER_65_0.S[1504]++;if ((((!_rendering) && (++__CLOVER_65_0.CT[289] != 0)) || (++__CLOVER_65_0.CF[289] == 0))){
            __CLOVER_65_0.S[1505]++;throw Tapestry.createRenderOnlyPropertyException(this, "index");}
        
        __CLOVER_65_0.S[1506]++;return _index;
    } finally { }}

}