/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.engine;

import java.io.IOException;

import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.Tapestry;
import org.apache.tapestry.request.RequestContext;
import org.apache.tapestry.services.DataSqueezer;
import org.apache.tapestry.util.StringSplitter;

/**
 *  Abstract base class for implementing engine services.  Instances of services
 *  are shared by many engines and threads, so they must be threadsafe.
 * 
 *  <p>
 *  Note; too much of the URL encoding/decoding stategy is fixed here.
 *  A future release of Tapestry may extract this strategy, allowing developers
 *  to choose the method via which URLs are encoded.
 * 
 *  @see org.apache.tapestry.engine.AbstractEngine#getService(String)
 * 
 *
 *  @author Howard Lewis Ship
 *  @since 1.0.3
 * 
 **/

public abstract class AbstractService implements IEngineService
{public static com.cortexeb.tools.clover.d __CLOVER_72_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    /**
     *  Constructs a link for the service.
     *
     *  @param cycle the request cycle
     *  @param serviceName the name of the service
     *  @param serviceContext context related to the service itself which is added to the URL as-is
     *  @param parameters additional service parameters provided by the component; 
     *  this is application specific information, and is encoded with 
     *  {@link java.net.URLEncoder#encode(String)} before being added
     *  to the query.
     *  @param stateful if true, the final URL must be encoded with the HttpSession id
     *
     **/

    protected ILink constructLink(
        IRequestCycle cycle,
        String serviceName,
        String[] serviceContext,
        Object[] parameters,
        boolean stateful)
    {try { __CLOVER_72_0.M[428]++;
        __CLOVER_72_0.S[1800]++;DataSqueezer squeezer = cycle.getEngine().getDataSqueezer();
        __CLOVER_72_0.S[1801]++;String[] squeezed = null;

        __CLOVER_72_0.S[1802]++;try
        {
            __CLOVER_72_0.S[1803]++;squeezed = squeezer.squeeze(parameters);
        }
        catch (IOException ex)
        {
            __CLOVER_72_0.S[1804]++;throw new ApplicationRuntimeException(ex);
        }

        __CLOVER_72_0.S[1805]++;return new EngineServiceLink(cycle, serviceName, serviceContext, squeezed, stateful);
    } finally { }}

    /**
     *  Returns the service context as an array of Strings.
     *  Returns null if there are no service context strings.
     *
     **/

    protected String[] getServiceContext(RequestContext context)
    {try { __CLOVER_72_0.M[429]++;
        __CLOVER_72_0.S[1806]++;String service = context.getParameter(Tapestry.SERVICE_QUERY_PARAMETER_NAME);

		__CLOVER_72_0.S[1807]++;int slashx = service.indexOf('/');
		
		__CLOVER_72_0.S[1808]++;if ((((slashx < 0) && (++__CLOVER_72_0.CT[337] != 0)) || (++__CLOVER_72_0.CF[337] == 0))){
			__CLOVER_72_0.S[1809]++;return null;}
			
		__CLOVER_72_0.S[1810]++;String serviceContext = service.substring(slashx + 1);

        __CLOVER_72_0.S[1811]++;return new StringSplitter('/').splitToArray(serviceContext);
    } finally { }}

    /**
     *  Returns the service parameters as an array of Strings.
     *
     **/

    protected Object[] getParameters(IRequestCycle cycle)
    {try { __CLOVER_72_0.M[430]++;
        __CLOVER_72_0.S[1812]++;RequestContext context = cycle.getRequestContext();

        __CLOVER_72_0.S[1813]++;String[] squeezed = context.getParameters(Tapestry.PARAMETERS_QUERY_PARAMETER_NAME);

        __CLOVER_72_0.S[1814]++;if ((((Tapestry.size(squeezed) == 0) && (++__CLOVER_72_0.CT[338] != 0)) || (++__CLOVER_72_0.CF[338] == 0))){
            __CLOVER_72_0.S[1815]++;return squeezed;}

        __CLOVER_72_0.S[1816]++;try
        {
            __CLOVER_72_0.S[1817]++;DataSqueezer squeezer = cycle.getEngine().getDataSqueezer();

            __CLOVER_72_0.S[1818]++;return squeezer.unsqueeze(squeezed);
        }
        catch (IOException ex)
        {
            __CLOVER_72_0.S[1819]++;throw new ApplicationRuntimeException(ex);
        }
    } finally { }}
}