/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.engine;

import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.codec.net.URLCodec;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.Tapestry;
import org.apache.tapestry.request.RequestContext;

/**
 *  A EngineServiceLink represents a possible action within the client web browser;
 *  either clicking a link or submitting a form, which is constructed primarily
 *  from the {@link org.apache.tapestry.IEngine#getServletPath() servlet path},
 *  with some additional query parameters.  A full URL for the EngineServiceLink
 *  can be generated, or the query parameters for the EngineServiceLink can be extracted
 *  (separately from the servlet path).  The latter case is used when submitting
 *  constructing {@link org.apache.tapestry.form.Form forms}.
 *
 *  @author Howard Lewis Ship
 *  @since 3.0
 * 
 **/

public class EngineServiceLink implements ILink
{public static com.cortexeb.tools.clover.d __CLOVER_81_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private static final int DEFAULT_HTTP_PORT = 80;
    private static final URLCodec _urlCodec = new URLCodec();

    private IRequestCycle _cycle;
    private String _service;
    private String[] _parameters;
    private boolean _stateful;

    /**
     *  Creates a new EngineServiceLink.  A EngineServiceLink always names a service to be activated
     *  by the link, has an optional list of service context strings,
     *  an optional list of service parameter strings and may be stateful
     *  or stateless.
     * 
     *  <p>ServiceLink parameter strings may contain any characters.
     * 
     *  <p>ServiceLink context strings must be URL safe, and may not contain
     *  slash ('/') characters.  Typically, only letters, numbers and simple
     *  punctuation ('.', '-', '_', ':') is recommended (no checks are currently made,
     *  however).  Context strings are generally built from page names
     *  and component ids, which are limited to safe characters.
     *  
     *  @param cycle The {@link IRequestCycle} the EngineServiceLink is to be created for.
     *  @param serviceName The name of the service to be invoked by the EngineServiceLink.
     *  @param serviceContext an optional array of strings to be provided
     *  to the service to provide a context for executing the service.  May be null
     *  or empty.  <b>Note: copied, not retained.</b>
     *  @param serviceParameters An array of parameters, may be 
     *  null or empty. <b>Note: retained, not copied.</b>
     *  @param stateful if true, the service which generated the EngineServiceLink
     *  is stateful and expects that the final URL will be passed through
     *  {@link IRequestCycle#encodeURL(String)}.
     **/

    public EngineServiceLink(
        IRequestCycle cycle,
        String serviceName,
        String[] serviceContext,
        String[] serviceParameters,
        boolean stateful)
    {try { __CLOVER_81_0.M[463]++;
        __CLOVER_81_0.S[2046]++;_cycle = cycle;
        __CLOVER_81_0.S[2047]++;_service = constructServiceValue(serviceName, serviceContext);
        __CLOVER_81_0.S[2048]++;_parameters = serviceParameters;
        __CLOVER_81_0.S[2049]++;_stateful = stateful;
    } finally { }}

    private String constructServiceValue(String serviceName, String[] serviceContext)
    {try { __CLOVER_81_0.M[464]++;
        __CLOVER_81_0.S[2050]++;int count = Tapestry.size(serviceContext);

        __CLOVER_81_0.S[2051]++;if ((((count == 0) && (++__CLOVER_81_0.CT[384] != 0)) || (++__CLOVER_81_0.CF[384] == 0))){
            __CLOVER_81_0.S[2052]++;return serviceName;}

        __CLOVER_81_0.S[2053]++;StringBuffer buffer = new StringBuffer(serviceName);

        __CLOVER_81_0.S[2054]++;for (int i = 0; (((i < count) && (++__CLOVER_81_0.CT[385] != 0)) || (++__CLOVER_81_0.CF[385] == 0)); i++){
        {
            __CLOVER_81_0.S[2055]++;buffer.append('/');

            __CLOVER_81_0.S[2056]++;buffer.append(serviceContext[i]);
        }}

        __CLOVER_81_0.S[2057]++;return buffer.toString();
    } finally { }}

    public String getURL()
    {try { __CLOVER_81_0.M[465]++;
        __CLOVER_81_0.S[2058]++;return getURL(null, true);
    } finally { }}

    public String getURL(String anchor, boolean includeParameters)
    {try { __CLOVER_81_0.M[466]++;
        __CLOVER_81_0.S[2059]++;return constructURL(new StringBuffer(), anchor, includeParameters);
    } finally { }}

    public String getAbsoluteURL()
    {try { __CLOVER_81_0.M[467]++;
        __CLOVER_81_0.S[2060]++;return getAbsoluteURL(null, null, 0, null, true);
    } finally { }}

    public String getAbsoluteURL(
        String scheme,
        String server,
        int port,
        String anchor,
        boolean includeParameters)
    {try { __CLOVER_81_0.M[468]++;
        __CLOVER_81_0.S[2061]++;StringBuffer buffer = new StringBuffer();
        __CLOVER_81_0.S[2062]++;RequestContext context = _cycle.getRequestContext();

        __CLOVER_81_0.S[2063]++;if ((((scheme == null) && (++__CLOVER_81_0.CT[386] != 0)) || (++__CLOVER_81_0.CF[386] == 0))){
            __CLOVER_81_0.S[2064]++;scheme = context.getScheme();}

        __CLOVER_81_0.S[2065]++;buffer.append(scheme);
        __CLOVER_81_0.S[2066]++;buffer.append("://");

        __CLOVER_81_0.S[2067]++;if ((((server == null) && (++__CLOVER_81_0.CT[387] != 0)) || (++__CLOVER_81_0.CF[387] == 0))){
            __CLOVER_81_0.S[2068]++;server = context.getServerName();}

        __CLOVER_81_0.S[2069]++;buffer.append(server);

        __CLOVER_81_0.S[2070]++;if ((((port == 0) && (++__CLOVER_81_0.CT[388] != 0)) || (++__CLOVER_81_0.CF[388] == 0))){
            __CLOVER_81_0.S[2071]++;port = context.getServerPort();}

        __CLOVER_81_0.S[2072]++;if ((((!(scheme.equals("http") && port == DEFAULT_HTTP_PORT)) && (++__CLOVER_81_0.CT[389] != 0)) || (++__CLOVER_81_0.CF[389] == 0))){
        {
            __CLOVER_81_0.S[2073]++;buffer.append(':');
            __CLOVER_81_0.S[2074]++;buffer.append(port);
        }}

        // Add the servlet path and the rest of the URL & query parameters.
        // The servlet path starts with a leading slash.

        __CLOVER_81_0.S[2075]++;return constructURL(buffer, anchor, includeParameters);
    } finally { }}

    private String constructURL(StringBuffer buffer, String anchor, boolean includeParameters)
    {try { __CLOVER_81_0.M[469]++;
        __CLOVER_81_0.S[2076]++;buffer.append(_cycle.getEngine().getServletPath());

        __CLOVER_81_0.S[2077]++;if ((((includeParameters) && (++__CLOVER_81_0.CT[390] != 0)) || (++__CLOVER_81_0.CF[390] == 0))){
        {
            __CLOVER_81_0.S[2078]++;buffer.append('?');
            __CLOVER_81_0.S[2079]++;buffer.append(Tapestry.SERVICE_QUERY_PARAMETER_NAME);
            __CLOVER_81_0.S[2080]++;buffer.append('=');
            __CLOVER_81_0.S[2081]++;buffer.append(_service);

            __CLOVER_81_0.S[2082]++;int count = Tapestry.size(_parameters);

            __CLOVER_81_0.S[2083]++;for (int i = 0; (((i < count) && (++__CLOVER_81_0.CT[391] != 0)) || (++__CLOVER_81_0.CF[391] == 0)); i++){
            {
                __CLOVER_81_0.S[2084]++;buffer.append('&');

                __CLOVER_81_0.S[2085]++;buffer.append(Tapestry.PARAMETERS_QUERY_PARAMETER_NAME);
                __CLOVER_81_0.S[2086]++;buffer.append('=');

                __CLOVER_81_0.S[2087]++;String encoding = _cycle.getEngine().getOutputEncoding();
                __CLOVER_81_0.S[2088]++;try
                {
                    __CLOVER_81_0.S[2089]++;String encoded = _urlCodec.encode(_parameters[i], encoding);
                    __CLOVER_81_0.S[2090]++;buffer.append(encoded);
                }
                catch (UnsupportedEncodingException e)
                {
                    __CLOVER_81_0.S[2091]++;throw new ApplicationRuntimeException(
                        Tapestry.format("illegal-encoding", encoding),
                        e);
                }
            }}
        }}

        __CLOVER_81_0.S[2092]++;if ((((anchor != null) && (++__CLOVER_81_0.CT[392] != 0)) || (++__CLOVER_81_0.CF[392] == 0))){
        {
            __CLOVER_81_0.S[2093]++;buffer.append('#');
            __CLOVER_81_0.S[2094]++;buffer.append(anchor);
        }}

        __CLOVER_81_0.S[2095]++;String result = buffer.toString();

        __CLOVER_81_0.S[2096]++;if ((((_stateful) && (++__CLOVER_81_0.CT[393] != 0)) || (++__CLOVER_81_0.CF[393] == 0))){
            __CLOVER_81_0.S[2097]++;result = _cycle.encodeURL(result);}

        __CLOVER_81_0.S[2098]++;return result;
    } finally { }}

    public String[] getParameterNames()
    {try { __CLOVER_81_0.M[470]++;
        __CLOVER_81_0.S[2099]++;List list = new ArrayList();

        __CLOVER_81_0.S[2100]++;list.add(Tapestry.SERVICE_QUERY_PARAMETER_NAME);

        __CLOVER_81_0.S[2101]++;if ((((Tapestry.size(_parameters) != 0) && (++__CLOVER_81_0.CT[394] != 0)) || (++__CLOVER_81_0.CF[394] == 0))){
            __CLOVER_81_0.S[2102]++;list.add(Tapestry.PARAMETERS_QUERY_PARAMETER_NAME);}

        __CLOVER_81_0.S[2103]++;return (String[]) list.toArray(new String[list.size()]);
    } finally { }}

    public String[] getParameterValues(String name)
    {try { __CLOVER_81_0.M[471]++;
        __CLOVER_81_0.S[2104]++;if ((((name.equals(Tapestry.SERVICE_QUERY_PARAMETER_NAME)) && (++__CLOVER_81_0.CT[395] != 0)) || (++__CLOVER_81_0.CF[395] == 0))){
        {
            __CLOVER_81_0.S[2105]++;return new String[] { _service };
        }}

        __CLOVER_81_0.S[2106]++;if ((((name.equals(Tapestry.PARAMETERS_QUERY_PARAMETER_NAME)) && (++__CLOVER_81_0.CT[396] != 0)) || (++__CLOVER_81_0.CF[396] == 0))){
        {
            __CLOVER_81_0.S[2107]++;return _parameters;
        }}

        __CLOVER_81_0.S[2108]++;throw new IllegalArgumentException(
            Tapestry.format("EngineServiceLink.unknown-parameter-name", name));
    } finally { }}

    public String toString()
    {try { __CLOVER_81_0.M[472]++;
        __CLOVER_81_0.S[2109]++;ToStringBuilder builder = new ToStringBuilder(this);

        __CLOVER_81_0.S[2110]++;builder.append("service", _service);
        __CLOVER_81_0.S[2111]++;builder.append("parameters", _parameters);
        __CLOVER_81_0.S[2112]++;builder.append("stateful", _stateful);

        __CLOVER_81_0.S[2113]++;return builder.toString();
    } finally { }}

}