/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.engine;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.hivemind.util.ClasspathResource;
import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.hivemind.Location;
import org.apache.hivemind.Resource;
import org.apache.tapestry.INamespace;
import org.apache.tapestry.Tapestry;
import org.apache.tapestry.spec.IComponentSpecification;
import org.apache.tapestry.spec.ILibrarySpecification;

/**
 *  Implementation of {@link org.apache.tapestry.INamespace}
 *  that works with a {@link ISpecificationSource} to
 *  obtain page and component specifications as needed.
 *
 *  @author Howard Lewis Ship
 *  @since 2.2
 *
 **/

public class Namespace implements INamespace
{public static com.cortexeb.tools.clover.d __CLOVER_97_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private ILibrarySpecification _specification;
    private ISpecificationSource _specificationSource;
    private String _id;
    private String _extendedId;
    private INamespace _parent;
    private boolean _frameworkNamespace;
    private boolean _applicationNamespace;

    /**
     *  Map of {@link org.apache.tapestry.spec.ComponentSpecification} keyed on page name.
     *  The map is synchronized because different threads may
     *  try to update it simultaneously (due to dynamic page
     *  discovery in the application namespace).
     * 
     **/

    private Map _pages = Collections.synchronizedMap(new HashMap());

    /**
     *  Map of {@link org.apache.tapestry.spec.ComponentSpecification} keyed on
     *  component alias.
     * 
     **/

    private Map _components = Collections.synchronizedMap(new HashMap());

    /**
     *  Map, keyed on id, of {@link INamespace}.
     * 
     **/

    private Map _children = Collections.synchronizedMap(new HashMap());

    public Namespace(
        String id,
        INamespace parent,
        ILibrarySpecification specification,
        ISpecificationSource specificationSource)
    {try { __CLOVER_97_0.M[479]++;
        __CLOVER_97_0.S[2142]++;_id = id;
        __CLOVER_97_0.S[2143]++;_parent = parent;
        __CLOVER_97_0.S[2144]++;_specification = specification;
        __CLOVER_97_0.S[2145]++;_specificationSource = specificationSource;

        __CLOVER_97_0.S[2146]++;_applicationNamespace = (_id == null);
        __CLOVER_97_0.S[2147]++;_frameworkNamespace = FRAMEWORK_NAMESPACE.equals(_id);
    } finally { }}

    public String toString()
    {try { __CLOVER_97_0.M[480]++;
        __CLOVER_97_0.S[2148]++;StringBuffer buffer = new StringBuffer("Namespace@");
        __CLOVER_97_0.S[2149]++;buffer.append(Integer.toHexString(hashCode()));
        __CLOVER_97_0.S[2150]++;buffer.append('[');

        __CLOVER_97_0.S[2151]++;if ((((_applicationNamespace) && (++__CLOVER_97_0.CT[400] != 0)) || (++__CLOVER_97_0.CF[400] == 0))){
            __CLOVER_97_0.S[2152]++;buffer.append("<application>");}
        else{
            __CLOVER_97_0.S[2153]++;buffer.append(getExtendedId());}

        __CLOVER_97_0.S[2154]++;buffer.append(']');

        __CLOVER_97_0.S[2155]++;return buffer.toString();
    } finally { }}

    public String getId()
    {try { __CLOVER_97_0.M[481]++;
        __CLOVER_97_0.S[2156]++;return _id;
    } finally { }}

    public String getExtendedId()
    {try { __CLOVER_97_0.M[482]++;
        __CLOVER_97_0.S[2157]++;if ((((_applicationNamespace) && (++__CLOVER_97_0.CT[401] != 0)) || (++__CLOVER_97_0.CF[401] == 0))){
            __CLOVER_97_0.S[2158]++;return null;}

        __CLOVER_97_0.S[2159]++;if ((((_extendedId == null) && (++__CLOVER_97_0.CT[402] != 0)) || (++__CLOVER_97_0.CF[402] == 0))){
            __CLOVER_97_0.S[2160]++;_extendedId = buildExtendedId();}

        __CLOVER_97_0.S[2161]++;return _extendedId;
    } finally { }}

    public INamespace getParentNamespace()
    {try { __CLOVER_97_0.M[483]++;
        __CLOVER_97_0.S[2162]++;return _parent;
    } finally { }}

    public INamespace getChildNamespace(String id)
    {try { __CLOVER_97_0.M[484]++;
        __CLOVER_97_0.S[2163]++;String firstId = id;
        __CLOVER_97_0.S[2164]++;String nextIds = null;

        // Split the id into first and next if it is a dot separated sequence
        __CLOVER_97_0.S[2165]++;int index = id.indexOf('.');
        __CLOVER_97_0.S[2166]++;if ((((index >= 0) && (++__CLOVER_97_0.CT[403] != 0)) || (++__CLOVER_97_0.CF[403] == 0))){
        {
            __CLOVER_97_0.S[2167]++;firstId = id.substring(0, index);
            __CLOVER_97_0.S[2168]++;nextIds = id.substring(index + 1);
        }}

        // Get the first namespace
        __CLOVER_97_0.S[2169]++;INamespace result = (INamespace) _children.get(firstId);

        __CLOVER_97_0.S[2170]++;if ((((result == null) && (++__CLOVER_97_0.CT[404] != 0)) || (++__CLOVER_97_0.CF[404] == 0))){
        {
            __CLOVER_97_0.S[2171]++;result = createNamespace(firstId);

            __CLOVER_97_0.S[2172]++;_children.put(firstId, result);
        }}

        // If the id is a dot separated sequence, recurse to find 
        // the needed namespace
        __CLOVER_97_0.S[2173]++;if ((((result != null && nextIds != null) && (++__CLOVER_97_0.CT[405] != 0)) || (++__CLOVER_97_0.CF[405] == 0))){
            __CLOVER_97_0.S[2174]++;result = result.getChildNamespace(nextIds);}

        __CLOVER_97_0.S[2175]++;return result;
    } finally { }}

    public List getChildIds()
    {try { __CLOVER_97_0.M[485]++;
        __CLOVER_97_0.S[2176]++;return _specification.getLibraryIds();
    } finally { }}

    public IComponentSpecification getPageSpecification(String name)
    {try { __CLOVER_97_0.M[486]++;
        __CLOVER_97_0.S[2177]++;IComponentSpecification result = (IComponentSpecification) _pages.get(name);

        __CLOVER_97_0.S[2178]++;if ((((result == null) && (++__CLOVER_97_0.CT[406] != 0)) || (++__CLOVER_97_0.CF[406] == 0))){
        {
            __CLOVER_97_0.S[2179]++;result = locatePageSpecification(name);

            __CLOVER_97_0.S[2180]++;_pages.put(name, result);
        }}

        __CLOVER_97_0.S[2181]++;return result;
    } finally { }}

    public List getPageNames()
    {try { __CLOVER_97_0.M[487]++;
        __CLOVER_97_0.S[2182]++;Set names = new HashSet();

        __CLOVER_97_0.S[2183]++;names.addAll(_pages.keySet());
        __CLOVER_97_0.S[2184]++;names.addAll(_specification.getPageNames());

        __CLOVER_97_0.S[2185]++;List result = new ArrayList(names);

        __CLOVER_97_0.S[2186]++;Collections.sort(result);

        __CLOVER_97_0.S[2187]++;return result;
    } finally { }}

    public IComponentSpecification getComponentSpecification(String alias)
    {try { __CLOVER_97_0.M[488]++;
        __CLOVER_97_0.S[2188]++;IComponentSpecification result = (IComponentSpecification) _components.get(alias);

        __CLOVER_97_0.S[2189]++;if ((((result == null) && (++__CLOVER_97_0.CT[407] != 0)) || (++__CLOVER_97_0.CF[407] == 0))){
        {
            __CLOVER_97_0.S[2190]++;result = locateComponentSpecification(alias);
            __CLOVER_97_0.S[2191]++;_components.put(alias, result);
        }}

        __CLOVER_97_0.S[2192]++;return result;
    } finally { }}

    public String getServiceClassName(String name)
    {try { __CLOVER_97_0.M[489]++;
        __CLOVER_97_0.S[2193]++;return _specification.getServiceClassName(name);
    } finally { }}

    public List getServiceNames()
    {try { __CLOVER_97_0.M[490]++;
        __CLOVER_97_0.S[2194]++;return _specification.getServiceNames();
    } finally { }}

    public ILibrarySpecification getSpecification()
    {try { __CLOVER_97_0.M[491]++;
        __CLOVER_97_0.S[2195]++;return _specification;
    } finally { }}

    private String buildExtendedId()
    {try { __CLOVER_97_0.M[492]++;
        __CLOVER_97_0.S[2196]++;if ((((_parent == null) && (++__CLOVER_97_0.CT[408] != 0)) || (++__CLOVER_97_0.CF[408] == 0))){
            __CLOVER_97_0.S[2197]++;return _id;}

        __CLOVER_97_0.S[2198]++;String parentId = _parent.getExtendedId();

        // If immediate child of application namespace

        __CLOVER_97_0.S[2199]++;if ((((parentId == null) && (++__CLOVER_97_0.CT[409] != 0)) || (++__CLOVER_97_0.CF[409] == 0))){
            __CLOVER_97_0.S[2200]++;return _id;}

        __CLOVER_97_0.S[2201]++;return parentId + "." + _id;
    } finally { }}

    /**
     *  Returns a string identifying the namespace, for use in
     *  error messages.  I.e., "Application namespace" or "namespace 'foo'".
     * 
     **/

    public String getNamespaceId()
    {try { __CLOVER_97_0.M[493]++;
        __CLOVER_97_0.S[2202]++;if ((((_frameworkNamespace) && (++__CLOVER_97_0.CT[410] != 0)) || (++__CLOVER_97_0.CF[410] == 0))){
            __CLOVER_97_0.S[2203]++;return Tapestry.getMessage("Namespace.framework-namespace");}

        __CLOVER_97_0.S[2204]++;if ((((_applicationNamespace) && (++__CLOVER_97_0.CT[411] != 0)) || (++__CLOVER_97_0.CF[411] == 0))){
            __CLOVER_97_0.S[2205]++;return Tapestry.getMessage("Namespace.application-namespace");}

        __CLOVER_97_0.S[2206]++;return Tapestry.format("Namespace.nested-namespace", getExtendedId());
    } finally { }}

    /**
     *  Gets the specification from the specification source.
     * 
     *  @throws ApplicationRuntimeException if the named page is not defined.
     * 
     **/

    private IComponentSpecification locatePageSpecification(String name)
    {try { __CLOVER_97_0.M[494]++;
        __CLOVER_97_0.S[2207]++;String path = _specification.getPageSpecificationPath(name);

        __CLOVER_97_0.S[2208]++;if ((((path == null) && (++__CLOVER_97_0.CT[412] != 0)) || (++__CLOVER_97_0.CF[412] == 0))){
            __CLOVER_97_0.S[2209]++;throw new ApplicationRuntimeException(
                Tapestry.format("Namespace.no-such-page", name, getNamespaceId()));}

        __CLOVER_97_0.S[2210]++;Resource location = getSpecificationLocation().getRelativeResource(path);

        __CLOVER_97_0.S[2211]++;return _specificationSource.getPageSpecification(location);
    } finally { }}

    private IComponentSpecification locateComponentSpecification(String type)
    {try { __CLOVER_97_0.M[495]++;
        __CLOVER_97_0.S[2212]++;String path = _specification.getComponentSpecificationPath(type);

        __CLOVER_97_0.S[2213]++;if ((((path == null) && (++__CLOVER_97_0.CT[413] != 0)) || (++__CLOVER_97_0.CF[413] == 0))){
            __CLOVER_97_0.S[2214]++;throw new ApplicationRuntimeException(
                Tapestry.format("Namespace.no-such-alias", type, getNamespaceId()));}

        __CLOVER_97_0.S[2215]++;Resource location = getSpecificationLocation().getRelativeResource(path);

        __CLOVER_97_0.S[2216]++;return _specificationSource.getComponentSpecification(location);
    } finally { }}

    private INamespace createNamespace(String id)
    {try { __CLOVER_97_0.M[496]++;
        __CLOVER_97_0.S[2217]++;String path = _specification.getLibrarySpecificationPath(id);

        __CLOVER_97_0.S[2218]++;if ((((path == null) && (++__CLOVER_97_0.CT[414] != 0)) || (++__CLOVER_97_0.CF[414] == 0))){
            __CLOVER_97_0.S[2219]++;throw new ApplicationRuntimeException(
                Tapestry.format("Namespace.library-id-not-found", id, getNamespaceId()));}

        __CLOVER_97_0.S[2220]++;Resource location = getSpecificationLocation().getRelativeResource(path);

        // Ok, an absolute path to a library for an application whose specification
        // is in the context root is problematic, cause getRelativeLocation()
        // will still be looking in the context.  Handle this case with the
        // following little kludge:

        __CLOVER_97_0.S[2221]++;if ((((location.getResourceURL() == null && path.startsWith("/")) && (++__CLOVER_97_0.CT[415] != 0)) || (++__CLOVER_97_0.CF[415] == 0))){
            __CLOVER_97_0.S[2222]++;location = new ClasspathResource(_specification.getResourceResolver(), path);}

        __CLOVER_97_0.S[2223]++;ILibrarySpecification ls = _specificationSource.getLibrarySpecification(location);

        __CLOVER_97_0.S[2224]++;return new Namespace(id, this, ls, _specificationSource);
    } finally { }}

    public boolean containsPage(String name)
    {try { __CLOVER_97_0.M[497]++;
        __CLOVER_97_0.S[2225]++;return _pages.containsKey(name) || (_specification.getPageSpecificationPath(name) != null);
    } finally { }}

    /** @since 2.3 **/

    public String constructQualifiedName(String pageName)
    {try { __CLOVER_97_0.M[498]++;
        __CLOVER_97_0.S[2226]++;String prefix = getExtendedId();

        __CLOVER_97_0.S[2227]++;if ((((prefix == null) && (++__CLOVER_97_0.CT[416] != 0)) || (++__CLOVER_97_0.CF[416] == 0))){
            __CLOVER_97_0.S[2228]++;return pageName;}

        __CLOVER_97_0.S[2229]++;return prefix + SEPARATOR + pageName;
    } finally { }}

    /** @since 3.0 **/

    public Resource getSpecificationLocation()
    {try { __CLOVER_97_0.M[499]++;
        __CLOVER_97_0.S[2230]++;return _specification.getSpecificationLocation();
    } finally { }}

    /** @since 3.0 **/

    public boolean isApplicationNamespace()
    {try { __CLOVER_97_0.M[500]++;
        __CLOVER_97_0.S[2231]++;return _applicationNamespace;
    } finally { }}

    /** @since 3.0 **/

    public synchronized void installPageSpecification(
        String pageName,
        IComponentSpecification specification)
    {try { __CLOVER_97_0.M[501]++;
        __CLOVER_97_0.S[2232]++;_pages.put(pageName, specification);
    } finally { }}

    /** @since 3.0 **/

    public synchronized void installComponentSpecification(
        String type,
        IComponentSpecification specification)
    {try { __CLOVER_97_0.M[502]++;
        __CLOVER_97_0.S[2233]++;_components.put(type, specification);
    } finally { }}

    /** @since 3.0 **/

    public boolean containsComponentType(String type)
    {try { __CLOVER_97_0.M[503]++;
        __CLOVER_97_0.S[2234]++;return _components.containsKey(type)
            || (_specification.getComponentSpecificationPath(type) != null);
    } finally { }}

    /** @since 3.0 **/

    public List getComponentTypes()
    {try { __CLOVER_97_0.M[504]++;
        __CLOVER_97_0.S[2235]++;Set types = new HashSet();

        __CLOVER_97_0.S[2236]++;types.addAll(_components.keySet());
        __CLOVER_97_0.S[2237]++;types.addAll(_specification.getComponentTypes());

        __CLOVER_97_0.S[2238]++;List result = new ArrayList(types);

        __CLOVER_97_0.S[2239]++;Collections.sort(result);

        __CLOVER_97_0.S[2240]++;return result;
    } finally { }}

    /** @since 3.0 **/

    public Location getLocation()
    {try { __CLOVER_97_0.M[505]++;
        __CLOVER_97_0.S[2241]++;if ((((_specification == null) && (++__CLOVER_97_0.CT[417] != 0)) || (++__CLOVER_97_0.CF[417] == 0))){
            __CLOVER_97_0.S[2242]++;return null;}

        __CLOVER_97_0.S[2243]++;return _specification.getLocation();
    } finally { }}

}
