/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.enhance;

import java.lang.reflect.Modifier;
import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.hivemind.service.ClassFab;
import org.apache.hivemind.service.MethodSignature;
import org.apache.tapestry.Tapestry;

/**
 *  Creates a synthetic property for a
 *  {@link org.apache.tapestry.spec.Direction#AUTO}
 *  parameter.
 *
 *  @author Mindbridge
 *  @since 3.0
 *
 */

public class CreateAutoParameterEnhancer implements IEnhancer
{public static com.cortexeb.tools.clover.d __CLOVER_108_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private static final Log LOG = LogFactory.getLog(CreateAutoParameterEnhancer.class);

    /**
     *  The code template for the parameter accessor method.    
     *                                      <p>
     *  Legend:                             <br>
     *      {0} = readBindingMethodName     <br>
     *      {1} = binding value accessor    <br>
     *      {2} = cast (if needed)          <br>
     */
    protected static final String PARAMETER_ACCESSOR_TEMPLATE =
        ""
            + "'{'"
            + "  org.apache.tapestry.IBinding binding = {0}();"
            + "  return {2} binding.{1}(); "
            + "'}'";

    /**
     *  The code template for the parameter mutator method.    
     *                                      <p>
     *  Legend:                             <br>
     *      {0} = readBindingMethodName     <br>
     *      {1} = binding value mutator     <br>
     *      {2} = value cast
     */
    protected static final String PARAMETER_MUTATOR_TEMPLATE =
        ""
            + "'{'"
            + "  org.apache.tapestry.IBinding binding = {0}();"
            + "  binding.{1}({2} $1); "
            + "'}'";

    /**
     *  The list of types that have accessors and mutators 
     *  other than getObject()/setObject.
     *  The key in the Map is the type, the value is the property name in IBinding 
     */
    private static final Map SPECIAL_BINDING_TYPES = new HashMap();

    static {
        __CLOVER_108_0.S[2669]++;SPECIAL_BINDING_TYPES.put("boolean", "boolean");
        __CLOVER_108_0.S[2670]++;SPECIAL_BINDING_TYPES.put("int", "int");
        __CLOVER_108_0.S[2671]++;SPECIAL_BINDING_TYPES.put("double", "double");
        __CLOVER_108_0.S[2672]++;SPECIAL_BINDING_TYPES.put("java.lang.String", "string");
    }

    private static final Map VALUE_CAST_TYPES = new HashMap();

    static {
        __CLOVER_108_0.S[2673]++;VALUE_CAST_TYPES.put("byte", "($w)");
        __CLOVER_108_0.S[2674]++;VALUE_CAST_TYPES.put("long", "($w)");
        __CLOVER_108_0.S[2675]++;VALUE_CAST_TYPES.put("short", "($w)");
        __CLOVER_108_0.S[2676]++;VALUE_CAST_TYPES.put("char", "($w)");
        __CLOVER_108_0.S[2677]++;VALUE_CAST_TYPES.put("float", "($w)");
    }

    private String _propertyName;
    private String _parameterName;
    private Class _type;
    private String _readMethodName;

    public CreateAutoParameterEnhancer(
        String propertyName,
        String parameterName,
        Class type,
        String readMethodName)
    {try { __CLOVER_108_0.M[614]++;
        __CLOVER_108_0.S[2678]++;Tapestry.notNull(propertyName, "propertyName");
        __CLOVER_108_0.S[2679]++;Tapestry.notNull(parameterName, "parameterName");
        __CLOVER_108_0.S[2680]++;Tapestry.notNull(type, "type");
        __CLOVER_108_0.S[2681]++;Tapestry.notNull(readMethodName, "readMethodName");

        __CLOVER_108_0.S[2682]++;_propertyName = propertyName;
        __CLOVER_108_0.S[2683]++;_parameterName = parameterName;
        __CLOVER_108_0.S[2684]++;_type = type;
        __CLOVER_108_0.S[2685]++;_readMethodName = readMethodName;
    } finally { }}

    public void performEnhancement(ClassFab classFab)
    {try { __CLOVER_108_0.M[615]++;
        __CLOVER_108_0.S[2686]++;if ((((LOG.isDebugEnabled()) && (++__CLOVER_108_0.CT[495] != 0)) || (++__CLOVER_108_0.CF[495] == 0))){
            __CLOVER_108_0.S[2687]++;LOG.debug("Creating auto property: " + _propertyName);}

        __CLOVER_108_0.S[2688]++;String readBindingMethodName =
            CreateAccessorUtils.buildMethodName(
                "get",
                _parameterName + Tapestry.PARAMETER_PROPERTY_NAME_SUFFIX);

        __CLOVER_108_0.S[2689]++;createReadMethod(classFab, readBindingMethodName);
        __CLOVER_108_0.S[2690]++;createWriteMethod(classFab, readBindingMethodName);
    } finally { }}

    private String getSpecialBindingType()
    {try { __CLOVER_108_0.M[616]++;
        __CLOVER_108_0.S[2691]++;String typeName = _type.getName();

        __CLOVER_108_0.S[2692]++;return (String) SPECIAL_BINDING_TYPES.get(typeName);
    } finally { }}

    private String getValueCastType()
    {try { __CLOVER_108_0.M[617]++;
        __CLOVER_108_0.S[2693]++;String typeName = _type.getName();

        __CLOVER_108_0.S[2694]++;return (String) VALUE_CAST_TYPES.get(typeName);
    } finally { }}

    private void createReadMethod(ClassFab classFab, String readBindingMethodName)
    {try { __CLOVER_108_0.M[618]++;
        __CLOVER_108_0.S[2695]++;String castToType = "";
        __CLOVER_108_0.S[2696]++;String bindingValueAccessor = "getObject";

        __CLOVER_108_0.S[2697]++;String specialBindingType = getSpecialBindingType();

        __CLOVER_108_0.S[2698]++;if ((((specialBindingType != null) && (++__CLOVER_108_0.CT[496] != 0)) || (++__CLOVER_108_0.CF[496] == 0))){
        {
            __CLOVER_108_0.S[2699]++;bindingValueAccessor = CreateAccessorUtils.buildMethodName("get", specialBindingType);
        }}
        else{
        {
            __CLOVER_108_0.S[2700]++;castToType = "($r)";
        }}

        __CLOVER_108_0.S[2701]++;String body =
            MessageFormat.format(
                PARAMETER_ACCESSOR_TEMPLATE,
                new Object[] { readBindingMethodName, bindingValueAccessor, castToType });

        __CLOVER_108_0.S[2702]++;MethodSignature sig = new MethodSignature(_type, _readMethodName, null, null);

        __CLOVER_108_0.S[2703]++;classFab.addMethod(Modifier.PUBLIC | Modifier.FINAL, sig, body);
    } finally { }}

    private void createWriteMethod(ClassFab classFab, String readBindingMethodName)
    {try { __CLOVER_108_0.M[619]++;
        __CLOVER_108_0.S[2704]++;String bindingValueAccessor = "setObject";
        __CLOVER_108_0.S[2705]++;String valueCast = "";

        __CLOVER_108_0.S[2706]++;String specialBindingType = getSpecialBindingType();

        __CLOVER_108_0.S[2707]++;if ((((specialBindingType != null) && (++__CLOVER_108_0.CT[497] != 0)) || (++__CLOVER_108_0.CF[497] == 0))){
        {
            __CLOVER_108_0.S[2708]++;bindingValueAccessor = CreateAccessorUtils.buildMethodName("set", specialBindingType);
        }}
        else{
        {
            __CLOVER_108_0.S[2709]++;String castForType = getValueCastType();

            __CLOVER_108_0.S[2710]++;if ((((castForType != null) && (++__CLOVER_108_0.CT[498] != 0)) || (++__CLOVER_108_0.CF[498] == 0))){
                __CLOVER_108_0.S[2711]++;valueCast = castForType;}
        }}

        __CLOVER_108_0.S[2712]++;String body =
            MessageFormat.format(
                PARAMETER_MUTATOR_TEMPLATE,
                new Object[] { readBindingMethodName, bindingValueAccessor, valueCast });

        __CLOVER_108_0.S[2713]++;String methodName = CreateAccessorUtils.buildMethodName("set", _propertyName);

        __CLOVER_108_0.S[2714]++;MethodSignature sig =
            new MethodSignature(void.class, methodName, new Class[] { _type }, null);

        __CLOVER_108_0.S[2715]++;classFab.addMethod(Modifier.PUBLIC | Modifier.FINAL, sig, body);
    } finally { }}
}
