/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.form;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.tapestry.Tapestry;

/**
 *  A utility class often used with the {@link org.apache.tapestry.form.ListEdit} component.
 *  A ListEditMap is loaded with data objects before the ListEdit renders, and again
 *  before the ListEdit rewinds.  This streamlines the synchronization of the form
 *  against data on the server.  It is most useful when the set of objects is of a manageable
 *  size (say, no more than a few hundred objects).
 * 
 *  <p>
 *  The map stores a list of keys, and relates each key to a value.
 *  It also tracks a deleted flag for each key.
 * 
 *  <p>
 *  Usage:
 *  <br>
 *  The page or component should implement {@link org.apache.tapestry.event.PageRenderListener}
 *  and implement {@link org.apache.tapestry.event.PageRenderListener#pageBeginRender(org.apache.tapestry.event.PageEvent)}
 *  to initialize the map.
 * 
 *  <p>
 *  The external data (from which keys and values are obtained) is queried, and
 *  each key/value pair is {@link #add(Object, Object) added} to the map, in the order
 *  that items should be presented.
 * 
 *  <p>
 *  The {@link org.apache.tapestry.form.ListEdit}'s source parameter should be bound
 *  to the map's {@link #getKeys() keys} property.  The value parameter
 *  should be bound to the map's {@link #setKey(Object) key} property.
 * 
 *  <p>
 *  The {@link org.apache.tapestry.form.ListEdit}'s listener parameter should be 
 *  bound to a listener method to synchronize a property of the component from
 *  the map.
 * 
 *  <code>
 *  public void synchronize({@link org.apache.tapestry.IRequestCycle} cycle)
 *  {
 *     ListEditMap map = ...;
 *     <i>Type</i> object = (<i>Type</i>)map.getValue();
 * 
 *     if (object == null)
 *       ...
 * 
 *     set<i>Property</i>(object);
 *  }
 *  </code>
 * 
 *  <p>
 *  You may also connect a {@link org.apache.tapestry.form.Checkbox}'s
 *  selected parameter to the map's {@link #isDeleted() deleted} property.
 *  
 *  <p>
 *  You may track inclusion in other sets by subclasses and implementing
 *  new boolean properties.  The accessor method should be a call to
 *  {@link #checkSet(Set)} and the mutator method should be a call
 *  to {@link #updateSet(Set, boolean)}.
 * 
 *  @author Howard Lewis Ship
 *  @since 3.0
 *
 **/

public class ListEditMap
{public static com.cortexeb.tools.clover.d __CLOVER_139_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private Map _map = new HashMap();
    private List _keys = new ArrayList();
    private Set _deletedKeys;
    private Object _currentKey;

    /**
     *  Records the key and value into this map.
     *  The keys may be obtained, in the order in which they are added,
     *  using {@link #getKeys()}.
     *  This also sets the current key (so that you
     *  may invoke {@link #setDeleted(boolean)}, for example).
     * 
     **/

    public void add(Object key, Object value)
    {try { __CLOVER_139_0.M[734]++;
    	__CLOVER_139_0.S[3397]++;_currentKey = key;
    	
        __CLOVER_139_0.S[3398]++;_keys.add(_currentKey);
        __CLOVER_139_0.S[3399]++;_map.put(_currentKey, value);
    } finally { }}

    /**
     *  Returns a List of keys, in the order that keys
     *  were added to the map (using {@link #add(Object, Object)}.
     *  The caller must not modify the List.
     * 
     **/

    public List getKeys()
    {try { __CLOVER_139_0.M[735]++;
        __CLOVER_139_0.S[3400]++;return _keys;
    } finally { }}

    /**
     *  Sets the key for the map.  This defines the key used
     *  with the other methods: {@link #getValue()}, 
     *  {@link #isDeleted()}, {@link #setDeleted(boolean)}.
     * 
     **/

    public void setKey(Object key)
    {try { __CLOVER_139_0.M[736]++;
        __CLOVER_139_0.S[3401]++;_currentKey = key;
    } finally { }}

    /**
     *  Returns the current key within the map.
     * 
     **/

    public Object getKey()
    {try { __CLOVER_139_0.M[737]++;
        __CLOVER_139_0.S[3402]++;return _currentKey;
    } finally { }}

    /**
     *  Returns the value for the key (set using {@link #setKey(Object)}).
     *  May return null if no such key has been added (this can occur
     *  if a data object is deleted between the time a form is rendered
     *  and the time a form is submitted).
     * 
     **/

    public Object getValue()
    {try { __CLOVER_139_0.M[738]++;
        __CLOVER_139_0.S[3403]++;return _map.get(_currentKey);
    } finally { }}

    /**
     *  Returns true if the {@link #setKey(Object) current key}
     *  is in the set of deleted keys.
     * 
     **/

    public boolean isDeleted()
    {try { __CLOVER_139_0.M[739]++;
        __CLOVER_139_0.S[3404]++;return checkSet(_deletedKeys);
    } finally { }}

    /**
     *  Returns true if the set contains the
     *  {@link #getKey() current key}.  Returns 
     *  false is the set is null, or doesn't contain
     *  the current key.
     * 
     **/

    protected boolean checkSet(Set set)
    {try { __CLOVER_139_0.M[740]++;
        __CLOVER_139_0.S[3405]++;if ((((set == null) && (++__CLOVER_139_0.CT[628] != 0)) || (++__CLOVER_139_0.CF[628] == 0))){
            __CLOVER_139_0.S[3406]++;return false;}

        __CLOVER_139_0.S[3407]++;return set.contains(_currentKey);
    } finally { }}

    /**
     *  Adds or removes the {@link #setKey(Object) current key}
     *  from the set of deleted keys.
     * 
     **/

    public void setDeleted(boolean value)
    {try { __CLOVER_139_0.M[741]++;
        __CLOVER_139_0.S[3408]++;_deletedKeys = updateSet(_deletedKeys, value);
    } finally { }}

    /**
     *  Updates the set, adding or removing the {@link #getKey() current key}
     *  from it.  Returns the set passed in.
     *  If the value is true and the set is null, an new instance
     *  of {@link HashSet} is created and returned.
     * 
     **/

    protected Set updateSet(Set set, boolean value)
    {try { __CLOVER_139_0.M[742]++;
        __CLOVER_139_0.S[3409]++;if ((((value) && (++__CLOVER_139_0.CT[629] != 0)) || (++__CLOVER_139_0.CF[629] == 0))){
        {
            __CLOVER_139_0.S[3410]++;if ((((set == null) && (++__CLOVER_139_0.CT[630] != 0)) || (++__CLOVER_139_0.CF[630] == 0))){
                __CLOVER_139_0.S[3411]++;set = new HashSet();}

            __CLOVER_139_0.S[3412]++;set.add(_currentKey);
        }}
        else{
        {
            __CLOVER_139_0.S[3413]++;if ((((set != null) && (++__CLOVER_139_0.CT[631] != 0)) || (++__CLOVER_139_0.CF[631] == 0))){
                __CLOVER_139_0.S[3414]++;set.remove(_currentKey);}
        }}

        __CLOVER_139_0.S[3415]++;return set;
    } finally { }}

    /**
     *  Returns the deleted keys in an unspecified order.  May return
     *  null if no keys are deleted.
     * 
     **/

    public List getDeletedKeys()
    {try { __CLOVER_139_0.M[743]++;
    	__CLOVER_139_0.S[3416]++;return convertSetToList(_deletedKeys);
    } finally { }}
    
    protected List convertSetToList(Set set)
    {try { __CLOVER_139_0.M[744]++;
        __CLOVER_139_0.S[3417]++;if ((((Tapestry.isEmpty(set)) && (++__CLOVER_139_0.CT[632] != 0)) || (++__CLOVER_139_0.CF[632] == 0))){
            __CLOVER_139_0.S[3418]++;return null;}

        __CLOVER_139_0.S[3419]++;return new ArrayList(set);
    } finally { }}

    /**
     *  Returns all the values stored in the map, in the
     *  order in which values were added to the map
     *  using {@link #add(Object, Object)}.
     * 
     **/

    public List getAllValues()
    {try { __CLOVER_139_0.M[745]++;
        __CLOVER_139_0.S[3420]++;int count = _keys.size();
        __CLOVER_139_0.S[3421]++;List result = new ArrayList(count);

        __CLOVER_139_0.S[3422]++;for (int i = 0; (((i < count) && (++__CLOVER_139_0.CT[633] != 0)) || (++__CLOVER_139_0.CF[633] == 0)); i++){
        {
            __CLOVER_139_0.S[3423]++;Object key = _keys.get(i);
            __CLOVER_139_0.S[3424]++;Object value = _map.get(key);

            __CLOVER_139_0.S[3425]++;result.add(value);
        }}

        __CLOVER_139_0.S[3426]++;return result;
    } finally { }}

    /**
      *  Returns all the values stored in the map, excluding
      *  those whose id has been marked deleted, in the
      *  order in which values were added to the map
      *  using {@link #add(Object, Object)}.
      * 
      **/

    public List getValues()
    {try { __CLOVER_139_0.M[746]++;
        __CLOVER_139_0.S[3427]++;int deletedCount = Tapestry.size(_deletedKeys);

        __CLOVER_139_0.S[3428]++;if ((((deletedCount == 0) && (++__CLOVER_139_0.CT[634] != 0)) || (++__CLOVER_139_0.CF[634] == 0))){
            __CLOVER_139_0.S[3429]++;return getAllValues();}

        __CLOVER_139_0.S[3430]++;int count = _keys.size();

        __CLOVER_139_0.S[3431]++;List result = new ArrayList(count - deletedCount);

        __CLOVER_139_0.S[3432]++;for (int i = 0; (((i < count) && (++__CLOVER_139_0.CT[635] != 0)) || (++__CLOVER_139_0.CF[635] == 0)); i++){
        {
            __CLOVER_139_0.S[3433]++;Object key = _keys.get(i);

            __CLOVER_139_0.S[3434]++;if ((((_deletedKeys.contains(key)) && (++__CLOVER_139_0.CT[636] != 0)) || (++__CLOVER_139_0.CF[636] == 0))){
                __CLOVER_139_0.S[3435]++;continue;}

            __CLOVER_139_0.S[3436]++;Object value = _map.get(key);
            __CLOVER_139_0.S[3437]++;result.add(value);
        }}

        __CLOVER_139_0.S[3438]++;return result;
    } finally { }}

}
