/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.form;

import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.tapestry.IBinding;
import org.apache.tapestry.IForm;
import org.apache.tapestry.IMarkupWriter;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.Tapestry;

/**
 *  A special type of form component that is used to contain {@link Radio}
 *  components.  The Radio and {@link Radio} group components work together to
 *  update a property of some other object, much like a more flexible
 *  version of a {@link PropertySelection}.
 *
 *  [<a href="../../../../../ComponentReference/RadioGroup.html">Component Reference</a>]
 *
 *  @author Howard Lewis Ship
 * 
 **/

public abstract class RadioGroup extends AbstractFormComponent
{public static com.cortexeb.tools.clover.d __CLOVER_143_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    // Cached copy of the value from the selectedBinding
    private Object _selection;

    // The value from the HTTP request indicating which
    // Radio was selected by the user.
    private int _selectedOption;

    private boolean _rewinding;
    private boolean _rendering;
    private int _nextOptionId;

    /**
     *  A <code>RadioGroup</code> places itself into the {@link IRequestCycle} as
     *  an attribute, so that its wrapped {@link Radio} components can identify thier
     *  state.
     *
     **/

    private static final String ATTRIBUTE_NAME = "org.apache.tapestry.active.RadioGroup";

    public static RadioGroup get(IRequestCycle cycle)
    {try { __CLOVER_143_0.M[752]++;
        __CLOVER_143_0.S[3540]++;return (RadioGroup) cycle.getAttribute(ATTRIBUTE_NAME);
    } finally { }}

    public abstract IBinding getSelectedBinding();

    public int getNextOptionId()
    {try { __CLOVER_143_0.M[753]++;
        __CLOVER_143_0.S[3541]++;if ((((!_rendering) && (++__CLOVER_143_0.CT[663] != 0)) || (++__CLOVER_143_0.CF[663] == 0))){
            __CLOVER_143_0.S[3542]++;throw Tapestry.createRenderOnlyPropertyException(this, "nextOptionId");}

        __CLOVER_143_0.S[3543]++;return _nextOptionId++;
    } finally { }}

    /**
     *  Used by {@link Radio} components wrapped by this <code>RadioGroup</code> to see
     *  if the group as a whole is disabled.
     *
     **/

    public abstract boolean isDisabled();

    public boolean isRewinding()
    {try { __CLOVER_143_0.M[754]++;
        __CLOVER_143_0.S[3544]++;if ((((!_rendering) && (++__CLOVER_143_0.CT[664] != 0)) || (++__CLOVER_143_0.CF[664] == 0))){
            __CLOVER_143_0.S[3545]++;throw Tapestry.createRenderOnlyPropertyException(this, "rewinding");}

        __CLOVER_143_0.S[3546]++;return _rewinding;
    } finally { }}

    /**
     *  Returns true if the value is equal to the current selection for the
     *  group.  This is invoked by a {@link Radio} during rendering
     *  to determine if it should be marked 'checked'.
     *
     **/

    public boolean isSelection(Object value)
    {try { __CLOVER_143_0.M[755]++;
        __CLOVER_143_0.S[3547]++;if ((((!_rendering) && (++__CLOVER_143_0.CT[665] != 0)) || (++__CLOVER_143_0.CF[665] == 0))){
            __CLOVER_143_0.S[3548]++;throw Tapestry.createRenderOnlyPropertyException(this, "selection");}

        __CLOVER_143_0.S[3549]++;if ((((_selection == value) && (++__CLOVER_143_0.CT[666] != 0)) || (++__CLOVER_143_0.CF[666] == 0))){
            __CLOVER_143_0.S[3550]++;return true;}

        __CLOVER_143_0.S[3551]++;if ((((_selection == null || value == null) && (++__CLOVER_143_0.CT[667] != 0)) || (++__CLOVER_143_0.CF[667] == 0))){
            __CLOVER_143_0.S[3552]++;return false;}

        __CLOVER_143_0.S[3553]++;return _selection.equals(value);
    } finally { }}

    /**
    *  Invoked by the {@link Radio} which is selected to update the 
    *  property bound to the selected parameter.
    *
    **/

    public void updateSelection(Object value)
    {try { __CLOVER_143_0.M[756]++;
        __CLOVER_143_0.S[3554]++;getSelectedBinding().setObject(value);
    } finally { }}

    /**
    *  Used by {@link Radio} components when rewinding to see if their value was submitted.
    *
    **/

    public boolean isSelected(int option)
    {try { __CLOVER_143_0.M[757]++;
        __CLOVER_143_0.S[3555]++;return _selectedOption == option;
    } finally { }}

    /**
     * Doesn't actual render an HTML element as there is no direct equivalent for
     * an HTML element.  A <code>RadioGroup</code> component exists to organize the
     * {@link Radio} components it wraps (directly or indirectly).
     *
     * A {@link Radio} can finds its {@link RadioGroup} as a {@link IRequestCycle} attribute.
     *
     **/

    protected void renderComponent(IMarkupWriter writer, IRequestCycle cycle)
    {try { __CLOVER_143_0.M[758]++;
        __CLOVER_143_0.S[3556]++;IForm form = getForm(cycle);

        __CLOVER_143_0.S[3557]++;if ((((cycle.getAttribute(ATTRIBUTE_NAME) != null) && (++__CLOVER_143_0.CT[668] != 0)) || (++__CLOVER_143_0.CF[668] == 0))){
            __CLOVER_143_0.S[3558]++;throw new ApplicationRuntimeException(
                Tapestry.getMessage("RadioGroup.may-not-nest"),
                this,
                null,
                null);}

        // It isn't enough to know whether the cycle in general is rewinding, need to know
        // specifically if the form which contains this component is rewinding.

        __CLOVER_143_0.S[3559]++;_rewinding = form.isRewinding();

        // Used whether rewinding or not.

        __CLOVER_143_0.S[3560]++;String name = form.getElementId(this);

        __CLOVER_143_0.S[3561]++;cycle.setAttribute(ATTRIBUTE_NAME, this);

        // When rewinding, find out which (if any) radio was selected by
        // the user.

        __CLOVER_143_0.S[3562]++;if ((((_rewinding) && (++__CLOVER_143_0.CT[669] != 0)) || (++__CLOVER_143_0.CF[669] == 0))){
        {
            __CLOVER_143_0.S[3563]++;String value = cycle.getRequestContext().getParameter(name);
            __CLOVER_143_0.S[3564]++;if ((((value == null) && (++__CLOVER_143_0.CT[670] != 0)) || (++__CLOVER_143_0.CF[670] == 0))){
                __CLOVER_143_0.S[3565]++;_selectedOption = -1;}
            else{
                __CLOVER_143_0.S[3566]++;_selectedOption = Integer.parseInt(value);}
        }}

        __CLOVER_143_0.S[3567]++;try
        {
            __CLOVER_143_0.S[3568]++;_rendering = true;
            __CLOVER_143_0.S[3569]++;_nextOptionId = 0;

            // For rendering, the Radio components need to know what the current
            // selection is, so that the correct one can mark itself 'checked'.

            __CLOVER_143_0.S[3570]++;if ((((!_rewinding) && (++__CLOVER_143_0.CT[671] != 0)) || (++__CLOVER_143_0.CF[671] == 0))){
                __CLOVER_143_0.S[3571]++;_selection = getSelectedBinding().getObject();}

            __CLOVER_143_0.S[3572]++;renderBody(writer, cycle);
        }
        finally
        {
            __CLOVER_143_0.S[3573]++;_rendering = false;
            __CLOVER_143_0.S[3574]++;_selection = null;
        }

        __CLOVER_143_0.S[3575]++;cycle.removeAttribute(ATTRIBUTE_NAME);
    } finally { }}
}