/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.listener;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import ognl.OgnlRuntime;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.tapestry.IActionListener;
import org.apache.tapestry.IComponent;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.Tapestry;

/**
 *  Maps a class to a set of listeners based on the public methods of the class.
 *  {@link org.apache.tapestry.listener.ListenerMapPropertyAccessor} is setup
 *  to provide these methods as named properties of the ListenerMap.
 *
 *  @author Howard Ship
 *  @since 1.0.2
 * 
 **/

public class ListenerMap
{public static com.cortexeb.tools.clover.d __CLOVER_183_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private static final Log LOG = LogFactory.getLog(ListenerMap.class);

    static {
        __CLOVER_183_0.S[4297]++;OgnlRuntime.setPropertyAccessor(ListenerMap.class, new ListenerMapPropertyAccessor());
    }

    private Object _target;

    /**
     *  A {@link Map} of relevant {@link Method}s, keyed on method name.
     *  This is just the public void methods that take an {@link IRequestCycle}
     *  and throw nothing or just {@link ApplicationRuntimeException}.
     */

    private Map _methodMap;

    /**
     * A {@link Map} of cached listener instances, keyed on method name
     *
     **/

    private Map _listenerCache = new HashMap();

    /**
     * A {@link Map}, keyed on Class, of {@link Map} ... the method map
     * for any particular instance of the given class.
     *
     **/

    private static Map _classMap = new HashMap();

    /**
     *  Implements both listener interfaces.
     *
     **/

    private class SyntheticListener implements IActionListener
    {
        private Method _method;

        SyntheticListener(Method method)
        {try { __CLOVER_183_0.M[907]++;
            __CLOVER_183_0.S[4298]++;_method = method;
        } finally { }}

        private void invoke(IRequestCycle cycle)
        {try { __CLOVER_183_0.M[908]++;
            __CLOVER_183_0.S[4299]++;Object[] args = new Object[] { cycle };

            __CLOVER_183_0.S[4300]++;invokeTargetMethod(_target, _method, args);
        } finally { }}

        public void actionTriggered(IComponent component, IRequestCycle cycle)
        {try { __CLOVER_183_0.M[909]++;
            __CLOVER_183_0.S[4301]++;invoke(cycle);
        } finally { }}

        public String toString()
        {try { __CLOVER_183_0.M[910]++;
            __CLOVER_183_0.S[4302]++;StringBuffer buffer = new StringBuffer("SyntheticListener[");

            __CLOVER_183_0.S[4303]++;buffer.append(_target);
            __CLOVER_183_0.S[4304]++;buffer.append(' ');
            __CLOVER_183_0.S[4305]++;buffer.append(_method);
            __CLOVER_183_0.S[4306]++;buffer.append(']');

            __CLOVER_183_0.S[4307]++;return buffer.toString();
        } finally { }}

    }

    public ListenerMap(Object target)
    {try { __CLOVER_183_0.M[911]++;
        __CLOVER_183_0.S[4308]++;_target = target;
    } finally { }}

    /**
     *  Gets a listener for the given name (which is both a property name
     *  and a method name).  The listener is created as needed, but is
     *  also cached for later use.
     *
     * @throws ApplicationRuntimeException if the listener can not be created.
     **/

    public synchronized Object getListener(String name)
    {try { __CLOVER_183_0.M[912]++;
        __CLOVER_183_0.S[4309]++;Object listener = null;

        __CLOVER_183_0.S[4310]++;listener = _listenerCache.get(name);

        __CLOVER_183_0.S[4311]++;if ((((listener == null) && (++__CLOVER_183_0.CT[793] != 0)) || (++__CLOVER_183_0.CF[793] == 0))){
        {
            __CLOVER_183_0.S[4312]++;listener = createListener(name);

            __CLOVER_183_0.S[4313]++;_listenerCache.put(name, listener);
        }}

        __CLOVER_183_0.S[4314]++;return listener;
    } finally { }}

    /**
     *  Returns an object that implements {@link IActionListener}.
     *  This involves looking up the method by name and determining which
     *  inner class to create.
     **/

    private synchronized Object createListener(String name)
    {try { __CLOVER_183_0.M[913]++;
        __CLOVER_183_0.S[4315]++;if ((((_methodMap == null) && (++__CLOVER_183_0.CT[794] != 0)) || (++__CLOVER_183_0.CF[794] == 0))){
            __CLOVER_183_0.S[4316]++;getMethodMap();}

        __CLOVER_183_0.S[4317]++;Method method = (Method) _methodMap.get(name);

        __CLOVER_183_0.S[4318]++;if ((((method == null) && (++__CLOVER_183_0.CT[795] != 0)) || (++__CLOVER_183_0.CF[795] == 0))){
            __CLOVER_183_0.S[4319]++;throw new ApplicationRuntimeException(
                Tapestry.format("ListenerMap.object-missing-method", _target, name));}

        __CLOVER_183_0.S[4320]++;return new SyntheticListener(method);
    } finally { }}

    /**
     *  Gets the method map for the current instance.  If necessary, it is constructed and cached (for other instances
     *  of the same class).
     *
     **/

    private synchronized Map getMethodMap()
    {try { __CLOVER_183_0.M[914]++;
        __CLOVER_183_0.S[4321]++;if ((((_methodMap != null) && (++__CLOVER_183_0.CT[796] != 0)) || (++__CLOVER_183_0.CF[796] == 0))){
            __CLOVER_183_0.S[4322]++;return _methodMap;}

        __CLOVER_183_0.S[4323]++;Class beanClass = _target.getClass();

        __CLOVER_183_0.S[4324]++;synchronized (_classMap)
        {
            __CLOVER_183_0.S[4325]++;_methodMap = (Map) _classMap.get(beanClass);

            __CLOVER_183_0.S[4326]++;if ((((_methodMap == null) && (++__CLOVER_183_0.CT[797] != 0)) || (++__CLOVER_183_0.CF[797] == 0))){
            {
                __CLOVER_183_0.S[4327]++;_methodMap = buildMethodMap(beanClass);

                __CLOVER_183_0.S[4328]++;_classMap.put(beanClass, _methodMap);
            }}
        }

        __CLOVER_183_0.S[4329]++;return _methodMap;
    } finally { }}

    private static Map buildMethodMap(Class beanClass)
    {try { __CLOVER_183_0.M[915]++;
        __CLOVER_183_0.S[4330]++;if ((((LOG.isDebugEnabled()) && (++__CLOVER_183_0.CT[798] != 0)) || (++__CLOVER_183_0.CF[798] == 0))){
            __CLOVER_183_0.S[4331]++;LOG.debug("Building method map for class " + beanClass.getName());}

        __CLOVER_183_0.S[4332]++;Map result = new HashMap();
        __CLOVER_183_0.S[4333]++;Method[] methods = beanClass.getMethods();

        __CLOVER_183_0.S[4334]++;for (int i = 0; (((i < methods.length) && (++__CLOVER_183_0.CT[799] != 0)) || (++__CLOVER_183_0.CF[799] == 0)); i++){
        {
            __CLOVER_183_0.S[4335]++;Method m = methods[i];
            __CLOVER_183_0.S[4336]++;int mods = m.getModifiers();

            __CLOVER_183_0.S[4337]++;if ((((Modifier.isStatic(mods)) && (++__CLOVER_183_0.CT[800] != 0)) || (++__CLOVER_183_0.CF[800] == 0))){
                __CLOVER_183_0.S[4338]++;continue;}

            // Probably not necessary, getMethods() returns only public
            // methods.

            __CLOVER_183_0.S[4339]++;if ((((!Modifier.isPublic(mods)) && (++__CLOVER_183_0.CT[801] != 0)) || (++__CLOVER_183_0.CF[801] == 0))){
                __CLOVER_183_0.S[4340]++;continue;}

            // Must return void

            __CLOVER_183_0.S[4341]++;if ((((m.getReturnType() != Void.TYPE) && (++__CLOVER_183_0.CT[802] != 0)) || (++__CLOVER_183_0.CF[802] == 0))){
                __CLOVER_183_0.S[4342]++;continue;}

            __CLOVER_183_0.S[4343]++;Class[] parmTypes = m.getParameterTypes();

            __CLOVER_183_0.S[4344]++;if ((((parmTypes.length != 1) && (++__CLOVER_183_0.CT[803] != 0)) || (++__CLOVER_183_0.CF[803] == 0))){
                __CLOVER_183_0.S[4345]++;continue;}

            // parm must be IRequestCycle

            __CLOVER_183_0.S[4346]++;if ((((!parmTypes[0].equals(IRequestCycle.class)) && (++__CLOVER_183_0.CT[804] != 0)) || (++__CLOVER_183_0.CF[804] == 0))){
                __CLOVER_183_0.S[4347]++;continue;}

            // Ha!  Passed all tests.

            __CLOVER_183_0.S[4348]++;result.put(m.getName(), m);
        }}

        __CLOVER_183_0.S[4349]++;return result;

    } finally { }}

    /**
     *  Invoked by the inner listener/adaptor classes to
     *  invoke the method.
     *
     **/

    private static void invokeTargetMethod(Object target, Method method, Object[] args)
    {try { __CLOVER_183_0.M[916]++;
        __CLOVER_183_0.S[4350]++;if ((((LOG.isDebugEnabled()) && (++__CLOVER_183_0.CT[805] != 0)) || (++__CLOVER_183_0.CF[805] == 0))){
            __CLOVER_183_0.S[4351]++;LOG.debug("Invoking listener method " + method + " on " + target);}

        __CLOVER_183_0.S[4352]++;try
        {
            __CLOVER_183_0.S[4353]++;try
            {
                __CLOVER_183_0.S[4354]++;method.invoke(target, args);
            }
            catch (InvocationTargetException ex)
            {
                __CLOVER_183_0.S[4355]++;Throwable inner = ex.getTargetException();

                __CLOVER_183_0.S[4356]++;if ((((inner instanceof ApplicationRuntimeException) && (++__CLOVER_183_0.CT[806] != 0)) || (++__CLOVER_183_0.CF[806] == 0))){
                    __CLOVER_183_0.S[4357]++;throw (ApplicationRuntimeException) inner;}

                // Edit out the InvocationTargetException, if possible.

                __CLOVER_183_0.S[4358]++;if ((((inner instanceof RuntimeException) && (++__CLOVER_183_0.CT[807] != 0)) || (++__CLOVER_183_0.CF[807] == 0))){
                    __CLOVER_183_0.S[4359]++;throw (RuntimeException) inner;}

                __CLOVER_183_0.S[4360]++;throw ex;
            }
        }
        catch (ApplicationRuntimeException ex)
        {
            __CLOVER_183_0.S[4361]++;throw ex;
        }
        catch (Exception ex)
        {
            // Catch InvocationTargetException or, preferrably,
            // the inner exception here (if its a runtime exception).

            __CLOVER_183_0.S[4362]++;throw new ApplicationRuntimeException(
                Tapestry.format("ListenerMap.unable-to-invoke-method", method.getName(), target, ex.getMessage()),
                ex);
        }
    } finally { }}

    /** 
     *  Returns an unmodifiable collection of the 
     *  names of the listeners implemented by the target class.
     *
     *  @since 1.0.6
     *
     **/

    public synchronized Collection getListenerNames()
    {try { __CLOVER_183_0.M[917]++;
         __CLOVER_183_0.S[4363]++;return Collections.unmodifiableCollection(getMethodMap().keySet());
    } finally { }}

    /**
     *  Returns true if this ListenerMap can provide a listener
     *  with the given name.
     * 
     *  @since 2.2
     * 
     **/

    public synchronized boolean canProvideListener(String name)
    {try { __CLOVER_183_0.M[918]++;
         __CLOVER_183_0.S[4364]++;return getMethodMap().containsKey(name);
    } finally { }}

    public String toString()
    {try { __CLOVER_183_0.M[919]++;
        __CLOVER_183_0.S[4365]++;return "ListenerMap[" + _target + "]";
    } finally { }}
}