/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.pageload;

import org.apache.hivemind.ClassResolver;
import org.apache.tapestry.IEngine;
import org.apache.tapestry.IPage;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.Tapestry;
import org.apache.tapestry.engine.IMonitor;
import org.apache.tapestry.engine.IPageLoader;
import org.apache.tapestry.engine.IPageSource;
import org.apache.tapestry.resolver.PageSpecificationResolver;
import org.apache.tapestry.services.ObjectPool;
import org.apache.tapestry.util.MultiKey;

/**
 *  A source for pages for a particular application.  Each application
 *  should have its own <code>PageSource</code>, storing it into the
 *  {@link javax.servlet.ServletContext} using a unique key (usually built from
 *  the application name).
 *
 *  <p>The <code>PageSource</code> acts as a pool for {@link IPage} instances.
 *  Pages are retrieved from the pool using {@link #getPage(IRequestCycle, String, IMonitor)}
 *  and are later returned to the pool using {@link #releasePage(IPage)}.
 *
 *
 *  <p>TBD: Pooled pages stay forever.  Need a strategy for cleaning up the pool,
 *  tracking which pages have been in the pool the longest, etc.  A mechanism
 *  for reporting pool statistics would be useful.
 *
 *  @author Howard Lewis Ship
 * 
 */

public class PageSource implements IPageSource
{public static com.cortexeb.tools.clover.d __CLOVER_195_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    /** set by container */
    private ClassResolver _classResolver;

    /** @since 3.1 */
    private PageSpecificationResolver _pageSpecificationResolver;
    
    /** @since 3.1 */
    
    private IPageLoader _loader;

    /**
     *  The pool of {@link IPage}s.  The key is a {@link MultiKey},
     *  built from the page name and the page locale.  This is a reference
     *  to a shared pool.
     * 
     *  @see IEngine#getPool()
     *
     */

    private ObjectPool _pool;

    public ClassResolver getClassResolver()
    {try { __CLOVER_195_0.M[982]++;
        __CLOVER_195_0.S[4687]++;return _classResolver;
    } finally { }}

    /**
     *  Builds a key for a named page in the application's current locale.
     *
     */

    protected MultiKey buildKey(IEngine engine, String pageName)
    {try { __CLOVER_195_0.M[983]++;
        __CLOVER_195_0.S[4688]++;Object[] keys;

        __CLOVER_195_0.S[4689]++;keys = new Object[] { pageName, engine.getLocale()};

        // Don't make a copy, this array is just for the MultiKey.

        __CLOVER_195_0.S[4690]++;return new MultiKey(keys, false);
    } finally { }}

    /**
     *  Builds a key from an existing page, using the page's name and locale.  This is
     *  used when storing a page into the pool.
     *
     */

    protected MultiKey buildKey(IPage page)
    {try { __CLOVER_195_0.M[984]++;
        __CLOVER_195_0.S[4691]++;Object[] keys;

        __CLOVER_195_0.S[4692]++;keys = new Object[] { page.getPageName(), page.getLocale()};

        // Don't make a copy, this array is just for the MultiKey.

        __CLOVER_195_0.S[4693]++;return new MultiKey(keys, false);
    } finally { }}

    /**
     *  Gets the page from a pool, or otherwise loads the page.  This operation
     *  is threadsafe.
     *
     */

    public IPage getPage(IRequestCycle cycle, String pageName, IMonitor monitor)
    {try { __CLOVER_195_0.M[985]++;
        __CLOVER_195_0.S[4694]++;IEngine engine = cycle.getEngine();
        __CLOVER_195_0.S[4695]++;Object key = buildKey(engine, pageName);
        __CLOVER_195_0.S[4696]++;IPage result = (IPage) _pool.get(key);

        __CLOVER_195_0.S[4697]++;if ((((result == null) && (++__CLOVER_195_0.CT[855] != 0)) || (++__CLOVER_195_0.CF[855] == 0))){
        {
            __CLOVER_195_0.S[4698]++;monitor.pageCreateBegin(pageName);

            __CLOVER_195_0.S[4699]++;_pageSpecificationResolver.resolve(cycle, pageName);

            __CLOVER_195_0.S[4700]++;result =
                _loader.loadPage(
                    _pageSpecificationResolver.getSimplePageName(),
                    _pageSpecificationResolver.getNamespace(),
                    cycle,
                    _pageSpecificationResolver.getSpecification());

            __CLOVER_195_0.S[4701]++;monitor.pageCreateEnd(pageName);
        }}
        else{
        {
            // The page loader attaches the engine, but a page from
            // the pool needs to be explicitly attached.

            __CLOVER_195_0.S[4702]++;result.attach(engine);
        }}

        __CLOVER_195_0.S[4703]++;return result;
    } finally { }}

    /**
     *  Returns the page to the appropriate pool.  Invokes
     *  {@link IPage#detach()}.
     *
     */

    public void releasePage(IPage page)
    {try { __CLOVER_195_0.M[986]++;
        __CLOVER_195_0.S[4704]++;Tapestry.clearMethodInvocations();

        __CLOVER_195_0.S[4705]++;page.detach();

        __CLOVER_195_0.S[4706]++;Tapestry.checkMethodInvocation(Tapestry.ABSTRACTPAGE_DETACH_METHOD_ID, "detach()", page);

        __CLOVER_195_0.S[4707]++;_pool.store(buildKey(page), page);
    } finally { }}

    /** @since 3.1 */

    public void setPool(ObjectPool pool)
    {try { __CLOVER_195_0.M[987]++;
        __CLOVER_195_0.S[4708]++;_pool = pool;
    } finally { }}

    /** @since 3.1 */

    public void setClassResolver(ClassResolver resolver)
    {try { __CLOVER_195_0.M[988]++;
        __CLOVER_195_0.S[4709]++;_classResolver = resolver;
    } finally { }}

    /** @since 3.1 */

    public void setPageSpecificationResolver(PageSpecificationResolver resolver)
    {try { __CLOVER_195_0.M[989]++;
        __CLOVER_195_0.S[4710]++;_pageSpecificationResolver = resolver;
    } finally { }}

	/** @since 3.1 */
	
    public void setLoader(IPageLoader loader)
    {try { __CLOVER_195_0.M[990]++;
        __CLOVER_195_0.S[4711]++;_loader = loader;
    } finally { }}

}