/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.param;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import ognl.PropertyAccessor;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.hivemind.ClassResolver;
import org.apache.hivemind.util.PropertyAdaptor;
import org.apache.hivemind.util.PropertyUtils;
import org.apache.tapestry.BindingException;
import org.apache.tapestry.IBinding;
import org.apache.tapestry.IComponent;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.Tapestry;
import org.apache.tapestry.enhance.JavaTypeUtils;
import org.apache.tapestry.spec.Direction;
import org.apache.tapestry.spec.IComponentSpecification;
import org.apache.tapestry.spec.IParameterSpecification;

/**
 * Manages a set of {@link IParameterConnector}s for a {@link IComponent}.
 * 
 * @author Howard Lewis Ship
 * @since 2.0.3
 */

public class ParameterManager
{public static com.cortexeb.tools.clover.d __CLOVER_210_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private static final Log LOG = LogFactory.getLog(ParameterManager.class);

    /**
     * Special types that aren't resolved by class lookups, including scalars, arrays of scalars,
     * etc.
     * <p>
     * There's some overlap here with ComponentClassFactory.
     */

    private static final Map SPECIAL_TYPE_MAP = new HashMap();

    static
    {
        __CLOVER_210_0.S[4841]++;SPECIAL_TYPE_MAP.put("boolean", boolean.class);
        __CLOVER_210_0.S[4842]++;SPECIAL_TYPE_MAP.put("boolean[]", boolean[].class);
        __CLOVER_210_0.S[4843]++;SPECIAL_TYPE_MAP.put("byte", byte.class);
        __CLOVER_210_0.S[4844]++;SPECIAL_TYPE_MAP.put("byte[]", byte[].class);
        __CLOVER_210_0.S[4845]++;SPECIAL_TYPE_MAP.put("char", char.class);
        __CLOVER_210_0.S[4846]++;SPECIAL_TYPE_MAP.put("char[]", char[].class);
        __CLOVER_210_0.S[4847]++;SPECIAL_TYPE_MAP.put("short", short.class);
        __CLOVER_210_0.S[4848]++;SPECIAL_TYPE_MAP.put("short[]", short[].class);
        __CLOVER_210_0.S[4849]++;SPECIAL_TYPE_MAP.put("int", int.class);
        __CLOVER_210_0.S[4850]++;SPECIAL_TYPE_MAP.put("int[]", int[].class);
        __CLOVER_210_0.S[4851]++;SPECIAL_TYPE_MAP.put("long", long.class);
        __CLOVER_210_0.S[4852]++;SPECIAL_TYPE_MAP.put("long[]", long[].class);
        __CLOVER_210_0.S[4853]++;SPECIAL_TYPE_MAP.put("float", float.class);
        __CLOVER_210_0.S[4854]++;SPECIAL_TYPE_MAP.put("float[]", float[].class);
        __CLOVER_210_0.S[4855]++;SPECIAL_TYPE_MAP.put("double", double.class);
        __CLOVER_210_0.S[4856]++;SPECIAL_TYPE_MAP.put("double[]", double[].class);

        __CLOVER_210_0.S[4857]++;SPECIAL_TYPE_MAP.put("java.lang.Object[]", Object[].class);
        __CLOVER_210_0.S[4858]++;SPECIAL_TYPE_MAP.put("java.lang.String[]", String[].class);
    }

    private IComponent _component;

    private IParameterConnector[] _connectors;

    public ParameterManager(IComponent component)
    {try { __CLOVER_210_0.M[1035]++;
        __CLOVER_210_0.S[4859]++;_component = component;
    } finally { }}

    /**
     * Invoked just before a component renders. Converts bindings to values that are assigned to
     * connected properties.
     */

    public void setParameters(IRequestCycle cycle)
    {try { __CLOVER_210_0.M[1036]++;
        __CLOVER_210_0.S[4860]++;if ((((_connectors == null) && (++__CLOVER_210_0.CT[877] != 0)) || (++__CLOVER_210_0.CF[877] == 0))){
            __CLOVER_210_0.S[4861]++;setup(cycle);}

        __CLOVER_210_0.S[4862]++;for (int i = 0; (((i < _connectors.length) && (++__CLOVER_210_0.CT[878] != 0)) || (++__CLOVER_210_0.CF[878] == 0)); i++){
            __CLOVER_210_0.S[4863]++;_connectors[i].setParameter(cycle);}
    } finally { }}

    /**
     * Invoked just after the component renders. Returns component properties back to initial values
     * (unless the corresponding binding is {@link IBinding#isInvariant() invariant}). In addition,
     * for {@link Direction#FORM}parameters, the property is read and the binding is set from the
     * property value (if the cycle is rewinding and the current form is rewinding).
     */

    public void resetParameters(IRequestCycle cycle)
    {try { __CLOVER_210_0.M[1037]++;
        __CLOVER_210_0.S[4864]++;if ((((_connectors == null) && (++__CLOVER_210_0.CT[879] != 0)) || (++__CLOVER_210_0.CF[879] == 0))){
            __CLOVER_210_0.S[4865]++;return;}

        __CLOVER_210_0.S[4866]++;for (int i = 0; (((i < _connectors.length) && (++__CLOVER_210_0.CT[880] != 0)) || (++__CLOVER_210_0.CF[880] == 0)); i++){
            __CLOVER_210_0.S[4867]++;_connectors[i].resetParameter(cycle);}
    } finally { }}

    private void setup(IRequestCycle cycle)
    {try { __CLOVER_210_0.M[1038]++;
        __CLOVER_210_0.S[4868]++;boolean debug = LOG.isDebugEnabled();

        __CLOVER_210_0.S[4869]++;if ((((debug) && (++__CLOVER_210_0.CT[881] != 0)) || (++__CLOVER_210_0.CF[881] == 0))){
            __CLOVER_210_0.S[4870]++;LOG.debug(_component + ": connecting parameters and properties");}

        __CLOVER_210_0.S[4871]++;List list = new ArrayList();
        __CLOVER_210_0.S[4872]++;IComponentSpecification spec = _component.getSpecification();
        __CLOVER_210_0.S[4873]++;ClassResolver resolver = _component.getPage().getEngine().getClassResolver();

        __CLOVER_210_0.S[4874]++;IParameterConnector disabledConnector = null;

        __CLOVER_210_0.S[4875]++;Collection names = spec.getParameterNames();
        __CLOVER_210_0.S[4876]++;Iterator i = names.iterator();
        __CLOVER_210_0.S[4877]++;while ((((i.hasNext()) && (++__CLOVER_210_0.CT[882] != 0)) || (++__CLOVER_210_0.CF[882] == 0))){
        {
            __CLOVER_210_0.S[4878]++;String name = (String) i.next();

            __CLOVER_210_0.S[4879]++;if ((((debug) && (++__CLOVER_210_0.CT[883] != 0)) || (++__CLOVER_210_0.CF[883] == 0))){
                __CLOVER_210_0.S[4880]++;LOG.debug("Connecting parameter " + name + ".");}

            __CLOVER_210_0.S[4881]++;IBinding binding = _component.getBinding(name);
            __CLOVER_210_0.S[4882]++;if ((((binding == null) && (++__CLOVER_210_0.CT[884] != 0)) || (++__CLOVER_210_0.CF[884] == 0))){
            {
                __CLOVER_210_0.S[4883]++;if ((((debug) && (++__CLOVER_210_0.CT[885] != 0)) || (++__CLOVER_210_0.CF[885] == 0))){
                    __CLOVER_210_0.S[4884]++;LOG.debug("Not bound.");}

                __CLOVER_210_0.S[4885]++;continue;
            }}

            __CLOVER_210_0.S[4886]++;IParameterSpecification pspec = spec.getParameter(name);
            __CLOVER_210_0.S[4887]++;Direction direction = pspec.getDirection();

            __CLOVER_210_0.S[4888]++;if ((((direction != Direction.IN && direction != Direction.FORM) && (++__CLOVER_210_0.CT[886] != 0)) || (++__CLOVER_210_0.CF[886] == 0))){
            {
                __CLOVER_210_0.S[4889]++;if ((((debug) && (++__CLOVER_210_0.CT[887] != 0)) || (++__CLOVER_210_0.CF[887] == 0))){
                    __CLOVER_210_0.S[4890]++;LOG.debug("Parameter is " + pspec.getDirection().getName() + ".");}

                __CLOVER_210_0.S[4891]++;continue;
            }}

            __CLOVER_210_0.S[4892]++;if ((((!direction.getAllowInvariant() && binding.isInvariant()) && (++__CLOVER_210_0.CT[888] != 0)) || (++__CLOVER_210_0.CF[888] == 0))){
                __CLOVER_210_0.S[4893]++;throw new ConnectedParameterException(Tapestry.format(
                        "ParameterManager.incompatible-direction-and-binding",
                        new Object[]
                        { name, _component.getExtendedId(), direction.getDisplayName(), binding }),
                        _component, name, null, binding.getLocation(), null);}

            __CLOVER_210_0.S[4894]++;String propertyName = pspec.getPropertyName();

            __CLOVER_210_0.S[4895]++;if ((((debug && !name.equals(propertyName)) && (++__CLOVER_210_0.CT[889] != 0)) || (++__CLOVER_210_0.CF[889] == 0))){
                __CLOVER_210_0.S[4896]++;LOG.debug("Connecting to property " + propertyName + ".");}

            // Next,verify that there is a writable property with the same
            // name as the parameter.

            __CLOVER_210_0.S[4897]++;PropertyAdaptor adaptor = null;

            __CLOVER_210_0.S[4898]++;try
            {
                __CLOVER_210_0.S[4899]++;adaptor = PropertyUtils.getPropertyAdaptor(_component, propertyName);
            }
            catch (Exception ex)
            {
                __CLOVER_210_0.S[4900]++;throw new ConnectedParameterException(ex.getMessage(), _component, name,
                        propertyName, binding.getLocation(), ex);
            }

            __CLOVER_210_0.S[4901]++;if ((((!(adaptor.isReadable() && adaptor.isWritable())) && (++__CLOVER_210_0.CT[890] != 0)) || (++__CLOVER_210_0.CF[890] == 0))){
            {
                __CLOVER_210_0.S[4902]++;throw new ConnectedParameterException(Tapestry.format(
                        "ParameterManager.property-not-read-write",
                        _component.getExtendedId(),
                        propertyName), _component, name, propertyName, binding.getLocation(), null);
            }}

            // Check if the parameter type matches the property type

            __CLOVER_210_0.S[4903]++;Class propertyType = adaptor.getPropertyType();
            __CLOVER_210_0.S[4904]++;Class parameterType = getType(pspec.getType(), resolver);

            __CLOVER_210_0.S[4905]++;if ((((parameterType == null) && (++__CLOVER_210_0.CT[891] != 0)) || (++__CLOVER_210_0.CF[891] == 0))){
            {
                __CLOVER_210_0.S[4906]++;throw new ConnectedParameterException(Tapestry.format(
                        "ParameterManager.java-type-not-specified",
                        name,
                        _component.getExtendedId()), _component, name, propertyName, binding
                        .getLocation(), null);
            }}

            __CLOVER_210_0.S[4907]++;if ((((!propertyType.equals(parameterType)) && (++__CLOVER_210_0.CT[892] != 0)) || (++__CLOVER_210_0.CF[892] == 0))){
            {
                __CLOVER_210_0.S[4908]++;throw new ConnectedParameterException(Tapestry.format(
                        "ParameterManager.type-mismatch",
                        new String[]
                        { name, _component.getExtendedId(), parameterType.toString(),
                                propertyType.toString() }), _component, name, propertyName, binding
                        .getLocation(), null);
            }}

            // Here's where we will sniff it for type, for the moment
            // assume its some form of object (not scalar) type.

            __CLOVER_210_0.S[4909]++;IParameterConnector connector = createConnector(
                    _component,
                    name,
                    binding,
                    propertyType,
                    parameterType);

            // Static bindings are set here and then forgotten
            // about. Dynamic bindings are kept for later.

            __CLOVER_210_0.S[4910]++;if ((((binding.isInvariant()) && (++__CLOVER_210_0.CT[893] != 0)) || (++__CLOVER_210_0.CF[893] == 0))){
            {
                __CLOVER_210_0.S[4911]++;if ((((debug) && (++__CLOVER_210_0.CT[894] != 0)) || (++__CLOVER_210_0.CF[894] == 0))){
                    __CLOVER_210_0.S[4912]++;LOG.debug("Setting invariant value using " + connector + ".");}

                __CLOVER_210_0.S[4913]++;try
                {
                    __CLOVER_210_0.S[4914]++;connector.setParameter(cycle);
                }
                catch (BindingException ex)
                {
                    __CLOVER_210_0.S[4915]++;throw new ConnectedParameterException(Tapestry.format(
                            "ParameterManager.static-initialization-failure",
                            propertyName,
                            _component.getExtendedId(),
                            binding.toString()), _component, name, propertyName, ex);
                }

                __CLOVER_210_0.S[4916]++;continue;
            }}

            __CLOVER_210_0.S[4917]++;if ((((debug) && (++__CLOVER_210_0.CT[895] != 0)) || (++__CLOVER_210_0.CF[895] == 0))){
                __CLOVER_210_0.S[4918]++;LOG.debug("Adding " + connector + ".");}

            // To properly support forms elements, the disabled parameter
            // must always be processed last.

            __CLOVER_210_0.S[4919]++;if ((((name.equals("disabled")) && (++__CLOVER_210_0.CT[896] != 0)) || (++__CLOVER_210_0.CF[896] == 0))){
                __CLOVER_210_0.S[4920]++;disabledConnector = connector;}
            else{
                __CLOVER_210_0.S[4921]++;list.add(connector);}

        }}

        __CLOVER_210_0.S[4922]++;if ((((disabledConnector != null) && (++__CLOVER_210_0.CT[897] != 0)) || (++__CLOVER_210_0.CF[897] == 0))){
            __CLOVER_210_0.S[4923]++;list.add(disabledConnector);}

        // Convert for List to array

        __CLOVER_210_0.S[4924]++;_connectors = (IParameterConnector[]) list.toArray(new IParameterConnector[list.size()]);

    } finally { }}

    private IParameterConnector createConnector(IComponent component, String parameterName,
            IBinding binding, Class propertyType, Class requiredType)
    {try { __CLOVER_210_0.M[1039]++;
        // Could convert this code to use a Decorator, but then I'd need
        // some kind of factory for these parameter connectors.

        __CLOVER_210_0.S[4925]++;if ((((propertyType.equals(Boolean.TYPE)) && (++__CLOVER_210_0.CT[898] != 0)) || (++__CLOVER_210_0.CF[898] == 0))){
            __CLOVER_210_0.S[4926]++;return new BooleanParameterConnector(component, parameterName, binding);}

        __CLOVER_210_0.S[4927]++;if ((((propertyType.equals(Integer.TYPE)) && (++__CLOVER_210_0.CT[899] != 0)) || (++__CLOVER_210_0.CF[899] == 0))){
            __CLOVER_210_0.S[4928]++;return new IntParameterConnector(component, parameterName, binding);}

        __CLOVER_210_0.S[4929]++;if ((((propertyType.equals(Double.TYPE)) && (++__CLOVER_210_0.CT[900] != 0)) || (++__CLOVER_210_0.CF[900] == 0))){
            __CLOVER_210_0.S[4930]++;return new DoubleParameterConnector(component, parameterName, binding);}

        __CLOVER_210_0.S[4931]++;if ((((propertyType.equals(String.class)) && (++__CLOVER_210_0.CT[901] != 0)) || (++__CLOVER_210_0.CF[901] == 0))){
            __CLOVER_210_0.S[4932]++;return new StringParameterConnector(component, parameterName, binding);}

        // The default is for any kind of object type

        __CLOVER_210_0.S[4933]++;return new ObjectParameterConnector(component, parameterName, binding, requiredType);
    } finally { }}

    private Class getType(String name, ClassResolver resolver)
    {try { __CLOVER_210_0.M[1040]++;
        __CLOVER_210_0.S[4934]++;if ((((Tapestry.isBlank(name)) && (++__CLOVER_210_0.CT[902] != 0)) || (++__CLOVER_210_0.CF[902] == 0))){
            __CLOVER_210_0.S[4935]++;return null;}

        __CLOVER_210_0.S[4936]++;Class result = (Class) SPECIAL_TYPE_MAP.get(name);

        __CLOVER_210_0.S[4937]++;if ((((result != null) && (++__CLOVER_210_0.CT[903] != 0)) || (++__CLOVER_210_0.CF[903] == 0))){
            __CLOVER_210_0.S[4938]++;return result;}

        // Convert array references into the format needed by the resolver.
        
        __CLOVER_210_0.S[4939]++;return resolver.findClass(JavaTypeUtils.getJVMClassName(name));
    } finally { }}
}