/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.parse;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.hivemind.Location;

/**
 * Token representing the open tag for a component. Components may be either specified or implicit.
 * Specified components (the traditional type, dating back to the origin of Tapestry) are matched by
 * an entry in the containing component's specification. Implicit components specify their type in
 * the component template and must not have an entry in the containing component's specification.
 * 
 * @see TokenType#OPEN
 * @author Howard Lewis Ship
 * @since 3.0
 */

public class OpenToken extends TemplateToken
{public static com.cortexeb.tools.clover.d __CLOVER_222_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private String _tag;

    private String _id;

    private String _componentType;

    private Map _attributes;

    /**
     * Creates a new token with the given tag, id and type
     * 
     * @param tag
     *            the template tag which represents the component, typically "span"
     * @param id
     *            the id for the component, which may be assigned by the template parser for
     *            implicit components
     * @param componentType
     *            the type of component, if an implicit component, or null for a specified component
     * @param location
     *            location of tag represented by this token
     */

    public OpenToken(String tag, String id, String componentType, Location location)
    {
        super(TokenType.OPEN, location);__CLOVER_222_0.S[4981]++;try { __CLOVER_222_0.M[1066]++;

        __CLOVER_222_0.S[4982]++;_tag = tag;
        __CLOVER_222_0.S[4983]++;_id = id;
        __CLOVER_222_0.S[4984]++;_componentType = componentType;
    } finally { }}

    /**
     * Returns the id for the component.
     */

    public String getId()
    {try { __CLOVER_222_0.M[1067]++;
        __CLOVER_222_0.S[4985]++;return _id;
    } finally { }}

    /**
     * Returns the tag used to represent the component within the template.
     */

    public String getTag()
    {try { __CLOVER_222_0.M[1068]++;
        __CLOVER_222_0.S[4986]++;return _tag;
    } finally { }}

    /**
     * Returns the specified component type, or null for a component where the type is not defined
     * in the template. The type may include a library id prefix.
     */

    public String getComponentType()
    {try { __CLOVER_222_0.M[1069]++;
        __CLOVER_222_0.S[4987]++;return _componentType;
    } finally { }}

    public void addAttribute(String name, String value)
    {try { __CLOVER_222_0.M[1070]++;
        __CLOVER_222_0.S[4988]++;if ((((_attributes == null) && (++__CLOVER_222_0.CT[905] != 0)) || (++__CLOVER_222_0.CF[905] == 0))){
            __CLOVER_222_0.S[4989]++;_attributes = new HashMap();}

        __CLOVER_222_0.S[4990]++;_attributes.put(name, value);
    } finally { }}

    /**
     * Returns a Map of attributes. Keys and values are strings.
     */

    public Map getAttributesMap()
    {try { __CLOVER_222_0.M[1071]++;
        __CLOVER_222_0.S[4991]++;return _attributes;
    } finally { }}

    protected void extendDescription(ToStringBuilder builder)
    {try { __CLOVER_222_0.M[1072]++;
        __CLOVER_222_0.S[4992]++;builder.append("id", _id);
        __CLOVER_222_0.S[4993]++;builder.append("componentType", _componentType);
        __CLOVER_222_0.S[4994]++;builder.append("tag", _tag);
        __CLOVER_222_0.S[4995]++;builder.append("attributes", _attributes);
    } finally { }}

}