/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.record;

import java.util.Collection;
import java.util.Iterator;

import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.tapestry.IComponent;
import org.apache.tapestry.IPage;
import org.apache.tapestry.Tapestry;
import org.apache.tapestry.engine.IPageRecorder;
import org.apache.tapestry.event.ObservedChangeEvent;
import org.apache.tapestry.util.prop.OgnlUtils;

/**
 *  Tracks changes to components on a page, allowing changes to be persisted across
 *  request cycles, and restoring the state of a page and component when needed.
 *
 *  <p>This is an abstract implementation; specific implementations can choose where
 *  and how to persist the data.
 *
 * @author Howard Ship
 * 
 **/

public abstract class PageRecorder implements IPageRecorder
{public static com.cortexeb.tools.clover.d __CLOVER_235_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private boolean _dirty = false;
    private boolean _locked = false;
    private boolean _discard = false;

    /**
     *  Invoked to persist all changes that have been accumulated.  If the recorder
     *  saves change incrementally, this should ensure that all changes have been persisted.
     *
     *  <p>Subclasses should check the dirty flag.  If the recorder is dirty, changes
     *  should be recorded and the dirty flag cleared.
     *
     **/

    public abstract void commit();

    /**
     *  Returns a <code>Collection</code> of 
     *  {@link IPageChange} objects
     *  identifying changes to the page and its components.
     *
     **/

    public abstract Collection getChanges();

    /**
     *  Returns true if the page has observed a change.
     *  The dirty flag is cleared by
     *  {@link #commit()}.
     *
     **/

    public boolean isDirty()
    {try { __CLOVER_235_0.M[1239]++;
        __CLOVER_235_0.S[6026]++;return _dirty;
    } finally { }}

    /**
     *  Returns true if the recorder is locked.  The locked flag
     *  is set by {@link #commit()}.
     *
     **/

    public boolean isLocked()
    {try { __CLOVER_235_0.M[1240]++;
        __CLOVER_235_0.S[6027]++;return _locked;
    } finally { }}

    public void setLocked(boolean value)
    {try { __CLOVER_235_0.M[1241]++;
        __CLOVER_235_0.S[6028]++;_locked = value;
    } finally { }}

    /**
     *  Observes the change.  The object of the event is expected to
     *  be an {@link IComponent}.  Ignores the change if not active,
     *  otherwise, sets invokes {@link #recordChange(String, String,
     *  Object)}.
     *
     *  <p>If the property name in the event is null, then the recorder
     *  is marked dirty (but 
     *  {@link #recordChange(String, String,
     *  Object)} is not invoked.  This is how a "distant" property changes
     *  are propogated to the page recorder (a distant property change is a change to
     *  a property of an object that is itself a property of the page).
     *
     *  <p>If the recorder is not active (typically, when a page is
     *  being rewound), then the event is simply ignored.
     *
     **/

    public void observeChange(ObservedChangeEvent event)
    {try { __CLOVER_235_0.M[1242]++;
        __CLOVER_235_0.S[6029]++;IComponent component = event.getComponent();
        __CLOVER_235_0.S[6030]++;String propertyName = event.getPropertyName();

        __CLOVER_235_0.S[6031]++;if ((((_locked) && (++__CLOVER_235_0.CT[1064] != 0)) || (++__CLOVER_235_0.CF[1064] == 0))){
            __CLOVER_235_0.S[6032]++;throw new ApplicationRuntimeException(
                Tapestry.format(
                    "PageRecorder.change-after-lock",
                    component.getPage().getPageName(),
                    propertyName,
                    component.getExtendedId()));}

        __CLOVER_235_0.S[6033]++;if ((((propertyName == null) && (++__CLOVER_235_0.CT[1065] != 0)) || (++__CLOVER_235_0.CF[1065] == 0))){
            __CLOVER_235_0.S[6034]++;throw new ApplicationRuntimeException(
                Tapestry.format("PageRecorder.null-property-name", component.getExtendedId()));}

        __CLOVER_235_0.S[6035]++;Object activeValue = event.getNewValue();

        __CLOVER_235_0.S[6036]++;try
        {
            __CLOVER_235_0.S[6037]++;recordChange(component.getIdPath(), propertyName, activeValue);
        }
        catch (Throwable t)
        {
            __CLOVER_235_0.S[6038]++;t.printStackTrace();
            __CLOVER_235_0.S[6039]++;throw new ApplicationRuntimeException(
                Tapestry.format(
                    "PageRecorder.unable-to-persist",
                    propertyName,
                    component.getExtendedId(),
                    activeValue),
                t);
        }
    } finally { }}

    /**
     *  Records a change to a particular component.  Subclasses may
     *  cache these in memory, or record them externally at this time.
     *
     *  <p>This method is responsible for setting the dirty flag if
     *  the described change is real.
     *
     *  @param componentPath the name of the component relative to the
     *  page which contains it.  May be null if the change was to a
     *  property of the page itself.
     *
     *  @param propertyName the name of the property which changed.
     *
     *  @param newValue the new value for the property, which may also
     *  be null.
     *
     *  @see IComponent#getIdPath()
     *
     **/

    protected abstract void recordChange(
        String componentPath,
        String propertyName,
        Object newValue);

    /**
     *  Rolls back the page to the currently persisted state.
     *
     **/

    public void rollback(IPage page)
    {try { __CLOVER_235_0.M[1243]++;
        __CLOVER_235_0.S[6040]++;Collection changes = getChanges();

        __CLOVER_235_0.S[6041]++;if ((((changes.isEmpty()) && (++__CLOVER_235_0.CT[1066] != 0)) || (++__CLOVER_235_0.CF[1066] == 0))){
            __CLOVER_235_0.S[6042]++;return;}

        __CLOVER_235_0.S[6043]++;Iterator i = changes.iterator();

        __CLOVER_235_0.S[6044]++;while ((((i.hasNext()) && (++__CLOVER_235_0.CT[1067] != 0)) || (++__CLOVER_235_0.CF[1067] == 0))){
        {
            __CLOVER_235_0.S[6045]++;PageChange change = (PageChange) i.next();

            __CLOVER_235_0.S[6046]++;String propertyName = change.getPropertyName();

            __CLOVER_235_0.S[6047]++;IComponent component = page.getNestedComponent(change.getComponentPath());

            __CLOVER_235_0.S[6048]++;Object storedValue = change.getNewValue();

            __CLOVER_235_0.S[6049]++;try
            {
                __CLOVER_235_0.S[6050]++;OgnlUtils.set(propertyName, component, storedValue);
            }
            catch (Throwable t)
            {
                __CLOVER_235_0.S[6051]++;throw new ApplicationRuntimeException(
                    Tapestry.format(
                        "PageRecorder.unable-to-rollback",
                        new Object[] { propertyName, component, storedValue, t.getMessage()}),
                    t);
            }
        }}
    } finally { }}

    /** @since 2.0.2 **/

    public boolean isMarkedForDiscard()
    {try { __CLOVER_235_0.M[1244]++;
        __CLOVER_235_0.S[6052]++;return _discard;
    } finally { }}

    /** @since 2.0.2 **/

    public void markForDiscard()
    {try { __CLOVER_235_0.M[1245]++;
        __CLOVER_235_0.S[6053]++;_discard = true;
    } finally { }}

    protected void setDirty(boolean dirty)
    {try { __CLOVER_235_0.M[1246]++;
        __CLOVER_235_0.S[6054]++;_dirty = dirty;
    } finally { }}

    protected boolean getDirty()
    {try { __CLOVER_235_0.M[1247]++;
        __CLOVER_235_0.S[6055]++;return _dirty;
    } finally { }}

}