/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.record;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.servlet.http.HttpSession;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.Tapestry;
import org.apache.tapestry.request.RequestContext;
import org.apache.tapestry.util.StringSplitter;

/**
 *  Simple implementation of {@link org.apache.tapestry.engine.IPageRecorder}
 *  that stores page changes as {@link javax.servlet.http.HttpSession} attributes.
 *
 *
 *  @author Howard Ship
 * 
 **/

public class SessionPageRecorder extends PageRecorder
{public static com.cortexeb.tools.clover.d __CLOVER_236_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private static final Log LOG = LogFactory.getLog(SessionPageRecorder.class);

    /**
     *  Dictionary of changes, keyed on an instance of 
     *  {@link ChangeKey}
     *  (which enapsulates component path and property name).  The
     *  value is the new value for the object.
     *  The same information is stored into the
     *  {@link HttpSession}, which is  used as a kind of
     *  write-behind cache.
     *
     **/

    private Map _changes;

    /**
     *  The session into which changes are recorded.
     * 
     *  @since 3.0
     * 
     **/

    private HttpSession _session;

    /**
     *  The fully qualified name of the page being recorded.
     * 
     *  @since 3.0
     * 
     **/

    private String _pageName;

    /**
     *  The prefix (for {@link HttpSession} attributes) used by this
     *  page recorder.
     * 
     **/

    private String _attributePrefix;

    public void initialize(String pageName, IRequestCycle cycle)
    {try { __CLOVER_236_0.M[1248]++;
        __CLOVER_236_0.S[6056]++;if ((((LOG.isDebugEnabled()) && (++__CLOVER_236_0.CT[1068] != 0)) || (++__CLOVER_236_0.CF[1068] == 0))){
            __CLOVER_236_0.S[6057]++;LOG.debug("Initializing for " + pageName);}

        __CLOVER_236_0.S[6058]++;RequestContext context = cycle.getRequestContext();

        __CLOVER_236_0.S[6059]++;_pageName = pageName;
        __CLOVER_236_0.S[6060]++;_session = context.getSession();

        __CLOVER_236_0.S[6061]++;_attributePrefix = context.getServlet().getServletName() + "/" + _pageName + "/";

        __CLOVER_236_0.S[6062]++;restorePageChanges();
    } finally { }}

    public void discard()
    {try { __CLOVER_236_0.M[1249]++;
        __CLOVER_236_0.S[6063]++;if ((((Tapestry.isEmpty(_changes)) && (++__CLOVER_236_0.CT[1069] != 0)) || (++__CLOVER_236_0.CF[1069] == 0))){
            __CLOVER_236_0.S[6064]++;return;}

        __CLOVER_236_0.S[6065]++;Iterator i = _changes.keySet().iterator();

        __CLOVER_236_0.S[6066]++;while ((((i.hasNext()) && (++__CLOVER_236_0.CT[1070] != 0)) || (++__CLOVER_236_0.CF[1070] == 0))){
        {
            __CLOVER_236_0.S[6067]++;ChangeKey key = (ChangeKey) i.next();

            __CLOVER_236_0.S[6068]++;String attributeKey =
                constructAttributeKey(key.getComponentPath(), key.getPropertyName());

            __CLOVER_236_0.S[6069]++;if ((((LOG.isDebugEnabled()) && (++__CLOVER_236_0.CT[1071] != 0)) || (++__CLOVER_236_0.CF[1071] == 0))){
                __CLOVER_236_0.S[6070]++;LOG.debug("Removing session attribute " + attributeKey);}

            __CLOVER_236_0.S[6071]++;_session.removeAttribute(attributeKey);
        }}
    } finally { }}

    /**
     *  Simply clears the dirty flag, because there is no external place
     *  to store changed page properties.  Sets the locked flag to prevent
     *  subsequent changes from occuring now.
     *
     **/

    public void commit()
    {try { __CLOVER_236_0.M[1250]++;
        __CLOVER_236_0.S[6072]++;setDirty(false);
        __CLOVER_236_0.S[6073]++;setLocked(true);
    } finally { }}

    /**
     *  Returns true if the recorder has any changes recorded.
     *
     **/

    public boolean getHasChanges()
    {try { __CLOVER_236_0.M[1251]++;
        __CLOVER_236_0.S[6074]++;if ((((_changes == null) && (++__CLOVER_236_0.CT[1072] != 0)) || (++__CLOVER_236_0.CF[1072] == 0))){
            __CLOVER_236_0.S[6075]++;return false;}

        __CLOVER_236_0.S[6076]++;return (_changes.size() > 0);
    } finally { }}

    public Collection getChanges()
    {try { __CLOVER_236_0.M[1252]++;
        __CLOVER_236_0.S[6077]++;if ((((_changes == null) && (++__CLOVER_236_0.CT[1073] != 0)) || (++__CLOVER_236_0.CF[1073] == 0))){
            __CLOVER_236_0.S[6078]++;return Collections.EMPTY_LIST;}

        __CLOVER_236_0.S[6079]++;int count = _changes.size();
        __CLOVER_236_0.S[6080]++;Collection result = new ArrayList(count);

        __CLOVER_236_0.S[6081]++;Iterator i = _changes.entrySet().iterator();
        __CLOVER_236_0.S[6082]++;while ((((i.hasNext()) && (++__CLOVER_236_0.CT[1074] != 0)) || (++__CLOVER_236_0.CF[1074] == 0))){
        {
            __CLOVER_236_0.S[6083]++;Map.Entry entry = (Map.Entry) i.next();

            __CLOVER_236_0.S[6084]++;ChangeKey key = (ChangeKey) entry.getKey();

            __CLOVER_236_0.S[6085]++;Object value = entry.getValue();

            __CLOVER_236_0.S[6086]++;PageChange change =
                new PageChange(key.getComponentPath(), key.getPropertyName(), value);

            __CLOVER_236_0.S[6087]++;result.add(change);
        }}

        __CLOVER_236_0.S[6088]++;return result;
    } finally { }}

    protected void recordChange(String componentPath, String propertyName, Object newValue)
    {try { __CLOVER_236_0.M[1253]++;
        __CLOVER_236_0.S[6089]++;ChangeKey key = new ChangeKey(componentPath, propertyName);

        __CLOVER_236_0.S[6090]++;if ((((_changes == null) && (++__CLOVER_236_0.CT[1075] != 0)) || (++__CLOVER_236_0.CF[1075] == 0))){
            __CLOVER_236_0.S[6091]++;_changes = new HashMap();}

        __CLOVER_236_0.S[6092]++;setDirty(true);

        __CLOVER_236_0.S[6093]++;_changes.put(key, newValue);

        // Now, build a key used to store the new value
        // in the HttpSession

        __CLOVER_236_0.S[6094]++;String attributeKey = constructAttributeKey(componentPath, propertyName);

        __CLOVER_236_0.S[6095]++;_session.setAttribute(attributeKey, newValue);

        __CLOVER_236_0.S[6096]++;if ((((LOG.isDebugEnabled()) && (++__CLOVER_236_0.CT[1076] != 0)) || (++__CLOVER_236_0.CF[1076] == 0))){
            __CLOVER_236_0.S[6097]++;LOG.debug("Stored session attribute " + attributeKey + " = " + newValue);}
    } finally { }}

    private String constructAttributeKey(String componentPath, String propertyName)
    {try { __CLOVER_236_0.M[1254]++;
        __CLOVER_236_0.S[6098]++;StringBuffer buffer = new StringBuffer(_attributePrefix);

        __CLOVER_236_0.S[6099]++;if ((((componentPath != null) && (++__CLOVER_236_0.CT[1077] != 0)) || (++__CLOVER_236_0.CF[1077] == 0))){
        {
            __CLOVER_236_0.S[6100]++;buffer.append(componentPath);
            __CLOVER_236_0.S[6101]++;buffer.append('/');
        }}

        __CLOVER_236_0.S[6102]++;buffer.append(propertyName);

        __CLOVER_236_0.S[6103]++;return buffer.toString();
    } finally { }}

    private void restorePageChanges()
    {try { __CLOVER_236_0.M[1255]++;
        __CLOVER_236_0.S[6104]++;int count = 0;
        __CLOVER_236_0.S[6105]++;Enumeration e = _session.getAttributeNames();
        __CLOVER_236_0.S[6106]++;StringSplitter splitter = null;

        __CLOVER_236_0.S[6107]++;while ((((e.hasMoreElements()) && (++__CLOVER_236_0.CT[1078] != 0)) || (++__CLOVER_236_0.CF[1078] == 0))){
        {
            __CLOVER_236_0.S[6108]++;String key = (String) e.nextElement();

            __CLOVER_236_0.S[6109]++;if ((((!key.startsWith(_attributePrefix)) && (++__CLOVER_236_0.CT[1079] != 0)) || (++__CLOVER_236_0.CF[1079] == 0))){
                __CLOVER_236_0.S[6110]++;continue;}

            __CLOVER_236_0.S[6111]++;if ((((LOG.isDebugEnabled()) && (++__CLOVER_236_0.CT[1080] != 0)) || (++__CLOVER_236_0.CF[1080] == 0))){
                __CLOVER_236_0.S[6112]++;LOG.debug("Restoring page change from session attribute " + key);}

            __CLOVER_236_0.S[6113]++;if ((((_changes == null) && (++__CLOVER_236_0.CT[1081] != 0)) || (++__CLOVER_236_0.CF[1081] == 0))){
            {
                __CLOVER_236_0.S[6114]++;_changes = new HashMap();

                __CLOVER_236_0.S[6115]++;splitter = new StringSplitter('/');
            }}

            __CLOVER_236_0.S[6116]++;String[] names = splitter.splitToArray(key);

            // The first name is the servlet name, which allows
            // multiple Tapestry apps to share a HttpSession, even
            // when they use the same page names.  The second name
            // is the page name, which we already know.

            __CLOVER_236_0.S[6117]++;int i = 2;

            __CLOVER_236_0.S[6118]++;String componentPath = ((((names.length == 4) ) && (++__CLOVER_236_0.CT[1082] != 0)) || (++__CLOVER_236_0.CF[1082] == 0))? names[i++] : null;
            __CLOVER_236_0.S[6119]++;String propertyName = names[i++];
            __CLOVER_236_0.S[6120]++;Object value = _session.getAttribute(key);

            __CLOVER_236_0.S[6121]++;ChangeKey changeKey = new ChangeKey(componentPath, propertyName);

            __CLOVER_236_0.S[6122]++;_changes.put(changeKey, value);

            __CLOVER_236_0.S[6123]++;count++;
        }}

        __CLOVER_236_0.S[6124]++;if ((((LOG.isDebugEnabled()) && (++__CLOVER_236_0.CT[1083] != 0)) || (++__CLOVER_236_0.CF[1083] == 0))){
            __CLOVER_236_0.S[6125]++;LOG.debug(
                (((count == 0 ) && (++__CLOVER_236_0.CT[1084] != 0)) || (++__CLOVER_236_0.CF[1084] == 0))? "No recorded changes." : "Restored " + count + " recorded changes.");}
    } finally { }}

}