/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.request;

import java.io.IOException;
import java.io.OutputStream;
import java.net.SocketException;

import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.tapestry.Tapestry;

/**
 *  A special output stream works with a {@link HttpServletResponse}, buffering
 *  data so as to defer opening the response's output stream.
 *
 *  <p>The buffering is pretty simple because the code
 *  between {@link org.apache.tapestry.IMarkupWriter} and this shows lots of buffering
 *  after the <code>PrintWriter</code> and inside the <code>OutputStreamWriter</code> that
 *  can't be configured.
 *
 *  <p>This class performs some buffering, but it is not all that
 *  useful because the 
 *  {@link org.apache.tapestry.html.Body} component (which will
 *  be used on virtually all Tapestry pages), buffers its wrapped contents
 *  (that is, evertyhing inside the &lt;body&gt; tag in the generated HTML).
 *
 *  @author Howard Lewis Ship
 * 
 **/

public class ResponseOutputStream extends OutputStream
{public static com.cortexeb.tools.clover.d __CLOVER_241_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private static final Log LOG = LogFactory.getLog(ResponseOutputStream.class);

    /**
     *  Default size for the buffer (2000 bytes).
     *
     **/

    public static final int DEFAULT_SIZE = 2000;

    private int _pos;
    private int _maxSize;
    private byte[] _buffer;

    private String _contentType;
    private HttpServletResponse _response;
    private OutputStream _out;

    private boolean _discard = false;

    /**
     *  Creates the stream with the default maximum buffer size.
     *
     **/

    public ResponseOutputStream(HttpServletResponse response)
    {
        this(response, DEFAULT_SIZE);__CLOVER_241_0.S[6469]++;try { __CLOVER_241_0.M[1306]++;
    } finally { }}

    /**
     *  Standard constructor.
     *
     **/

    public ResponseOutputStream(HttpServletResponse response, int maxSize)
    {try { __CLOVER_241_0.M[1307]++;
        __CLOVER_241_0.S[6470]++;_response = response;
        __CLOVER_241_0.S[6471]++;_maxSize = maxSize;
    } finally { }}

    /**
     *  Does nothing.  This is because of chaining of <code>close()</code> from
     *  {@link org.apache.tapestry.IMarkupWriter#close()} ... see {@link #flush()}.
     * 
     **/

    public void close() throws IOException
    {try { __CLOVER_241_0.M[1308]++;
        // Does nothing.
    } finally { }}

    /**
     *  Flushes the underlying output stream, if is has been opened.  
     *
     *  <p>This method explicitly <em>does not</em> flush the internal buffer ...
     *  that's because when an {@link org.apache.tapestry.IMarkupWriter} is closed (for instance, because
     *  an exception is thrown), that <code>close()</code> spawns <code>flush()</code>es
     *  and <code>close()</code>s throughout the output stream chain, eventually
     *  reaching this method.
     *
     *  @see #forceFlush()
     *
     **/

    public void flush() throws IOException
    {try { __CLOVER_241_0.M[1309]++;
        __CLOVER_241_0.S[6472]++;try
        {
            __CLOVER_241_0.S[6473]++;if ((((_out != null) && (++__CLOVER_241_0.CT[1140] != 0)) || (++__CLOVER_241_0.CF[1140] == 0))){
                __CLOVER_241_0.S[6474]++;_out.flush();}
        }
        catch (SocketException ex)
        {
            __CLOVER_241_0.S[6475]++;LOG.debug("Socket exception.");
        }
    } finally { }}

    /**
     *  Writes the internal buffer to the output stream, opening it if necessary, then
     *  flushes the output stream.  Future writes will go directly to the output stream.
     *
     **/

    public void forceFlush() throws IOException
    {try { __CLOVER_241_0.M[1310]++;
        __CLOVER_241_0.S[6476]++;if ((((_out == null) && (++__CLOVER_241_0.CT[1141] != 0)) || (++__CLOVER_241_0.CF[1141] == 0))){
        {

            // In certain cases (such as when the Tapestry service sends a redirect),
            // there is no output to send back (and no content type set).  In this
            // case, forceFlush() does nothing.

            __CLOVER_241_0.S[6477]++;if ((((_buffer == null) && (++__CLOVER_241_0.CT[1142] != 0)) || (++__CLOVER_241_0.CF[1142] == 0))){
                __CLOVER_241_0.S[6478]++;return;}

            __CLOVER_241_0.S[6479]++;open();
        }}

        __CLOVER_241_0.S[6480]++;try
        {
            __CLOVER_241_0.S[6481]++;_out.flush();
        }
        catch (SocketException ex)
        {
            __CLOVER_241_0.S[6482]++;LOG.debug("Socket exception.");
        }
    } finally { }}

    public String getContentType()
    {try { __CLOVER_241_0.M[1311]++;
        __CLOVER_241_0.S[6483]++;return _contentType;
    } finally { }}

    public boolean getDiscard()
    {try { __CLOVER_241_0.M[1312]++;
        __CLOVER_241_0.S[6484]++;return _discard;
    } finally { }}

    /**
     *  Sets the response type to from the contentType property (which
     *  defaults to "text/html") and gets an output stream
     *  from the response, then writes the current buffer to it and
     *  releases the buffer.
     *
     *  @throws IOException if the content type has never been set.
     *
     **/

    private void open() throws IOException
    {try { __CLOVER_241_0.M[1313]++;
        __CLOVER_241_0.S[6485]++;if ((((_contentType == null) && (++__CLOVER_241_0.CT[1143] != 0)) || (++__CLOVER_241_0.CF[1143] == 0))){
            __CLOVER_241_0.S[6486]++;throw new IOException(Tapestry.getMessage("ResponseOutputStream.content-type-not-set"));}

        __CLOVER_241_0.S[6487]++;_response.setContentType(_contentType);

        __CLOVER_241_0.S[6488]++;_out = _response.getOutputStream();

        __CLOVER_241_0.S[6489]++;innerWrite(_buffer, 0, _pos);

        __CLOVER_241_0.S[6490]++;_pos = 0;
        __CLOVER_241_0.S[6491]++;_buffer = null;
    } finally { }}

    /**
     *  Discards all output in the buffer.  This is used after an error to
     *  restart the output (so that the error may be presented).
     *
     *  <p>Clears the discard flag.
     *
     **/

    public void reset() throws IOException
    {try { __CLOVER_241_0.M[1314]++;
        __CLOVER_241_0.S[6492]++;_pos = 0;
        __CLOVER_241_0.S[6493]++;_discard = false;
    } finally { }}

    /**
     *  Changes the maximum buffer size.  If the new buffer size is smaller
     *  than the number of
     *  bytes already in the buffer, the buffer is immediately flushed.
     *
     **/

    public void setBufferSize(int value) throws IOException
    {try { __CLOVER_241_0.M[1315]++;
        __CLOVER_241_0.S[6494]++;if ((((value < _pos) && (++__CLOVER_241_0.CT[1144] != 0)) || (++__CLOVER_241_0.CF[1144] == 0))){
        {
            __CLOVER_241_0.S[6495]++;open();
            __CLOVER_241_0.S[6496]++;return;
        }}

        __CLOVER_241_0.S[6497]++;_maxSize = value;
    } finally { }}

    public void setContentType(String value)
    {try { __CLOVER_241_0.M[1316]++;
        __CLOVER_241_0.S[6498]++;_contentType = value;
    } finally { }}

    /**
     *  Indicates whether the stream should ignore all data written to it.
     *
     **/

    public void setDiscard(boolean value)
    {try { __CLOVER_241_0.M[1317]++;
        __CLOVER_241_0.S[6499]++;_discard = value;
    } finally { }}

    private void innerWrite(byte[] b, int off, int len) throws IOException
    {try { __CLOVER_241_0.M[1318]++;
        __CLOVER_241_0.S[6500]++;if ((((b == null || len == 0 || _discard) && (++__CLOVER_241_0.CT[1145] != 0)) || (++__CLOVER_241_0.CF[1145] == 0))){
            __CLOVER_241_0.S[6501]++;return;}

        __CLOVER_241_0.S[6502]++;try
        {
            __CLOVER_241_0.S[6503]++;_out.write(b, off, len);
        }
        catch (SocketException ex)
        {
            __CLOVER_241_0.S[6504]++;LOG.debug("Socket exception.");
        }
    } finally { }}

    public void write(byte b[], int off, int len) throws IOException
    {try { __CLOVER_241_0.M[1319]++;
        __CLOVER_241_0.S[6505]++;if ((((len == 0 || _discard) && (++__CLOVER_241_0.CT[1146] != 0)) || (++__CLOVER_241_0.CF[1146] == 0))){
            __CLOVER_241_0.S[6506]++;return;}

        __CLOVER_241_0.S[6507]++;if ((((_out != null) && (++__CLOVER_241_0.CT[1147] != 0)) || (++__CLOVER_241_0.CF[1147] == 0))){
        {
            __CLOVER_241_0.S[6508]++;_out.write(b, off, len);
            __CLOVER_241_0.S[6509]++;return;
        }}

        // If too large for the maximum size buffer, then open the output stream
        // write out and free the buffer, and write out the new stuff.

        __CLOVER_241_0.S[6510]++;if ((((_pos + len >= _maxSize) && (++__CLOVER_241_0.CT[1148] != 0)) || (++__CLOVER_241_0.CF[1148] == 0))){
        {
            __CLOVER_241_0.S[6511]++;open();
            __CLOVER_241_0.S[6512]++;innerWrite(b, off, len);
            __CLOVER_241_0.S[6513]++;return;
        }}

        // Allocate the buffer when it is initially needed.

        __CLOVER_241_0.S[6514]++;if ((((_buffer == null) && (++__CLOVER_241_0.CT[1149] != 0)) || (++__CLOVER_241_0.CF[1149] == 0))){
            __CLOVER_241_0.S[6515]++;_buffer = new byte[_maxSize];}

        // Copy the new bytes into the buffer and advance the position.

        __CLOVER_241_0.S[6516]++;System.arraycopy(b, off, _buffer, _pos, len);
        __CLOVER_241_0.S[6517]++;_pos += len;
    } finally { }}

    public void write(int b) throws IOException
    {try { __CLOVER_241_0.M[1320]++;
        __CLOVER_241_0.S[6518]++;if ((((_discard) && (++__CLOVER_241_0.CT[1150] != 0)) || (++__CLOVER_241_0.CF[1150] == 0))){
            __CLOVER_241_0.S[6519]++;return;}

        // This method is rarely called so this little inefficiency is better than
        // maintaining that ugly buffer expansion code in two places.

        __CLOVER_241_0.S[6520]++;byte[] tiny = new byte[] {(byte) b };

        __CLOVER_241_0.S[6521]++;write(tiny, 0, 1);
    } finally { }}
}