/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.resolver;

import org.apache.commons.logging.Log;
import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.hivemind.Location;
import org.apache.hivemind.Resource;
import org.apache.tapestry.INamespace;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.spec.IComponentSpecification;

/**
 *  Utility class that understands the rules of component types (which
 *  may optionally have a library prefix) and can resolve 
 *  the type to a {@link org.apache.tapestry.INamespace} and a 
 *  {@link org.apache.tapestry.spec.IComponentSpecification}.
 * 
 *  <p>Like {@link org.apache.tapestry.resolver.PageSpecificationResolver},
 *  if the component is not defined explicitly in the namespace, a search
 *  may occur:
 * 
 *  Performs the tricky work of resolving a page name to a page specification.
 *  The search for pages in the application namespace is the most complicated,
 *  since Tapestry searches for pages that aren't explicitly defined in the
 *  application specification.  The search, based on the <i>simple-name</i>
 *  of the page, goes as follows:
 * 
 *  <ul>
 *  <li>As declared in the application specification
 *  <li><i>type</i>.jwc in the same folder as the application specification
 *  <li><i>type</i> jwc in the WEB-INF/<i>servlet-name</i> directory of the context root
 *  <li><i>type</i>.jwc in WEB-INF
 *  <li><i>type</i>.jwc in the application root (within the context root)
 *  <li>By searching the framework namespace
 *  </ul> 
 * 
 *  The search for components in library namespaces is more abbreviated:
 *  <li>As declared in the library specification
 *  <li><i>type</i>.jwc in the same folder as the library specification
 *  <li>By searching the framework namespace
 *  </ul>
 *
 * 
 *  @author Howard Lewis Ship
 *  @since 3.0
 *
 */

public class ComponentSpecificationResolverImpl
    extends AbstractSpecificationResolver
    implements ComponentSpecificationResolver
{public static com.cortexeb.tools.clover.d __CLOVER_244_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    /** Set by container */
    private Log _log;

    /** Set by resolve() */
    private String _type;

    protected void reset()
    {try { __CLOVER_244_0.M[1336]++;
        __CLOVER_244_0.S[6540]++;_type = null;

        __CLOVER_244_0.S[6541]++;super.reset();
    } finally { }}

    /**
     *  Passed the namespace of a container (to resolve the type in)
     *  and the type to resolve, performs the processing.  A "bare type"
     *  (without a library prefix) may be in the containerNamespace,
     *  or the framework namespace
     *  (a search occurs in that order).
     * 
     *  @param cycle current request cycle
     *  @param containerNamespace namespace that may contain
     *  a library referenced in the type
     *  @param type the component specification
     *  to  find, either a simple name, or prefixed with a library id
     *  (defined for the container namespace)
     * 
     *  @see #getNamespace()
     *  @see #getSpecification()
     * 
     */

    public void resolve(
        IRequestCycle cycle,
        INamespace containerNamespace,
        String type,
        Location location)
    {try { __CLOVER_244_0.M[1337]++;
        __CLOVER_244_0.S[6542]++;int colonx = type.indexOf(':');

        __CLOVER_244_0.S[6543]++;if ((((colonx > 0) && (++__CLOVER_244_0.CT[1151] != 0)) || (++__CLOVER_244_0.CF[1151] == 0))){
        {
            __CLOVER_244_0.S[6544]++;String libraryId = type.substring(0, colonx);
            __CLOVER_244_0.S[6545]++;String simpleType = type.substring(colonx + 1);

            __CLOVER_244_0.S[6546]++;resolve(cycle, containerNamespace, libraryId, simpleType, location);
        }}
        else{
            __CLOVER_244_0.S[6547]++;resolve(cycle, containerNamespace, null, type, location);}
    } finally { }}

    /**
     *  Like {@link #resolve(org.apache.tapestry.IRequestCycle, org.apache.tapestry.INamespace, java.lang.String, org.apache.tapestry.ILocation)},
     *  but used when the type has already been parsed into a library id and a simple type.
     * 
     *  @param cycle current request cycle
     *  @param containerNamespace namespace that may contain
     *  a library referenced in the type
     *  @param libraryId the library id within the container namespace, or null
     *  @param type the component specification
     *  to  find as a simple name (without a library prefix)
     *  @param location of reference to be resolved
     *  @throws ApplicationRuntimeException if the type cannot be resolved
     * 
     */

    public void resolve(
        IRequestCycle cycle,
        INamespace containerNamespace,
        String libraryId,
        String type,
        Location location)
    {try { __CLOVER_244_0.M[1338]++;
        __CLOVER_244_0.S[6548]++;reset();
        __CLOVER_244_0.S[6549]++;_type = type;

        __CLOVER_244_0.S[6550]++;INamespace namespace = null;

        __CLOVER_244_0.S[6551]++;if ((((libraryId != null) && (++__CLOVER_244_0.CT[1152] != 0)) || (++__CLOVER_244_0.CF[1152] == 0))){
            __CLOVER_244_0.S[6552]++;namespace = containerNamespace.getChildNamespace(libraryId);}
        else{
            __CLOVER_244_0.S[6553]++;namespace = containerNamespace;}

        __CLOVER_244_0.S[6554]++;setNamespace(namespace);

        __CLOVER_244_0.S[6555]++;if ((((namespace.containsComponentType(type)) && (++__CLOVER_244_0.CT[1153] != 0)) || (++__CLOVER_244_0.CF[1153] == 0))){
            __CLOVER_244_0.S[6556]++;setSpecification(namespace.getComponentSpecification(type));}
        else{
            __CLOVER_244_0.S[6557]++;searchForComponent(cycle);}

        // If not found after search, check to see if it's in
        // the framework instead.

        __CLOVER_244_0.S[6558]++;if ((((getSpecification() == null) && (++__CLOVER_244_0.CT[1154] != 0)) || (++__CLOVER_244_0.CF[1154] == 0))){
        {

            __CLOVER_244_0.S[6559]++;throw new ApplicationRuntimeException(
                ResolverMessages.noSuchComponentType(type, namespace),
                location,
                null);

        }}
    } finally { }}

    private void searchForComponent(IRequestCycle cycle)
    {try { __CLOVER_244_0.M[1339]++;
        __CLOVER_244_0.S[6560]++;INamespace namespace = getNamespace();

        __CLOVER_244_0.S[6561]++;if ((((_log.isDebugEnabled()) && (++__CLOVER_244_0.CT[1155] != 0)) || (++__CLOVER_244_0.CF[1155] == 0))){
            __CLOVER_244_0.S[6562]++;_log.debug("Resolving unknown component '" + _type + "' in " + namespace);}

        __CLOVER_244_0.S[6563]++;String expectedName = _type + ".jwc";
        __CLOVER_244_0.S[6564]++;Resource namespaceLocation = namespace.getSpecificationLocation();

        // Look for appropriate file in same folder as the library (or application)
        // specificaiton.

        __CLOVER_244_0.S[6565]++;if ((((found(namespaceLocation.getRelativeResource(expectedName))) && (++__CLOVER_244_0.CT[1156] != 0)) || (++__CLOVER_244_0.CF[1156] == 0))){
            __CLOVER_244_0.S[6566]++;return;}

        __CLOVER_244_0.S[6567]++;if ((((namespace.isApplicationNamespace()) && (++__CLOVER_244_0.CT[1157] != 0)) || (++__CLOVER_244_0.CF[1157] == 0))){
        {

            // The application namespace gets some extra searching.

            __CLOVER_244_0.S[6568]++;if ((((found(getWebInfAppLocation().getRelativeResource(expectedName))) && (++__CLOVER_244_0.CT[1158] != 0)) || (++__CLOVER_244_0.CF[1158] == 0))){
                __CLOVER_244_0.S[6569]++;return;}

            __CLOVER_244_0.S[6570]++;if ((((found(getWebInfLocation().getRelativeResource(expectedName))) && (++__CLOVER_244_0.CT[1159] != 0)) || (++__CLOVER_244_0.CF[1159] == 0))){
                __CLOVER_244_0.S[6571]++;return;}

            __CLOVER_244_0.S[6572]++;if ((((found(getApplicationRootLocation().getRelativeResource(expectedName))) && (++__CLOVER_244_0.CT[1160] != 0)) || (++__CLOVER_244_0.CF[1160] == 0))){
                __CLOVER_244_0.S[6573]++;return;}
        }}

        // Not in the library or app spec; does it match a component
        // provided by the Framework?

        __CLOVER_244_0.S[6574]++;INamespace framework = getSpecificationSource().getFrameworkNamespace();

        __CLOVER_244_0.S[6575]++;if ((((framework.containsComponentType(_type)) && (++__CLOVER_244_0.CT[1161] != 0)) || (++__CLOVER_244_0.CF[1161] == 0))){
        {
            __CLOVER_244_0.S[6576]++;setSpecification(framework.getComponentSpecification(_type));
            __CLOVER_244_0.S[6577]++;return;
        }}

        __CLOVER_244_0.S[6578]++;IComponentSpecification specification =
            getDelegate().findComponentSpecification(cycle, namespace, _type);

        __CLOVER_244_0.S[6579]++;setSpecification(specification);

        // If not found by here, an exception will be thrown.
    } finally { }}

    private boolean found(Resource resource)
    {try { __CLOVER_244_0.M[1340]++;
        __CLOVER_244_0.S[6580]++;if ((((_log.isDebugEnabled()) && (++__CLOVER_244_0.CT[1162] != 0)) || (++__CLOVER_244_0.CF[1162] == 0))){
            __CLOVER_244_0.S[6581]++;_log.debug("Checking: " + resource);}

        __CLOVER_244_0.S[6582]++;if ((((resource.getResourceURL() == null) && (++__CLOVER_244_0.CT[1163] != 0)) || (++__CLOVER_244_0.CF[1163] == 0))){
            __CLOVER_244_0.S[6583]++;return false;}

        __CLOVER_244_0.S[6584]++;setSpecification(getSpecificationSource().getComponentSpecification(resource));

        __CLOVER_244_0.S[6585]++;install();

        __CLOVER_244_0.S[6586]++;return true;
    } finally { }}

    private void install()
    {try { __CLOVER_244_0.M[1341]++;
        __CLOVER_244_0.S[6587]++;INamespace namespace = getNamespace();
        __CLOVER_244_0.S[6588]++;IComponentSpecification specification = getSpecification();

        __CLOVER_244_0.S[6589]++;if ((((_log.isDebugEnabled()) && (++__CLOVER_244_0.CT[1164] != 0)) || (++__CLOVER_244_0.CF[1164] == 0))){
            __CLOVER_244_0.S[6590]++;_log.debug(
                "Installing component type "
                    + _type
                    + " into "
                    + namespace
                    + " as "
                    + specification);}

        __CLOVER_244_0.S[6591]++;namespace.installComponentSpecification(_type, specification);
    } finally { }}

    public void setLog(Log log)
    {try { __CLOVER_244_0.M[1342]++;
        __CLOVER_244_0.S[6592]++;_log = log;
    } finally { }}

}
