/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.spec;

import org.apache.commons.lang.enum.Enum;
import org.apache.tapestry.Tapestry;

/**
 *  Represents different types of parameters.  Currently only 
 *  in and custom are supported, but this will likely change
 *  when Tapestry supports out parameters is some form (that reflects
 *  form style processing).
 *
 *  @author Howard Lewis Ship
 *  @since 2.0.3
 *
 */
 
public class Direction extends Enum
{public static com.cortexeb.tools.clover.d __CLOVER_348_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    /**
     *  The parameter value is input only; the component property value
     *  is unchanged or not relevant after the component renders.
     *  The property is set from the binding before the component renders,
     *  then reset to initial value after the component renders.
     * 
     */
    
	public static final Direction IN = new Direction("IN");
	
    
    /**
     *  Encapsulates the semantics of a form component's value parameter.
     * 
     *  <p>The parameter is associated with a {@link org.apache.tapestry.form.IFormComponent}.
     *  The property value is set from the binding before the component renders (when renderring,
     *  but not when rewinding).
     *  The binding is updated from the property value
     *  after after the component renders when the
     *  <b>containing form</b> is <b>rewinding</b>, <i>and</i>
     *  the component is not {@link org.apache.tapestry.form.IFormComponent#isDisabled() disabled}.
     * 
     *  @since 2.2
     * 
     */

    public static final Direction FORM = new Direction("FORM", false);    
    
	/**
	 *  Processing of the parameter is entirely the responsibility
	 *  of the component, which must obtain an manipulate
	 *  the {@link org.apache.tapestry.IBinding} (if any) for the parameter.
	 * 
	 **/
	
	public static final Direction CUSTOM = new Direction("CUSTOM");


	/**
	 *  Causes a synthetic property to be created that automatically
	 *  references and de-references the underlying binding.
	 * 
	 *  @since 3.0
	 * 
	 */
	
	public static final Direction AUTO = new Direction("AUTO");
	
	/**
	 * If true, then this direction is allowed with invariant bindings (the usual case).
	 * If false, then {@link org.apache.tapestry.param.ParameterManager} will not allow
	 * an invariant binding.
	 * 
	 * @since 3.0
	 */
	
	private boolean _allowInvariant;
	
    protected Direction(String name)
    {
        this(name, true);__CLOVER_348_0.S[7814]++;try { __CLOVER_348_0.M[1733]++;
    } finally { }}

	protected Direction(String name, boolean allowInvariant)
	{
		super(name);__CLOVER_348_0.S[7815]++;try { __CLOVER_348_0.M[1734]++;
		
		__CLOVER_348_0.S[7816]++;_allowInvariant = allowInvariant;
	} finally { }}
	
	/**
	 * @since 3.0
	 */
	public boolean getAllowInvariant()
	{try { __CLOVER_348_0.M[1735]++;
		__CLOVER_348_0.S[7817]++;return _allowInvariant;
	} finally { }}

    /**
     *  Returns a user-presentable name for the direction.
     * 
     */
    
    public String getDisplayName()
    {try { __CLOVER_348_0.M[1736]++;
        __CLOVER_348_0.S[7818]++;return Tapestry.getMessage("Direction." + getName());
    } finally { }}
}
