/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.test.assertions;

import java.util.ArrayList;
import java.util.List;

import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.hivemind.impl.BaseLocatable;
import org.apache.tapestry.test.ResponseAssertion;
import org.apache.tapestry.test.ScriptMessages;
import org.apache.tapestry.test.ScriptedTestSession;

/**
 * Assertion used to check for the presence of a regular expression
 * within the response output. Alternately, it can check for a specific
 * list of matches of a regular expression within the response.
 *
 * @author Howard Lewis Ship
 * @since 3.1
 */
public class AssertRegexp extends BaseLocatable implements ResponseAssertion
{public static com.cortexeb.tools.clover.d __CLOVER_378_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);

    private List _matches;
    private String _regexp;
    private int _subgroup;

    public void addMatch(RegexpMatch m)
    {try { __CLOVER_378_0.M[1927]++;
        __CLOVER_378_0.S[8365]++;if ((((_matches == null) && (++__CLOVER_378_0.CT[1413] != 0)) || (++__CLOVER_378_0.CF[1413] == 0))){
            __CLOVER_378_0.S[8366]++;_matches = new ArrayList();}

        __CLOVER_378_0.S[8367]++;_matches.add(m);
    } finally { }}

    public void execute(ScriptedTestSession session)
    {try { __CLOVER_378_0.M[1928]++;
        __CLOVER_378_0.S[8368]++;if ((((_matches != null) && (++__CLOVER_378_0.CT[1414] != 0)) || (++__CLOVER_378_0.CF[1414] == 0))){
        {
            __CLOVER_378_0.S[8369]++;executeMatches(session);
            __CLOVER_378_0.S[8370]++;return;
        }}

        __CLOVER_378_0.S[8371]++;String responseContent = session.getResponse().getOutputString();

        __CLOVER_378_0.S[8372]++;if ((((session.getMatcher().contains(_regexp, responseContent)) && (++__CLOVER_378_0.CT[1415] != 0)) || (++__CLOVER_378_0.CF[1415] == 0))){
            __CLOVER_378_0.S[8373]++;return;}

        __CLOVER_378_0.S[8374]++;throw new ApplicationRuntimeException(
            ScriptMessages.expectedRegexpMissing(_regexp, getLocation()),
            getLocation(),
            null);
    } finally { }}

    private void executeMatches(ScriptedTestSession session)
    {try { __CLOVER_378_0.M[1929]++;
        __CLOVER_378_0.S[8375]++;String responseContent = session.getResponse().getOutputString();

        __CLOVER_378_0.S[8376]++;String[] matches = session.getMatcher().getMatches(_regexp, responseContent, _subgroup);

        __CLOVER_378_0.S[8377]++;int expectedCount = _matches.size();
        __CLOVER_378_0.S[8378]++;int count = Math.min(expectedCount, matches.length);

        __CLOVER_378_0.S[8379]++;for (int i = 0; (((i < count) && (++__CLOVER_378_0.CT[1416] != 0)) || (++__CLOVER_378_0.CF[1416] == 0)); i++){
        {
            __CLOVER_378_0.S[8380]++;RegexpMatch m = (RegexpMatch) _matches.get(i);

            __CLOVER_378_0.S[8381]++;String expected = m.getExpectedString();
            __CLOVER_378_0.S[8382]++;String actual = matches[i];

            __CLOVER_378_0.S[8383]++;if ((((expected.equals(actual)) && (++__CLOVER_378_0.CT[1417] != 0)) || (++__CLOVER_378_0.CF[1417] == 0))){
                __CLOVER_378_0.S[8384]++;continue;}

            __CLOVER_378_0.S[8385]++;throw new ApplicationRuntimeException(
                ScriptMessages.incorrectRegexpMatch(expected, m.getLocation(), actual),
                m.getLocation(),
                null);

        }}

        __CLOVER_378_0.S[8386]++;if ((((matches.length != expectedCount) && (++__CLOVER_378_0.CT[1418] != 0)) || (++__CLOVER_378_0.CF[1418] == 0))){
            __CLOVER_378_0.S[8387]++;throw new ApplicationRuntimeException(
                ScriptMessages.incorrectRegexpMatchCount(
                    _regexp,
                    getLocation(),
                    expectedCount,
                    matches.length),
                getLocation(),
                null);}
    } finally { }}

    public void setRegexp(String string)
    {try { __CLOVER_378_0.M[1930]++;
        __CLOVER_378_0.S[8388]++;_regexp = string;
    } finally { }}

    public void setSubgroup(int subgroup)
    {try { __CLOVER_378_0.M[1931]++;
        __CLOVER_378_0.S[8389]++;_subgroup = subgroup;
    } finally { }}

}
