/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.util.io;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.Writer;

/**
 *  A kind of super-formatter.  It is sent a stream of binary data and
 *  formats it in a human-readable dump format which is forwarded to
 *  its output stream.
 *
 * <p>Currently, output is in hex though options to change that may
 * be introduced.
 *
 *  @author Howard Lewis Ship
 * 
 **/

public class BinaryDumpOutputStream extends OutputStream
{public static com.cortexeb.tools.clover.d __CLOVER_409_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private PrintWriter out;

    private boolean locked = false;

    private boolean showOffset = true;
    private int bytesPerLine = 16;
    private int spacingInterval = 4;
    private char substituteChar = '.';
    private String offsetSeperator = ": ";
    private int offset = 0;
    private int lineCount = 0;
    private int bytesSinceSpace = 0;
    private char[] ascii = null;
    private boolean showAscii = true;
    private String asciiBegin = "  |";
    private String asciiEnd = "|";

    private static final char[] HEX =
        {
            '0',
            '1',
            '2',
            '3',
            '4',
            '5',
            '6',
            '7',
            '8',
            '9',
            'a',
            'b',
            'c',
            'd',
            'e',
            'f' };

    /**
     *  Creates a <code>PrintWriter</code> for <code>System.out</code>.
     *
     **/

    public BinaryDumpOutputStream()
    {
        this(new PrintWriter(System.out, true));__CLOVER_409_0.S[9143]++;try { __CLOVER_409_0.M[2204]++;
    } finally { }}

    public BinaryDumpOutputStream(PrintWriter out)
    {try { __CLOVER_409_0.M[2205]++;
        __CLOVER_409_0.S[9144]++;this.out = out;
    } finally { }}

    public BinaryDumpOutputStream(Writer out)
    {try { __CLOVER_409_0.M[2206]++;
        __CLOVER_409_0.S[9145]++;this.out = new PrintWriter(out);
    } finally { }}

    public void close() throws IOException
    {try { __CLOVER_409_0.M[2207]++;
        __CLOVER_409_0.S[9146]++;if ((((out != null) && (++__CLOVER_409_0.CT[1547] != 0)) || (++__CLOVER_409_0.CF[1547] == 0))){
        {
            __CLOVER_409_0.S[9147]++;if ((((lineCount > 0) && (++__CLOVER_409_0.CT[1548] != 0)) || (++__CLOVER_409_0.CF[1548] == 0))){
                __CLOVER_409_0.S[9148]++;finishFinalLine();}

            __CLOVER_409_0.S[9149]++;out.close();
        }}

        __CLOVER_409_0.S[9150]++;out = null;
    } finally { }}

    private void finishFinalLine()
    {try { __CLOVER_409_0.M[2208]++;
        // Since we only finish the final line after at least one byte has
        // been written to it, we don't need to worry about
        // the offset.

        __CLOVER_409_0.S[9151]++;while ((((lineCount < bytesPerLine) && (++__CLOVER_409_0.CT[1549] != 0)) || (++__CLOVER_409_0.CF[1549] == 0))){
        {
            // After every <n> bytes, emit a space.

            __CLOVER_409_0.S[9152]++;if ((((spacingInterval > 0 && bytesSinceSpace == spacingInterval) && (++__CLOVER_409_0.CT[1550] != 0)) || (++__CLOVER_409_0.CF[1550] == 0))){
            {
                __CLOVER_409_0.S[9153]++;out.print(' ');
                __CLOVER_409_0.S[9154]++;bytesSinceSpace = 0;
            }}

            // Two spaces to substitute for the two hex digits.

            __CLOVER_409_0.S[9155]++;out.print("  ");

            __CLOVER_409_0.S[9156]++;if ((((showAscii) && (++__CLOVER_409_0.CT[1551] != 0)) || (++__CLOVER_409_0.CF[1551] == 0))){
                __CLOVER_409_0.S[9157]++;ascii[lineCount] = ' ';}

            __CLOVER_409_0.S[9158]++;lineCount++;
            __CLOVER_409_0.S[9159]++;bytesSinceSpace++;
        }}

        __CLOVER_409_0.S[9160]++;if ((((showAscii) && (++__CLOVER_409_0.CT[1552] != 0)) || (++__CLOVER_409_0.CF[1552] == 0))){
        {
            __CLOVER_409_0.S[9161]++;out.print(asciiBegin);
            __CLOVER_409_0.S[9162]++;out.print(ascii);
            __CLOVER_409_0.S[9163]++;out.print(asciiEnd);
        }}

        __CLOVER_409_0.S[9164]++;out.println();
    } finally { }}

    /**
     *  Forwards the <code>flush()</code> to the <code>PrintWriter</code>.
     *
     **/

    public void flush() throws IOException
    {try { __CLOVER_409_0.M[2209]++;
        __CLOVER_409_0.S[9165]++;out.flush();
    } finally { }}

    public String getAsciiBegin()
    {try { __CLOVER_409_0.M[2210]++;
        __CLOVER_409_0.S[9166]++;return asciiBegin;
    } finally { }}

    public String getAsciiEnd()
    {try { __CLOVER_409_0.M[2211]++;
        __CLOVER_409_0.S[9167]++;return asciiEnd;
    } finally { }}

    public int getBytesPerLine()
    {try { __CLOVER_409_0.M[2212]++;
        __CLOVER_409_0.S[9168]++;return bytesPerLine;
    } finally { }}

    public String getOffsetSeperator()
    {try { __CLOVER_409_0.M[2213]++;
        __CLOVER_409_0.S[9169]++;return offsetSeperator;
    } finally { }}

    public boolean getShowAscii()
    {try { __CLOVER_409_0.M[2214]++;
        __CLOVER_409_0.S[9170]++;return showAscii;
    } finally { }}

    public char getSubstituteChar()
    {try { __CLOVER_409_0.M[2215]++;
        __CLOVER_409_0.S[9171]++;return substituteChar;
    } finally { }}

    public void setAsciiBegin(String value)
    {try { __CLOVER_409_0.M[2216]++;
        __CLOVER_409_0.S[9172]++;if ((((locked) && (++__CLOVER_409_0.CT[1553] != 0)) || (++__CLOVER_409_0.CF[1553] == 0))){
            __CLOVER_409_0.S[9173]++;throw new IllegalStateException();}

        __CLOVER_409_0.S[9174]++;asciiBegin = value;
    } finally { }}

    public void setAsciiEnd(String value)
    {try { __CLOVER_409_0.M[2217]++;
        __CLOVER_409_0.S[9175]++;if ((((locked) && (++__CLOVER_409_0.CT[1554] != 0)) || (++__CLOVER_409_0.CF[1554] == 0))){
            __CLOVER_409_0.S[9176]++;throw new IllegalStateException();}

        __CLOVER_409_0.S[9177]++;asciiEnd = value;
    } finally { }}

    public void setBytesPerLine(int value)
    {try { __CLOVER_409_0.M[2218]++;
        __CLOVER_409_0.S[9178]++;if ((((locked) && (++__CLOVER_409_0.CT[1555] != 0)) || (++__CLOVER_409_0.CF[1555] == 0))){
            __CLOVER_409_0.S[9179]++;throw new IllegalStateException();}

        __CLOVER_409_0.S[9180]++;bytesPerLine = value;

        __CLOVER_409_0.S[9181]++;ascii = null;
    } finally { }}

    public void setOffsetSeperator(String value)
    {try { __CLOVER_409_0.M[2219]++;
        __CLOVER_409_0.S[9182]++;if ((((locked) && (++__CLOVER_409_0.CT[1556] != 0)) || (++__CLOVER_409_0.CF[1556] == 0))){
            __CLOVER_409_0.S[9183]++;throw new IllegalStateException();}

        __CLOVER_409_0.S[9184]++;offsetSeperator = value;
    } finally { }}

    public void setShowAscii(boolean value)
    {try { __CLOVER_409_0.M[2220]++;
        __CLOVER_409_0.S[9185]++;if ((((locked) && (++__CLOVER_409_0.CT[1557] != 0)) || (++__CLOVER_409_0.CF[1557] == 0))){
            __CLOVER_409_0.S[9186]++;throw new IllegalStateException();}

        __CLOVER_409_0.S[9187]++;showAscii = value;
    } finally { }}

    /**
     *  Sets the character used in the ASCII dump that substitutes for characters
     *  outside the range of 32..126.
     *
     **/

    public void setSubstituteChar(char value)
    {try { __CLOVER_409_0.M[2221]++;
        __CLOVER_409_0.S[9188]++;if ((((locked) && (++__CLOVER_409_0.CT[1558] != 0)) || (++__CLOVER_409_0.CF[1558] == 0))){
            __CLOVER_409_0.S[9189]++;throw new IllegalStateException();}

        __CLOVER_409_0.S[9190]++;substituteChar = value;
    } finally { }}

    public void write(int b) throws IOException
    {try { __CLOVER_409_0.M[2222]++;
        __CLOVER_409_0.S[9191]++;char letter;

        __CLOVER_409_0.S[9192]++;if ((((showAscii && ascii == null) && (++__CLOVER_409_0.CT[1559] != 0)) || (++__CLOVER_409_0.CF[1559] == 0))){
            __CLOVER_409_0.S[9193]++;ascii = new char[bytesPerLine];}

        // Prevent further customization after output starts being written.

        __CLOVER_409_0.S[9194]++;locked = true;

        __CLOVER_409_0.S[9195]++;if ((((lineCount == bytesPerLine) && (++__CLOVER_409_0.CT[1560] != 0)) || (++__CLOVER_409_0.CF[1560] == 0))){
        {
            __CLOVER_409_0.S[9196]++;if ((((showAscii) && (++__CLOVER_409_0.CT[1561] != 0)) || (++__CLOVER_409_0.CF[1561] == 0))){
            {
                __CLOVER_409_0.S[9197]++;out.print(asciiBegin);
                __CLOVER_409_0.S[9198]++;out.print(ascii);
                __CLOVER_409_0.S[9199]++;out.print(asciiEnd);
            }}

            __CLOVER_409_0.S[9200]++;out.println();

            __CLOVER_409_0.S[9201]++;bytesSinceSpace = 0;
            __CLOVER_409_0.S[9202]++;lineCount = 0;
            __CLOVER_409_0.S[9203]++;offset += bytesPerLine;
        }}

        __CLOVER_409_0.S[9204]++;if ((((lineCount == 0 && showOffset) && (++__CLOVER_409_0.CT[1562] != 0)) || (++__CLOVER_409_0.CF[1562] == 0))){
        {
            __CLOVER_409_0.S[9205]++;writeHex(offset, 4);
            __CLOVER_409_0.S[9206]++;out.print(offsetSeperator);
        }}

        // After every <n> bytes, emit a space.

        __CLOVER_409_0.S[9207]++;if ((((spacingInterval > 0 && bytesSinceSpace == spacingInterval) && (++__CLOVER_409_0.CT[1563] != 0)) || (++__CLOVER_409_0.CF[1563] == 0))){
        {
            __CLOVER_409_0.S[9208]++;out.print(' ');
            __CLOVER_409_0.S[9209]++;bytesSinceSpace = 0;
        }}

        __CLOVER_409_0.S[9210]++;writeHex(b, 2);

        __CLOVER_409_0.S[9211]++;if ((((showAscii) && (++__CLOVER_409_0.CT[1564] != 0)) || (++__CLOVER_409_0.CF[1564] == 0))){
        {
            __CLOVER_409_0.S[9212]++;if ((((b < 32 | b > 127) && (++__CLOVER_409_0.CT[1565] != 0)) || (++__CLOVER_409_0.CF[1565] == 0))){
                __CLOVER_409_0.S[9213]++;letter = substituteChar;}
            else{
                __CLOVER_409_0.S[9214]++;letter = (char) b;}

            __CLOVER_409_0.S[9215]++;ascii[lineCount] = letter;
        }}

        __CLOVER_409_0.S[9216]++;lineCount++;
        __CLOVER_409_0.S[9217]++;bytesSinceSpace++;
    } finally { }}

    private void writeHex(int value, int digits)
    {try { __CLOVER_409_0.M[2223]++;
        __CLOVER_409_0.S[9218]++;int i;
        __CLOVER_409_0.S[9219]++;int nybble;

        __CLOVER_409_0.S[9220]++;for (i = 0; (((i < digits) && (++__CLOVER_409_0.CT[1566] != 0)) || (++__CLOVER_409_0.CF[1566] == 0)); i++){
        {
            __CLOVER_409_0.S[9221]++;nybble = (value >> 4 * (digits - i - 1)) & 0x0f;

            __CLOVER_409_0.S[9222]++;out.print(HEX[nybble]);
        }}
    } finally { }}
}