/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.util.prop;

import java.util.HashMap;
import java.util.Map;

import ognl.ClassResolver;
import ognl.Ognl;
import ognl.OgnlException;

import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.tapestry.Tapestry;

/**
 *  Utilities wrappers around <a href="http://www.ognl.org">OGNL</a>.
 * 
 *  <p>This has been temporarily hacked for Tapestry 3.1. It's not a good solution,
 *  because it only really supports a single class loader, which could be a problem
 *  if the Tapestry framework is loaded by the system class loader.
 * 
 *  @author Howard Lewis Ship
 *  @since 2.2
 * 
 **/

public class OgnlUtils
{public static com.cortexeb.tools.clover.d __CLOVER_426_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private static final Map _cache = new HashMap();

    /**
     * @since 3.1
     */
    private static ClassResolver _ognlClassResolver = new OgnlClassResolver();

    private OgnlUtils()
    {try { __CLOVER_426_0.M[2282]++;
    } finally { }}

    /**
     *  Gets a parsed OGNL expression from the input string.
     * 
     *  @throws ApplicationRuntimeException if the expression can not be parsed.
     * 
     **/

    public static synchronized Object getParsedExpression(String expression)
    {try { __CLOVER_426_0.M[2283]++;
        __CLOVER_426_0.S[9463]++;Object result = _cache.get(expression);

        __CLOVER_426_0.S[9464]++;if ((((result == null) && (++__CLOVER_426_0.CT[1614] != 0)) || (++__CLOVER_426_0.CF[1614] == 0))){
        {
            __CLOVER_426_0.S[9465]++;try
            {
                __CLOVER_426_0.S[9466]++;result = Ognl.parseExpression(expression);
            }
            catch (Exception ex)
            {
                __CLOVER_426_0.S[9467]++;throw new ApplicationRuntimeException(
                    Tapestry.format("OgnlUtils.unable-to-parse-expression", expression),
                    ex);
            }

            __CLOVER_426_0.S[9468]++;_cache.put(expression, result);
        }}

        __CLOVER_426_0.S[9469]++;return result;
    } finally { }}

    /**
     *  Parses and caches the expression and uses it to update
     *  the target object with the provided value.
     * 
     *  @throws ApplicationRuntimeException if the expression
     *  can not be parsed, or the target can not be updated.
     * 
     **/

    public static void set(String expression, Object target, Object value)
    {try { __CLOVER_426_0.M[2284]++;
        __CLOVER_426_0.S[9470]++;set(getParsedExpression(expression), target, value);
    } finally { }}

    /** 
     *  Updates the target object with the provided value.
     * 
     *  @param expression a parsed OGNL expression
     *  @throws ApplicationRuntimeException if the target can not be updated.
     * 
     **/

    public static void set(Object expression, Object target, Object value)
    {try { __CLOVER_426_0.M[2285]++;
        __CLOVER_426_0.S[9471]++;try
        {
            __CLOVER_426_0.S[9472]++;Map context = Ognl.createDefaultContext(target, _ognlClassResolver);

            __CLOVER_426_0.S[9473]++;Ognl.setValue(expression, context, target, value);
        }
        catch (Exception ex)
        {
            __CLOVER_426_0.S[9474]++;throw new ApplicationRuntimeException(
                Tapestry.format(
                    "OgnlUtils.unable-to-update-expression",
                    "<parsed expression>",
                    target,
                    value),
                ex);
        }
    } finally { }}

    /**
     *   Returns the value of the expression evaluated against
     *   the object.
     * 
     *   @param expression a parsed OGNL expression
     *   @param object the root object
     * 
     *   @throws ApplicationRuntimeException
     *   if the value can not be obtained from the object.
     * 
     **/

    public static Object get(Object expression, Object object)
    {try { __CLOVER_426_0.M[2286]++;
        __CLOVER_426_0.S[9475]++;try
        {
            __CLOVER_426_0.S[9476]++;Map context = Ognl.createDefaultContext(object, _ognlClassResolver);

            __CLOVER_426_0.S[9477]++;return Ognl.getValue(expression, context, object);
        }
        catch (Exception ex)
        {
            __CLOVER_426_0.S[9478]++;throw new ApplicationRuntimeException(
                Tapestry.format(
                    "OgnlUtils.unable-to-read-expression",
                    "<parsed expression>",
                    object),
                ex);
        }
    } finally { }}

	/**
	 * @return true if the expession evaluates to a constant or literal value. 
	 *
	 * @since 3.1
	 */
    public static boolean isConstant(String expression)
    {try { __CLOVER_426_0.M[2287]++;
        __CLOVER_426_0.S[9479]++;return isConstant(getParsedExpression(expression));
    } finally { }}

	/**
	 * @return true if the parsed expression evaluates to a constant or
	 * literal value
	 * 
	 * @since 3.1
	 */
    public static boolean isConstant(Object expression)
    {try { __CLOVER_426_0.M[2288]++;
        __CLOVER_426_0.S[9480]++;try
        {
            __CLOVER_426_0.S[9481]++;return Ognl.isConstant(expression);
        }
        catch (OgnlException ex)
        {
            __CLOVER_426_0.S[9482]++;throw new ApplicationRuntimeException(
                Tapestry.format("OgnlUtils.unable-to-parse-expression", expression),
                ex);
        }
    } finally { }}

    /**
     *   Returns the value of the expression evaluated against
     *   the object.
     * 
     * 
     *   @throws ApplicationRuntimeException if the
     *   expression can not be parsed, or the value
     *   not obtained from the object.
     **/

    public static Object get(String expression, Object object)
    {try { __CLOVER_426_0.M[2289]++;
        __CLOVER_426_0.S[9483]++;return get(getParsedExpression(expression), object);
    } finally { }}

    /**
     * @since 3.1
     */
    public static ClassResolver getOgnlClassResolver()
    {try { __CLOVER_426_0.M[2290]++;
        __CLOVER_426_0.S[9484]++;return _ognlClassResolver;
    } finally { }}
    /**
     * @since 3.1
     */
    public static void setOgnlClassResolver(ClassResolver resolver)
    {try { __CLOVER_426_0.M[2291]++;
        __CLOVER_426_0.S[9485]++;_ognlClassResolver = resolver;
    } finally { }}

}
