/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.util.text;

import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.Map;

/**
 * A version of java.util.Properties that can read the properties from files
 * using an encoding other than ISO-8859-1. All non-latin characters are read 
 * correctly using the given encoding and no longer need to be quoted using native2ascii.
 * 
 * In addition, the properties may be stored in an arbitrary map, rather than
 * only in Properties. For example, using LinkedHashMap will preserve the order
 * of the properties as defined in the file. 
 * 
 * @author mb
 * @since 3.1
 */
public class LocalizedProperties
{public static com.cortexeb.tools.clover.d __CLOVER_436_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private Map _propertyMap;
    
    /**
     * Create a new object with an empty property storage 
     */
    public LocalizedProperties()
    {
        this(new HashMap());__CLOVER_436_0.S[9578]++;try { __CLOVER_436_0.M[2313]++;
    } finally { }}

    /**
     * Use the provided argument as the storage location for the properties managed
     * by this object. This allows different types of Map implementations to be used,
     * such as a LinkedHashMap to preserve the order of the keys, for example.
     * The provided map may contain the default property values as well.
     * 
     * @param propertyMap the map where properties are to be stored
     */
    public LocalizedProperties(Map propertyMap)
    {try { __CLOVER_436_0.M[2314]++;
        __CLOVER_436_0.S[9579]++;_propertyMap = propertyMap;
    } finally { }}
    
    /**
     * Returns the property value corresponding the provided key. If there is no such property,
     * or the value in the provided map is not of type String, null is returned.
     * 
     * @param key the property key
     * @return the value of the property, or null if there is no such property
     */
    public String getProperty(String key)
    {try { __CLOVER_436_0.M[2315]++;
       __CLOVER_436_0.S[9580]++;Object value = _propertyMap.get(key);
       __CLOVER_436_0.S[9581]++;if ((((value instanceof String) && (++__CLOVER_436_0.CT[1637] != 0)) || (++__CLOVER_436_0.CF[1637] == 0))){
           __CLOVER_436_0.S[9582]++;return (String) value;}
       __CLOVER_436_0.S[9583]++;return null;
    } finally { }}
    
    /**
     * Returns the property value corresponding to the provided key, 
     * or the provided default value if no such property exists.
     * 
     * @param key the property key
     * @param defaultValue the default value of the property
     * @return the value of the property, or the default value if there is no such property
     */
    public String getProperty(String key, String defaultValue)
    {try { __CLOVER_436_0.M[2316]++;
        __CLOVER_436_0.S[9584]++;String value = getProperty(key);
        __CLOVER_436_0.S[9585]++;if ((((value != null) && (++__CLOVER_436_0.CT[1638] != 0)) || (++__CLOVER_436_0.CF[1638] == 0))){
            __CLOVER_436_0.S[9586]++;return value;}
        __CLOVER_436_0.S[9587]++;return defaultValue;
    } finally { }}
    
    /**
     * Stores a property value
     * 
     * @param key the property key
     * @param value the property value
     */
    public void setProperty(String key, String value)
    {try { __CLOVER_436_0.M[2317]++;
        __CLOVER_436_0.S[9588]++;_propertyMap.put(key, value);
    } finally { }}
    
    /**
     * Returns the map containing all properties. The map can be used to enumerate the 
     * properties or their keys. 
     * 
     * @return a map containing the properties
     */
    public Map getPropertyMap()
    {try { __CLOVER_436_0.M[2318]++;
        __CLOVER_436_0.S[9589]++;return _propertyMap;
    } finally { }}
    
    /**
     * Loads the properties from the given stream using the default character encoding.
     * This method operates in the same way as the equivalent method in {@link java.util.Properties},
     * but it also handles non-ascii symbols. 
     * 
     * @param ins the stream to load the properties from
     * @throws IOException
     */
    public void load(InputStream ins) throws IOException
    {try { __CLOVER_436_0.M[2319]++;
    	__CLOVER_436_0.S[9590]++;LocalizedPropertiesLoader loader = new LocalizedPropertiesLoader(ins);
        __CLOVER_436_0.S[9591]++;loader.load(_propertyMap);
    } finally { }}
    
    /**
     * Loads the properties from the given stream using the provided character encoding.
     * This method operates in the same way as the equivalent method in {@link java.util.Properties},
     * but it also handles non-ascii symbols. 
     * 
     * @param ins the stream to load the properties from
     * @param encoding the encoding the use when parsing the stream
     * @throws IOException
     */
    public void load(InputStream ins, String encoding) throws UnsupportedEncodingException, IOException
    {try { __CLOVER_436_0.M[2320]++;
    	__CLOVER_436_0.S[9592]++;LocalizedPropertiesLoader loader = new LocalizedPropertiesLoader(ins, encoding);
        __CLOVER_436_0.S[9593]++;loader.load(_propertyMap);
    } finally { }}
    
    /**
     * Loads the properties from the given reader.
     * This method operates in the same way as the equivalent method in {@link java.util.Properties},
     * but it also handles non-ascii symbols. 
     * 
     * @param reader the reader to load the properties from
     * @throws IOException
     */
    public void load(Reader reader) throws IOException
    {try { __CLOVER_436_0.M[2321]++;
    	__CLOVER_436_0.S[9594]++;LocalizedPropertiesLoader loader = new LocalizedPropertiesLoader(reader);
        __CLOVER_436_0.S[9595]++;loader.load(_propertyMap);
    } finally { }}
}
