/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.util.xml;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.hivemind.Location;
import org.apache.hivemind.Resource;
import org.apache.hivemind.impl.LocationImpl;
import org.apache.tapestry.Tapestry;
import org.apache.tapestry.util.RegexpMatcher;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * A simplified version of {@link org.apache.commons.digester.Digester}.
 * This version is without as many bells and whistles but has some key features needed when parsing
 * a document (rather than a configuration file):
 * <br>
 * <ul>
 *   <li>Notifications for each bit of text</ul>
 *   <li>Tracking of exact location within the document.</li>
 * </ul>
 * 
 * <p>
 * Like Digester, there's an object stack and a rule stack.  The rules are much
 * simpler (more coding), in that there's a one-to-one relationship between
 * an element and a rule.
 *
 * <p>
 *  Based on SAX2.
 * 
 * @author Howard Lewis Ship
 * @since 3.0
 */

public class RuleDirectedParser extends DefaultHandler
{public static com.cortexeb.tools.clover.d __CLOVER_444_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private static final Log LOG = LogFactory.getLog(RuleDirectedParser.class);

    private Resource _documentLocation;
    private List _ruleStack = new ArrayList();
    private List _objectStack = new ArrayList();
    private Object _documentObject;

    private Locator _locator;
    private int _line = -1;
    private int _column = -1;
    private Location _location;

    private static SAXParserFactory _parserFactory;
    private SAXParser _parser;

    private RegexpMatcher _matcher;

    private String _uri;
    private String _localName;
    private String _qName;

    /**
     *  Map of {@link IRule} keyed on the local name
     *  of the element.
     */
    private Map _ruleMap = new HashMap();

    /**
     * Used to accumlate content provided by
     * {@link org.xml.sax.ContentHandler#characters(char[], int, int)}.
     */

    private StringBuffer _contentBuffer = new StringBuffer();

    /**
     *  Map of paths to external entities (such as the DTD) keyed on public id.
     */

    private Map _entities = new HashMap();

    public Object parse(Resource documentLocation)
    {try { __CLOVER_444_0.M[2355]++;
        __CLOVER_444_0.S[9707]++;if ((((LOG.isDebugEnabled()) && (++__CLOVER_444_0.CT[1662] != 0)) || (++__CLOVER_444_0.CF[1662] == 0))){
            __CLOVER_444_0.S[9708]++;LOG.debug("Parsing: " + documentLocation);}

        __CLOVER_444_0.S[9709]++;try
        {
            __CLOVER_444_0.S[9710]++;_documentLocation = documentLocation;

            __CLOVER_444_0.S[9711]++;URL url = documentLocation.getResourceURL();

            __CLOVER_444_0.S[9712]++;if ((((url == null) && (++__CLOVER_444_0.CT[1663] != 0)) || (++__CLOVER_444_0.CF[1663] == 0))){
                __CLOVER_444_0.S[9713]++;throw new DocumentParseException(
                    Tapestry.format("RuleDrivenParser.resource-missing", documentLocation),
                    documentLocation,
                    null);}

            __CLOVER_444_0.S[9714]++;return parse(url);
        }
        finally
        {
            __CLOVER_444_0.S[9715]++;_documentLocation = null;
            __CLOVER_444_0.S[9716]++;_ruleStack.clear();
            __CLOVER_444_0.S[9717]++;_objectStack.clear();
            __CLOVER_444_0.S[9718]++;_documentObject = null;

            __CLOVER_444_0.S[9719]++;_uri = null;
            __CLOVER_444_0.S[9720]++;_localName = null;
            __CLOVER_444_0.S[9721]++;_qName = null;

            __CLOVER_444_0.S[9722]++;_line = -1;
            __CLOVER_444_0.S[9723]++;_column = -1;
            __CLOVER_444_0.S[9724]++;_location = null;
            __CLOVER_444_0.S[9725]++;_locator = null;

            __CLOVER_444_0.S[9726]++;_contentBuffer.setLength(0);
        }
    } finally { }}

    protected Object parse(URL url)
    {try { __CLOVER_444_0.M[2356]++;
        __CLOVER_444_0.S[9727]++;if ((((_parser == null) && (++__CLOVER_444_0.CT[1664] != 0)) || (++__CLOVER_444_0.CF[1664] == 0))){
            __CLOVER_444_0.S[9728]++;_parser = constructParser();}

        __CLOVER_444_0.S[9729]++;InputStream stream = null;

        __CLOVER_444_0.S[9730]++;try
        {
            __CLOVER_444_0.S[9731]++;stream = url.openStream();
        }
        catch (IOException ex)
        {
            __CLOVER_444_0.S[9732]++;throw new DocumentParseException(
                Tapestry.format("RuleDrivenParser.unable-to-open-resource", url),
                _documentLocation,
                ex);
        }

        __CLOVER_444_0.S[9733]++;InputSource source = new InputSource(stream);

        __CLOVER_444_0.S[9734]++;try
        {
            __CLOVER_444_0.S[9735]++;_parser.parse(source, this);

            __CLOVER_444_0.S[9736]++;stream.close();
        }
        catch (Exception ex)
        {
            __CLOVER_444_0.S[9737]++;throw new DocumentParseException(
                Tapestry.format("RuleDrivenParser.parse-error", url, ex.getMessage()),
                getLocation(),
                ex);
        }

        __CLOVER_444_0.S[9738]++;if ((((LOG.isDebugEnabled()) && (++__CLOVER_444_0.CT[1665] != 0)) || (++__CLOVER_444_0.CF[1665] == 0))){
            __CLOVER_444_0.S[9739]++;LOG.debug("Document parsed as: " + _documentObject);}

        __CLOVER_444_0.S[9740]++;return _documentObject;
    } finally { }}

    /**
     * Returns an {@link ILocation} representing the
     * current position within the document (depending
     * on the parser, this may be accurate to
     * column number level).
     */

    public Location getLocation()
    {try { __CLOVER_444_0.M[2357]++;
        __CLOVER_444_0.S[9741]++;if ((((_locator == null) && (++__CLOVER_444_0.CT[1666] != 0)) || (++__CLOVER_444_0.CF[1666] == 0))){
            __CLOVER_444_0.S[9742]++;return null;}

        __CLOVER_444_0.S[9743]++;int line = _locator.getLineNumber();
        __CLOVER_444_0.S[9744]++;int column = _locator.getColumnNumber();

        __CLOVER_444_0.S[9745]++;if ((((_line != line || _column != column) && (++__CLOVER_444_0.CT[1667] != 0)) || (++__CLOVER_444_0.CF[1667] == 0))){
        {
            __CLOVER_444_0.S[9746]++;_location = null;
            __CLOVER_444_0.S[9747]++;_line = line;
            __CLOVER_444_0.S[9748]++;_column = column;
        }}

        __CLOVER_444_0.S[9749]++;if ((((_location == null) && (++__CLOVER_444_0.CT[1668] != 0)) || (++__CLOVER_444_0.CF[1668] == 0))){
            __CLOVER_444_0.S[9750]++;_location = new LocationImpl(_documentLocation, _line, _column);}

        __CLOVER_444_0.S[9751]++;return _location;
    } finally { }}

    /**
     * Pushes an object onto the object stack.  The first object
     * pushed is the "document object", the root object returned
     * by the parse.
     */
    public void push(Object object)
    {try { __CLOVER_444_0.M[2358]++;
        __CLOVER_444_0.S[9752]++;if ((((_documentObject == null) && (++__CLOVER_444_0.CT[1669] != 0)) || (++__CLOVER_444_0.CF[1669] == 0))){
            __CLOVER_444_0.S[9753]++;_documentObject = object;}

        __CLOVER_444_0.S[9754]++;push(_objectStack, object, "object stack");
    } finally { }}

    /**
     * Returns the top object on the object stack.
     */
    public Object peek()
    {try { __CLOVER_444_0.M[2359]++;
        __CLOVER_444_0.S[9755]++;return peek(_objectStack, 0);
    } finally { }}

    /**
     * Returns an object within the object stack, at depth.
     * Depth 0 is the top object, depth 1 is the next-to-top object,
     * etc.
     */

    public Object peek(int depth)
    {try { __CLOVER_444_0.M[2360]++;
        __CLOVER_444_0.S[9756]++;return peek(_objectStack, depth);
    } finally { }}

    /**
     * Removes and returns the top object on the object stack.
     */
    public Object pop()
    {try { __CLOVER_444_0.M[2361]++;
        __CLOVER_444_0.S[9757]++;return pop(_objectStack, "object stack");
    } finally { }}

    private Object pop(List list, String name)
    {try { __CLOVER_444_0.M[2362]++;
        __CLOVER_444_0.S[9758]++;Object result = list.remove(list.size() - 1);

        __CLOVER_444_0.S[9759]++;if ((((LOG.isDebugEnabled()) && (++__CLOVER_444_0.CT[1670] != 0)) || (++__CLOVER_444_0.CF[1670] == 0))){
            __CLOVER_444_0.S[9760]++;LOG.debug("Popped " + result + " off " + name + " (at " + getLocation() + ")");}

        __CLOVER_444_0.S[9761]++;return result;
    } finally { }}

    private Object peek(List list, int depth)
    {try { __CLOVER_444_0.M[2363]++;
        __CLOVER_444_0.S[9762]++;return list.get(list.size() - 1 - depth);
    } finally { }}

    private void push(List list, Object object, String name)
    {try { __CLOVER_444_0.M[2364]++;
        __CLOVER_444_0.S[9763]++;if ((((LOG.isDebugEnabled()) && (++__CLOVER_444_0.CT[1671] != 0)) || (++__CLOVER_444_0.CF[1671] == 0))){
            __CLOVER_444_0.S[9764]++;LOG.debug("Pushing " + object + " onto " + name + " (at " + getLocation() + ")");}

        __CLOVER_444_0.S[9765]++;list.add(object);
    } finally { }}

    /**
     * Pushes a new rule onto the rule stack.
     */

    protected void pushRule(IRule rule)
    {try { __CLOVER_444_0.M[2365]++;
        __CLOVER_444_0.S[9766]++;push(_ruleStack, rule, "rule stack");
    } finally { }}

    /**
     * Returns the top rule on the stack.
     */

    protected IRule peekRule()
    {try { __CLOVER_444_0.M[2366]++;
        __CLOVER_444_0.S[9767]++;return (IRule) peek(_ruleStack, 0);
    } finally { }}

    protected IRule popRule()
    {try { __CLOVER_444_0.M[2367]++;
        __CLOVER_444_0.S[9768]++;return (IRule) pop(_ruleStack, "rule stack");
    } finally { }}

    public void addRule(String localElementName, IRule rule)
    {try { __CLOVER_444_0.M[2368]++;
        __CLOVER_444_0.S[9769]++;_ruleMap.put(localElementName, rule);
    } finally { }}

    /**
     *  Registers
     *  a public id and corresponding input source.  Generally, the source
     *  is a wrapper around an input stream to a package resource.
     *
     *  @param publicId the public identifier to be registerred, generally
     *  the publicId of a DTD related to the document being parsed
     *  @param entityPath the resource path of the entity, typically a DTD
     *  file.  Relative files names are expected to be stored in the same package
     *  as the class file, otherwise a leading slash is an absolute pathname
     *  within the classpath.
     *
     **/

    public void registerEntity(String publicId, String entityPath)
    {try { __CLOVER_444_0.M[2369]++;
        __CLOVER_444_0.S[9770]++;if ((((LOG.isDebugEnabled()) && (++__CLOVER_444_0.CT[1672] != 0)) || (++__CLOVER_444_0.CF[1672] == 0))){
            __CLOVER_444_0.S[9771]++;LOG.debug("Registering " + publicId + " as " + entityPath);}

        __CLOVER_444_0.S[9772]++;if ((((_entities == null) && (++__CLOVER_444_0.CT[1673] != 0)) || (++__CLOVER_444_0.CF[1673] == 0))){
            __CLOVER_444_0.S[9773]++;_entities = new HashMap();}

        __CLOVER_444_0.S[9774]++;_entities.put(publicId, entityPath);
    } finally { }}

    protected IRule selectRule(String localName, Attributes attributes)
    {try { __CLOVER_444_0.M[2370]++;
        __CLOVER_444_0.S[9775]++;IRule rule = (IRule) _ruleMap.get(localName);

        __CLOVER_444_0.S[9776]++;if ((((rule == null) && (++__CLOVER_444_0.CT[1674] != 0)) || (++__CLOVER_444_0.CF[1674] == 0))){
            __CLOVER_444_0.S[9777]++;throw new DocumentParseException(
                Tapestry.format("RuleDrivenParser.no-rule-for-element", localName),
                getLocation(),
                null);}

        __CLOVER_444_0.S[9778]++;return rule;
    } finally { }}

    /**
     * Uses the {@link Locator} to track the position
     * in the document as a {@link ILocation}. This is invoked
     * once (before the initial element is parsed) and
     * the Locator is retained and queried as to
     * the current file location.
     * 
     * @see #getLocation()
     */
    public void setDocumentLocator(Locator locator)
    {try { __CLOVER_444_0.M[2371]++;
        __CLOVER_444_0.S[9779]++;_locator = locator;
    } finally { }}

    /**
     * Accumulates the content in a buffer; the concatinated content
     * is provided to the top rule just before any start or end tag. 
     */
    public void characters(char[] ch, int start, int length) throws SAXException
    {try { __CLOVER_444_0.M[2372]++;
        __CLOVER_444_0.S[9780]++;_contentBuffer.append(ch, start, length);
    } finally { }}

    /**
     * Pops the top rule off the stack and
     * invokes {@link IRule#endElementt(RuleDirectedParser)}.
     */
    public void endElement(String uri, String localName, String qName) throws SAXException
    {try { __CLOVER_444_0.M[2373]++;
        __CLOVER_444_0.S[9781]++;fireContentRule();

        __CLOVER_444_0.S[9782]++;_uri = uri;
        __CLOVER_444_0.S[9783]++;_localName = localName;
        __CLOVER_444_0.S[9784]++;_qName = qName;

        __CLOVER_444_0.S[9785]++;popRule().endElement(this);
    } finally { }}

    /**
     * Ignorable content is ignored.
     */
    public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException
    {try { __CLOVER_444_0.M[2374]++;
    } finally { }}

    /**
     * Invokes {@link #selectRule(String, Attributes)} to choose a new rule,
     * which is pushed onto the rule stack, then invokes
     * {@link IRule#startElement(RuleDirectedParser, Attributes)}.
     */
    public void startElement(String uri, String localName, String qName, Attributes attributes)
        throws SAXException
    {try { __CLOVER_444_0.M[2375]++;
        __CLOVER_444_0.S[9786]++;fireContentRule();

        __CLOVER_444_0.S[9787]++;_uri = uri;
        __CLOVER_444_0.S[9788]++;_localName = localName;
        __CLOVER_444_0.S[9789]++;_qName = qName;

        __CLOVER_444_0.S[9790]++;String name = extractName(uri, localName, qName);

        __CLOVER_444_0.S[9791]++;IRule newRule = selectRule(name, attributes);

        __CLOVER_444_0.S[9792]++;pushRule(newRule);

        __CLOVER_444_0.S[9793]++;newRule.startElement(this, attributes);
    } finally { }}

    private String extractName(String uri, String localName, String qName)
    {try { __CLOVER_444_0.M[2376]++;
        __CLOVER_444_0.S[9794]++;return (((Tapestry.isBlank(localName) ) && (++__CLOVER_444_0.CT[1675] != 0)) || (++__CLOVER_444_0.CF[1675] == 0))? qName : localName;
    } finally { }}

    /**
     * Uses {@link javax.xml.parsers.SAXParserFactory} to create a instance
     * of a validation SAX2 parser.
     */
    protected synchronized SAXParser constructParser()
    {try { __CLOVER_444_0.M[2377]++;
        __CLOVER_444_0.S[9795]++;if ((((_parserFactory == null) && (++__CLOVER_444_0.CT[1676] != 0)) || (++__CLOVER_444_0.CF[1676] == 0))){
        {
            __CLOVER_444_0.S[9796]++;_parserFactory = SAXParserFactory.newInstance();
            __CLOVER_444_0.S[9797]++;configureParserFactory(_parserFactory);
        }}

        __CLOVER_444_0.S[9798]++;try
        {
            __CLOVER_444_0.S[9799]++;return _parserFactory.newSAXParser();
        }
        catch (SAXException ex)
        {
            __CLOVER_444_0.S[9800]++;throw new ApplicationRuntimeException(ex);
        }
        catch (ParserConfigurationException ex)
        {
            __CLOVER_444_0.S[9801]++;throw new ApplicationRuntimeException(ex);
        }

    } finally { }}

    /**
     * Configures a {@link SAXParserFactory} before
     * {@link SAXParserFactory#newSAXParser()} is invoked.
     * The default implementation sets validating to true
     * and namespaceAware to false,
     */

    protected void configureParserFactory(SAXParserFactory factory)
    {try { __CLOVER_444_0.M[2378]++;
        __CLOVER_444_0.S[9802]++;factory.setValidating(true);
        __CLOVER_444_0.S[9803]++;factory.setNamespaceAware(false);
    } finally { }}

    /**
     *  Throws the exception.
     */
    public void error(SAXParseException ex) throws SAXException
    {try { __CLOVER_444_0.M[2379]++;
        __CLOVER_444_0.S[9804]++;fatalError(ex);
    } finally { }}

    /**
     *  Throws the exception.
     */
    public void fatalError(SAXParseException ex) throws SAXException
    {try { __CLOVER_444_0.M[2380]++;
        // Sometimes, a bad parse "corrupts" a parser so that it doesn't
        // work properly for future parses (of valid documents),
        // so discard it here.

        __CLOVER_444_0.S[9805]++;_parser = null;

        __CLOVER_444_0.S[9806]++;throw ex;
    } finally { }}

    /**
     *  Throws the exception.
     */
    public void warning(SAXParseException ex) throws SAXException
    {try { __CLOVER_444_0.M[2381]++;
        __CLOVER_444_0.S[9807]++;fatalError(ex);
    } finally { }}

    public InputSource resolveEntity(String publicId, String systemId) throws SAXException
    {try { __CLOVER_444_0.M[2382]++;
        __CLOVER_444_0.S[9808]++;String entityPath = null;

        __CLOVER_444_0.S[9809]++;if ((((LOG.isDebugEnabled()) && (++__CLOVER_444_0.CT[1677] != 0)) || (++__CLOVER_444_0.CF[1677] == 0))){
            __CLOVER_444_0.S[9810]++;LOG.debug(
                "Attempting to resolve entity; publicId = " + publicId + " systemId = " + systemId);}

        __CLOVER_444_0.S[9811]++;if ((((_entities != null) && (++__CLOVER_444_0.CT[1678] != 0)) || (++__CLOVER_444_0.CF[1678] == 0))){
            __CLOVER_444_0.S[9812]++;entityPath = (String) _entities.get(publicId);}

        __CLOVER_444_0.S[9813]++;if ((((entityPath == null) && (++__CLOVER_444_0.CT[1679] != 0)) || (++__CLOVER_444_0.CF[1679] == 0))){
        {
            __CLOVER_444_0.S[9814]++;if ((((LOG.isDebugEnabled()) && (++__CLOVER_444_0.CT[1680] != 0)) || (++__CLOVER_444_0.CF[1680] == 0))){
                __CLOVER_444_0.S[9815]++;LOG.debug("Entity not found, using " + systemId);}

            __CLOVER_444_0.S[9816]++;return null;
        }}

        __CLOVER_444_0.S[9817]++;InputStream stream = getClass().getResourceAsStream(entityPath);

        __CLOVER_444_0.S[9818]++;InputSource result = new InputSource(stream);

        __CLOVER_444_0.S[9819]++;if ((((result != null && LOG.isDebugEnabled()) && (++__CLOVER_444_0.CT[1681] != 0)) || (++__CLOVER_444_0.CF[1681] == 0))){
            __CLOVER_444_0.S[9820]++;LOG.debug("Resolved " + publicId + " as " + result + " (for " + entityPath + ")");}

        __CLOVER_444_0.S[9821]++;return result;
    } finally { }}

    /**
     *  Validates that the input value matches against the specified
     *  Perl5 pattern.  If valid, the method simply returns.
     *  If not a match, then an error message is generated (using the
     *  errorKey and the input value) and a
     *  {@link InvalidStringException} is thrown.
     * 
     **/

    public void validate(String value, String pattern, String errorKey)
        throws DocumentParseException
    {try { __CLOVER_444_0.M[2383]++;
        __CLOVER_444_0.S[9822]++;if ((((_matcher == null) && (++__CLOVER_444_0.CT[1682] != 0)) || (++__CLOVER_444_0.CF[1682] == 0))){
            __CLOVER_444_0.S[9823]++;_matcher = new RegexpMatcher();}

        __CLOVER_444_0.S[9824]++;if ((((_matcher.matches(pattern, value)) && (++__CLOVER_444_0.CT[1683] != 0)) || (++__CLOVER_444_0.CF[1683] == 0))){
            __CLOVER_444_0.S[9825]++;return;}

        __CLOVER_444_0.S[9826]++;throw new InvalidStringException(Tapestry.format(errorKey, value), value, getLocation());
    } finally { }}

    public Resource getDocumentLocation()
    {try { __CLOVER_444_0.M[2384]++;
        __CLOVER_444_0.S[9827]++;return _documentLocation;
    } finally { }}

    /**
     * Returns the localName for the current element.
     * @see org.xml.sax.ContentHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
     */
    public String getLocalName()
    {try { __CLOVER_444_0.M[2385]++;
        __CLOVER_444_0.S[9828]++;return _localName;
    } finally { }}

    /**
     * Returns the qualified name for the current element.
     * @see org.xml.sax.ContentHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
     */
    public String getQName()
    {try { __CLOVER_444_0.M[2386]++;
        __CLOVER_444_0.S[9829]++;return _qName;
    } finally { }}

    /**
     * Returns the URI for the current element.
     * @see org.xml.sax.ContentHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes) 
     */
    public String getUri()
    {try { __CLOVER_444_0.M[2387]++;
        __CLOVER_444_0.S[9830]++;return _uri;
    } finally { }}

    private void fireContentRule()
    {try { __CLOVER_444_0.M[2388]++;
        __CLOVER_444_0.S[9831]++;String content = _contentBuffer.toString();
        __CLOVER_444_0.S[9832]++;_contentBuffer.setLength(0);

        __CLOVER_444_0.S[9833]++;if ((((!_ruleStack.isEmpty()) && (++__CLOVER_444_0.CT[1684] != 0)) || (++__CLOVER_444_0.CF[1684] == 0))){
            __CLOVER_444_0.S[9834]++;peekRule().content(this, content);}
    } finally { }}

}
