/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.valid;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Map;

import org.apache.tapestry.IMarkupWriter;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.form.IFormComponent;

/**
 *  Provides input validation for strings treated as dates.  In addition,
 *  allows a minimum and maximum date to be set.
 *
 *  @author Howard Lewis Ship
 *  @since 1.0.8
 *
 **/

public class DateValidator extends BaseValidator
{public static com.cortexeb.tools.clover.d __CLOVER_446_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private DateFormat _format;
    private String _displayFormat;
    private Date _minimum;
    private Date _maximum;
    private Calendar _calendar;
    private String _scriptPath = "/org/apache/tapestry/valid/DateValidator.script";

    private static DateFormat defaultDateFormat = new SimpleDateFormat("MM/dd/yyyy");
    private static final String defaultDateDisplayFormat = "MM/DD/YYYY";

    private String _dateTooEarlyMessage;
    private String _dateTooLateMessage;
    private String _invalidDateFormatMessage;

    public void setFormat(DateFormat value)
    {try { __CLOVER_446_0.M[2405]++;
        __CLOVER_446_0.S[9870]++;_format = value;
    } finally { }}

    public DateFormat getFormat()
    {try { __CLOVER_446_0.M[2406]++;
        __CLOVER_446_0.S[9871]++;return _format;
    } finally { }}

	/**
	 * @return the {@link DateFormat} the validator will use, returning the default if no
	 * other date format is specified via {@link #setFormat(DateFormat)}
	 * 
	 * @since 3.0
	 */
    public DateFormat getEffectiveFormat()
    {try { __CLOVER_446_0.M[2407]++;
        __CLOVER_446_0.S[9872]++;if ((((_format == null) && (++__CLOVER_446_0.CT[1689] != 0)) || (++__CLOVER_446_0.CF[1689] == 0))){
            __CLOVER_446_0.S[9873]++;return defaultDateFormat;}

        __CLOVER_446_0.S[9874]++;return _format;
    } finally { }}

    public String getDisplayFormat()
    {try { __CLOVER_446_0.M[2408]++;
        __CLOVER_446_0.S[9875]++;return _displayFormat;
    } finally { }}

    public void setDisplayFormat(String value)
    {try { __CLOVER_446_0.M[2409]++;
        __CLOVER_446_0.S[9876]++;_displayFormat = value;
    } finally { }}

	/**
     * @return the display format message the validator will use, returning the default if no
     * other display format message is specified.  The default is the {@link SimpleDateFormat#toPattern()}
     * for {@link SimpleDateFormat}s, or "MM/DD/YYYY" for unknown {@link DateFormat} subclasses.
     * 
     * @since 3.0
     */
    public String getEffectiveDisplayFormat()
    {try { __CLOVER_446_0.M[2410]++;
        __CLOVER_446_0.S[9877]++;if ((((_displayFormat == null) && (++__CLOVER_446_0.CT[1690] != 0)) || (++__CLOVER_446_0.CF[1690] == 0))){
        {
            __CLOVER_446_0.S[9878]++;DateFormat format = getEffectiveFormat();
            __CLOVER_446_0.S[9879]++;if ((((format instanceof SimpleDateFormat) && (++__CLOVER_446_0.CT[1691] != 0)) || (++__CLOVER_446_0.CF[1691] == 0))){ 
                __CLOVER_446_0.S[9880]++;return ((SimpleDateFormat)format).toPattern();}
            else{
                __CLOVER_446_0.S[9881]++;return defaultDateDisplayFormat;}
        }}

        __CLOVER_446_0.S[9882]++;return _displayFormat;
    } finally { }}

    public String toString(IFormComponent file, Object value)
    {try { __CLOVER_446_0.M[2411]++;
        __CLOVER_446_0.S[9883]++;if ((((value == null) && (++__CLOVER_446_0.CT[1692] != 0)) || (++__CLOVER_446_0.CF[1692] == 0))){
            __CLOVER_446_0.S[9884]++;return null;}

        __CLOVER_446_0.S[9885]++;Date date = (Date) value;

        __CLOVER_446_0.S[9886]++;DateFormat format = getEffectiveFormat();

        // DateFormat is not threadsafe, so guard access to it.

        __CLOVER_446_0.S[9887]++;synchronized (format)
        {
            __CLOVER_446_0.S[9888]++;return format.format(date);
        }
    } finally { }}

    public Object toObject(IFormComponent field, String value) throws ValidatorException
    {try { __CLOVER_446_0.M[2412]++;
        __CLOVER_446_0.S[9889]++;if ((((checkRequired(field, value)) && (++__CLOVER_446_0.CT[1693] != 0)) || (++__CLOVER_446_0.CF[1693] == 0))){
            __CLOVER_446_0.S[9890]++;return null;}

        __CLOVER_446_0.S[9891]++;DateFormat format = getEffectiveFormat();

        __CLOVER_446_0.S[9892]++;Date result;

        __CLOVER_446_0.S[9893]++;try
        {
            // DateFormat is not threadsafe, so guard access
            // to it.

            __CLOVER_446_0.S[9894]++;synchronized (format)
            {
                __CLOVER_446_0.S[9895]++;result = format.parse(value);
            }

            __CLOVER_446_0.S[9896]++;if ((((_calendar == null) && (++__CLOVER_446_0.CT[1694] != 0)) || (++__CLOVER_446_0.CF[1694] == 0))){
                __CLOVER_446_0.S[9897]++;_calendar = new GregorianCalendar();}

            __CLOVER_446_0.S[9898]++;_calendar.setTime(result);

            // SimpleDateFormat allows two-digit dates to be
            // entered, i.e., 12/24/66 is Dec 24 0066 ... that's
            // probably not what is really wanted, so treat
            // it as an invalid date.

            __CLOVER_446_0.S[9899]++;if ((((_calendar.get(Calendar.YEAR) < 1000) && (++__CLOVER_446_0.CT[1695] != 0)) || (++__CLOVER_446_0.CF[1695] == 0))){
                __CLOVER_446_0.S[9900]++;result = null;}

        }
        catch (ParseException ex)
        {
            // ParseException does not include a useful error message
            // about what's wrong.
            __CLOVER_446_0.S[9901]++;result = null;
        }

        __CLOVER_446_0.S[9902]++;if ((((result == null) && (++__CLOVER_446_0.CT[1696] != 0)) || (++__CLOVER_446_0.CF[1696] == 0))){
            __CLOVER_446_0.S[9903]++;throw new ValidatorException(
                buildInvalidDateFormatMessage(field),
                ValidationConstraint.DATE_FORMAT);}

        // OK, check that the date is in range.

        __CLOVER_446_0.S[9904]++;if ((((_minimum != null && _minimum.compareTo(result) > 0) && (++__CLOVER_446_0.CT[1697] != 0)) || (++__CLOVER_446_0.CF[1697] == 0))){
            __CLOVER_446_0.S[9905]++;throw new ValidatorException(
                buildDateTooEarlyMessage(field, format.format(_minimum)),
                ValidationConstraint.TOO_SMALL);}

        __CLOVER_446_0.S[9906]++;if ((((_maximum != null && _maximum.compareTo(result) < 0) && (++__CLOVER_446_0.CT[1698] != 0)) || (++__CLOVER_446_0.CF[1698] == 0))){
            __CLOVER_446_0.S[9907]++;throw new ValidatorException(
                buildDateTooLateMessage(field, format.format(_maximum)),
                ValidationConstraint.TOO_LARGE);}

        __CLOVER_446_0.S[9908]++;return result;

    } finally { }}

    public Date getMaximum()
    {try { __CLOVER_446_0.M[2413]++;
        __CLOVER_446_0.S[9909]++;return _maximum;
    } finally { }}

    public void setMaximum(Date maximum)
    {try { __CLOVER_446_0.M[2414]++;
        __CLOVER_446_0.S[9910]++;_maximum = maximum;
    } finally { }}

    public Date getMinimum()
    {try { __CLOVER_446_0.M[2415]++;
        __CLOVER_446_0.S[9911]++;return _minimum;
    } finally { }}

    public void setMinimum(Date minimum)
    {try { __CLOVER_446_0.M[2416]++;
        __CLOVER_446_0.S[9912]++;_minimum = minimum;
    } finally { }}

    /** 
     * 
     *  @since 2.2
     * 
     **/

    public void renderValidatorContribution(
        IFormComponent field,
        IMarkupWriter writer,
        IRequestCycle cycle)
    {try { __CLOVER_446_0.M[2417]++;
        __CLOVER_446_0.S[9913]++;if ((((!(isClientScriptingEnabled() && isRequired())) && (++__CLOVER_446_0.CT[1699] != 0)) || (++__CLOVER_446_0.CF[1699] == 0))){
            __CLOVER_446_0.S[9914]++;return;}

        __CLOVER_446_0.S[9915]++;Map symbols = new HashMap();

        __CLOVER_446_0.S[9916]++;symbols.put("requiredMessage", buildRequiredMessage(field));

        __CLOVER_446_0.S[9917]++;processValidatorScript(_scriptPath, cycle, field, symbols);
    } finally { }}

    /**
     *  @since 2.2
     * 
     **/

    public String getScriptPath()
    {try { __CLOVER_446_0.M[2418]++;
        __CLOVER_446_0.S[9918]++;return _scriptPath;
    } finally { }}

    /**
     *  Allows a developer to use the existing validation logic with a different client-side
     *  script.  This is often sufficient to allow application-specific error presentation
     *  (perhaps by using DHTML to update the content of a &lt;span&gt; tag, or to use
     *  a more sophisticated pop-up window than <code>window.alert()</code>).
     * 
     *  @since 2.2
     * 
     **/

    public void setScriptPath(String scriptPath)
    {try { __CLOVER_446_0.M[2419]++;
        __CLOVER_446_0.S[9919]++;_scriptPath = scriptPath;
    } finally { }}

    /** @since 3.0 */

    public String getDateTooEarlyMessage()
    {try { __CLOVER_446_0.M[2420]++;
        __CLOVER_446_0.S[9920]++;return _dateTooEarlyMessage;
    } finally { }}

    /** @since 3.0 */

    public String getDateTooLateMessage()
    {try { __CLOVER_446_0.M[2421]++;
        __CLOVER_446_0.S[9921]++;return _dateTooLateMessage;
    } finally { }}

    /** @since 3.0 */

    public String getInvalidDateFormatMessage()
    {try { __CLOVER_446_0.M[2422]++;
        __CLOVER_446_0.S[9922]++;return _invalidDateFormatMessage;
    } finally { }}

    /** @since 3.0 */

    protected String buildInvalidDateFormatMessage(IFormComponent field)
    {try { __CLOVER_446_0.M[2423]++;
        __CLOVER_446_0.S[9923]++;String pattern =
            getPattern(
                _invalidDateFormatMessage,
                "invalid-date-format",
                field.getPage().getLocale());

        __CLOVER_446_0.S[9924]++;return formatString(pattern, field.getDisplayName(), getEffectiveDisplayFormat());
    } finally { }}

    /** @since 3.0 **/

    protected String buildDateTooEarlyMessage(IFormComponent field, String earliestDate)
    {try { __CLOVER_446_0.M[2424]++;
        __CLOVER_446_0.S[9925]++;String pattern =
            getPattern(_dateTooEarlyMessage, "date-too-early", field.getPage().getLocale());

        __CLOVER_446_0.S[9926]++;return formatString(pattern, field.getDisplayName(), earliestDate);
    } finally { }}

    /** @since 3.0 */

    protected String buildDateTooLateMessage(IFormComponent field, String latestDate)
    {try { __CLOVER_446_0.M[2425]++;
        __CLOVER_446_0.S[9927]++;String pattern =
            getPattern(_dateTooLateMessage, "date-too-late", field.getPage().getLocale());

        __CLOVER_446_0.S[9928]++;return formatString(pattern, field.getDisplayName(), latestDate);
    } finally { }}

    /**
     *  Overrides the bundle key
     *  <code>date-too-early</code>.
     *  Parameter {0} is the display name of the field.
     *  Parameter {1} is the earliest allowed date.
     * 
     *  @since 3.0
     */

    public void setDateTooEarlyMessage(String string)
    {try { __CLOVER_446_0.M[2426]++;
        __CLOVER_446_0.S[9929]++;_dateTooEarlyMessage = string;
    } finally { }}

    /**
     *  Overrides the bundle key
     *  <code>date-too-late</code>.
     *  Parameter {0} is the display name of the field.
     *  Parameter {1} is the latest allowed date.
     * 
     *  @since 3.0
     */

    public void setDateTooLateMessage(String string)
    {try { __CLOVER_446_0.M[2427]++;
        __CLOVER_446_0.S[9930]++;_dateTooLateMessage = string;
    } finally { }}

    /**
     *  Overrides the bundle key
     *  <code>invalid-date-format</code>.
     *  Parameter {0} is the display name of the field.
     *  Parameter {1} is the allowed format.
     * 
     *  @since 3.0
     */

    public void setInvalidDateFormatMessage(String string)
    {try { __CLOVER_446_0.M[2428]++;
        __CLOVER_446_0.S[9931]++;_invalidDateFormatMessage = string;
    } finally { }}

}