/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.valid;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.tapestry.IMarkupWriter;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.Tapestry;
import org.apache.tapestry.form.IFormComponent;
import org.apache.tapestry.util.AdaptorRegistry;

/**
 *  Simple validation for standard number classes.  This is probably insufficient
 *  for anything tricky and application specific, such as parsing currency.  
 *
 *  @author Howard Lewis Ship
 *  @since 1.0.8
 *
 **/

public class NumberValidator extends BaseValidator
{public static com.cortexeb.tools.clover.d __CLOVER_453_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private static final Map TYPES = new HashMap();

    static {
        __CLOVER_453_0.S[10006]++;TYPES.put("boolean", boolean.class);
        __CLOVER_453_0.S[10007]++;TYPES.put("Boolean", Boolean.class);
        __CLOVER_453_0.S[10008]++;TYPES.put("java.lang.Boolean", Boolean.class);
        __CLOVER_453_0.S[10009]++;TYPES.put("char", char.class);
        __CLOVER_453_0.S[10010]++;TYPES.put("Character", Character.class);
        __CLOVER_453_0.S[10011]++;TYPES.put("java.lang.Character", Character.class);
        __CLOVER_453_0.S[10012]++;TYPES.put("short", short.class);
        __CLOVER_453_0.S[10013]++;TYPES.put("Short", Short.class);
        __CLOVER_453_0.S[10014]++;TYPES.put("java.lang.Short", Short.class);
        __CLOVER_453_0.S[10015]++;TYPES.put("int", int.class);
        __CLOVER_453_0.S[10016]++;TYPES.put("Integer", Integer.class);
        __CLOVER_453_0.S[10017]++;TYPES.put("java.lang.Integer", Integer.class);
        __CLOVER_453_0.S[10018]++;TYPES.put("long", long.class);
        __CLOVER_453_0.S[10019]++;TYPES.put("Long", Long.class);
        __CLOVER_453_0.S[10020]++;TYPES.put("java.lang.Long", Long.class);
        __CLOVER_453_0.S[10021]++;TYPES.put("float", float.class);
        __CLOVER_453_0.S[10022]++;TYPES.put("Float", Float.class);
        __CLOVER_453_0.S[10023]++;TYPES.put("java.lang.Float", Float.class);
        __CLOVER_453_0.S[10024]++;TYPES.put("byte", byte.class);
        __CLOVER_453_0.S[10025]++;TYPES.put("Byte", Byte.class);
        __CLOVER_453_0.S[10026]++;TYPES.put("java.lang.Byte", Byte.class);
        __CLOVER_453_0.S[10027]++;TYPES.put("double", double.class);
        __CLOVER_453_0.S[10028]++;TYPES.put("Double", Double.class);
        __CLOVER_453_0.S[10029]++;TYPES.put("java.lang.Double", Double.class);
        __CLOVER_453_0.S[10030]++;TYPES.put("java.math.BigInteger", BigInteger.class);
        __CLOVER_453_0.S[10031]++;TYPES.put("java.math.BigDecimal", BigDecimal.class);
    }

    private static final Set INT_TYPES = new HashSet();

    private Class _valueTypeClass = int.class;

    private boolean _zeroIsNull;
    private Number _minimum;
    private Number _maximum;

    private String _scriptPath = "/org/apache/tapestry/valid/NumberValidator.script";

    private String _invalidNumericFormatMessage;
    private String _invalidIntegerFormatMessage;
    private String _numberTooSmallMessage;
    private String _numberTooLargeMessage;
    private String _numberRangeMessage;

    private static AdaptorRegistry _numberAdaptors = new AdaptorRegistry();

    public final static int NUMBER_TYPE_INTEGER = 0;
	public final static int NUMBER_TYPE_REAL = 1;

	/**
	 * This class is not meant for use outside of NumberValidator; it
	 * is public only to fascilitate some unit testing.
	 * 
	 */
    public static abstract class NumberAdaptor
    {
        /**
         *  Parses a non-empty {@link String} into the correct subclass of
         *  {@link Number}.
         *
         *  @throws NumberFormatException if the String can not be parsed.
         **/

        abstract public Number parse(String value);

        /**
         *  Indicates the type of the number represented -- integer or real.
         *  The information is used to build the client-side validator.  
         *  This method could return a boolean, but returns an int to allow
         *  future extensions of the validator.
         *   
         *  @return one of the predefined number types
         **/
        abstract public int getNumberType();

        public int compare(Number left, Number right)
        {try { __CLOVER_453_0.M[2457]++;
            __CLOVER_453_0.S[10032]++;if ((((!left.getClass().equals(right.getClass())) && (++__CLOVER_453_0.CT[1715] != 0)) || (++__CLOVER_453_0.CF[1715] == 0))){
                __CLOVER_453_0.S[10033]++;right = coerce(right);}

            __CLOVER_453_0.S[10034]++;Comparable lc = (Comparable) left;

            __CLOVER_453_0.S[10035]++;return lc.compareTo(right);
        } finally { }}

        /**
         * Invoked when comparing two Numbers of different types.
         * The number is cooerced from its ordinary type to 
         * the correct type for comparison.
         * 
         * @since 3.0
         */
        protected abstract Number coerce(Number number);
    }

    private static abstract class IntegerNumberAdaptor extends NumberAdaptor
    {
        public int getNumberType()
        {try { __CLOVER_453_0.M[2458]++;
            __CLOVER_453_0.S[10036]++;return NUMBER_TYPE_INTEGER;
        } finally { }}
    }

    private static abstract class RealNumberAdaptor extends NumberAdaptor
    {
        public int getNumberType()
        {try { __CLOVER_453_0.M[2459]++;
            __CLOVER_453_0.S[10037]++;return NUMBER_TYPE_REAL;
        } finally { }}
    }

    private static class ByteAdaptor extends IntegerNumberAdaptor
    {
        public Number parse(String value)
        {try { __CLOVER_453_0.M[2460]++;
            __CLOVER_453_0.S[10038]++;return new Byte(value);
        } finally { }}

        protected Number coerce(Number number)
        {try { __CLOVER_453_0.M[2461]++;
            __CLOVER_453_0.S[10039]++;return new Byte(number.byteValue());
        } finally { }}
    }

    private static class ShortAdaptor extends IntegerNumberAdaptor
    {
        public Number parse(String value)
        {try { __CLOVER_453_0.M[2462]++;
            __CLOVER_453_0.S[10040]++;return new Short(value);
        } finally { }}

        protected Number coerce(Number number)
        {try { __CLOVER_453_0.M[2463]++;
            __CLOVER_453_0.S[10041]++;return new Short(number.shortValue());
        } finally { }}
    }

    private static class IntAdaptor extends IntegerNumberAdaptor
    {
        public Number parse(String value)
        {try { __CLOVER_453_0.M[2464]++;
            __CLOVER_453_0.S[10042]++;return new Integer(value);
        } finally { }}

        protected Number coerce(Number number)
        {try { __CLOVER_453_0.M[2465]++;
            __CLOVER_453_0.S[10043]++;return new Integer(number.intValue());
        } finally { }}
    }

    private static class LongAdaptor extends IntegerNumberAdaptor
    {
        public Number parse(String value)
        {try { __CLOVER_453_0.M[2466]++;
            __CLOVER_453_0.S[10044]++;return new Long(value);
        } finally { }}

        protected Number coerce(Number number)
        {try { __CLOVER_453_0.M[2467]++;
            __CLOVER_453_0.S[10045]++;return new Long(number.longValue());
        } finally { }}
    }

    private static class FloatAdaptor extends RealNumberAdaptor
    {
        public Number parse(String value)
        {try { __CLOVER_453_0.M[2468]++;
            __CLOVER_453_0.S[10046]++;return new Float(value);
        } finally { }}

        protected Number coerce(Number number)
        {try { __CLOVER_453_0.M[2469]++;
            __CLOVER_453_0.S[10047]++;return new Float(number.floatValue());
        } finally { }}
    }

    private static class DoubleAdaptor extends RealNumberAdaptor
    {
        public Number parse(String value)
        {try { __CLOVER_453_0.M[2470]++;
            __CLOVER_453_0.S[10048]++;return new Double(value);
        } finally { }}

        protected Number coerce(Number number)
        {try { __CLOVER_453_0.M[2471]++;
            __CLOVER_453_0.S[10049]++;return new Double(number.doubleValue());
        } finally { }}
    }

    private static class BigDecimalAdaptor extends RealNumberAdaptor
    {
        public Number parse(String value)
        {try { __CLOVER_453_0.M[2472]++;
            __CLOVER_453_0.S[10050]++;return new BigDecimal(value);
        } finally { }}

        protected Number coerce(Number number)
        {try { __CLOVER_453_0.M[2473]++;
            __CLOVER_453_0.S[10051]++;return new BigDecimal(number.doubleValue());
        } finally { }}
    }

    private static class BigIntegerAdaptor extends IntegerNumberAdaptor
    {
        public Number parse(String value)
        {try { __CLOVER_453_0.M[2474]++;
            __CLOVER_453_0.S[10052]++;return new BigInteger(value);
        } finally { }}

        protected Number coerce(Number number)
        {try { __CLOVER_453_0.M[2475]++;
            __CLOVER_453_0.S[10053]++;return new BigInteger(number.toString());
        } finally { }}
    }

    static {
        __CLOVER_453_0.S[10054]++;NumberAdaptor byteAdaptor = new ByteAdaptor();
        __CLOVER_453_0.S[10055]++;NumberAdaptor shortAdaptor = new ShortAdaptor();
        __CLOVER_453_0.S[10056]++;NumberAdaptor intAdaptor = new IntAdaptor();
        __CLOVER_453_0.S[10057]++;NumberAdaptor longAdaptor = new LongAdaptor();
        __CLOVER_453_0.S[10058]++;NumberAdaptor floatAdaptor = new FloatAdaptor();
        __CLOVER_453_0.S[10059]++;NumberAdaptor doubleAdaptor = new DoubleAdaptor();

        __CLOVER_453_0.S[10060]++;_numberAdaptors.register(Byte.class, byteAdaptor);
        __CLOVER_453_0.S[10061]++;_numberAdaptors.register(byte.class, byteAdaptor);
        __CLOVER_453_0.S[10062]++;_numberAdaptors.register(Short.class, shortAdaptor);
        __CLOVER_453_0.S[10063]++;_numberAdaptors.register(short.class, shortAdaptor);
        __CLOVER_453_0.S[10064]++;_numberAdaptors.register(Integer.class, intAdaptor);
        __CLOVER_453_0.S[10065]++;_numberAdaptors.register(int.class, intAdaptor);
        __CLOVER_453_0.S[10066]++;_numberAdaptors.register(Long.class, longAdaptor);
        __CLOVER_453_0.S[10067]++;_numberAdaptors.register(long.class, longAdaptor);
        __CLOVER_453_0.S[10068]++;_numberAdaptors.register(Float.class, floatAdaptor);
        __CLOVER_453_0.S[10069]++;_numberAdaptors.register(float.class, floatAdaptor);
        __CLOVER_453_0.S[10070]++;_numberAdaptors.register(Double.class, doubleAdaptor);
        __CLOVER_453_0.S[10071]++;_numberAdaptors.register(double.class, doubleAdaptor);

        __CLOVER_453_0.S[10072]++;_numberAdaptors.register(BigDecimal.class, new BigDecimalAdaptor());
        __CLOVER_453_0.S[10073]++;_numberAdaptors.register(BigInteger.class, new BigIntegerAdaptor());
    }

    public String toString(IFormComponent field, Object value)
    {try { __CLOVER_453_0.M[2476]++;
        __CLOVER_453_0.S[10074]++;if ((((value == null) && (++__CLOVER_453_0.CT[1716] != 0)) || (++__CLOVER_453_0.CF[1716] == 0))){
            __CLOVER_453_0.S[10075]++;return null;}

        __CLOVER_453_0.S[10076]++;if ((((_zeroIsNull) && (++__CLOVER_453_0.CT[1717] != 0)) || (++__CLOVER_453_0.CF[1717] == 0))){
        {
            __CLOVER_453_0.S[10077]++;Number number = (Number) value;

            __CLOVER_453_0.S[10078]++;if ((((number.doubleValue() == 0.0) && (++__CLOVER_453_0.CT[1718] != 0)) || (++__CLOVER_453_0.CF[1718] == 0))){
                __CLOVER_453_0.S[10079]++;return null;}
        }}

        __CLOVER_453_0.S[10080]++;return value.toString();
    } finally { }}

    private NumberAdaptor getAdaptor(IFormComponent field)
    {try { __CLOVER_453_0.M[2477]++;
        __CLOVER_453_0.S[10081]++;NumberAdaptor result = getAdaptor(_valueTypeClass);

        __CLOVER_453_0.S[10082]++;if ((((result == null) && (++__CLOVER_453_0.CT[1719] != 0)) || (++__CLOVER_453_0.CF[1719] == 0))){
            __CLOVER_453_0.S[10083]++;throw new ApplicationRuntimeException(
                Tapestry.format(
                    "NumberValidator.no-adaptor-for-field",
                    field,
                    _valueTypeClass.getName()));}

        __CLOVER_453_0.S[10084]++;return result;
    } finally { }}

	/**
	 * Returns an adaptor for the given type.
	 * 
	 * <p>
	 * Note: this method exists only for testing purposes. It is not meant to
	 * be invoked by user code and is subject to change at any time.
	 * 
	 * @param type the type (a Number subclass) for which to return an adaptor
	 * @return the adaptor, or null if no such adaptor may be found
	 * @since 3.0
	 */
    public static NumberAdaptor getAdaptor(Class type)
    {try { __CLOVER_453_0.M[2478]++;
        __CLOVER_453_0.S[10085]++;return (NumberAdaptor) _numberAdaptors.getAdaptor(type);
    } finally { }}

    public Object toObject(IFormComponent field, String value) throws ValidatorException
    {try { __CLOVER_453_0.M[2479]++;
        __CLOVER_453_0.S[10086]++;if ((((checkRequired(field, value)) && (++__CLOVER_453_0.CT[1720] != 0)) || (++__CLOVER_453_0.CF[1720] == 0))){
            __CLOVER_453_0.S[10087]++;return null;}

        __CLOVER_453_0.S[10088]++;NumberAdaptor adaptor = getAdaptor(field);
        __CLOVER_453_0.S[10089]++;Number result = null;

        __CLOVER_453_0.S[10090]++;try
        {
            __CLOVER_453_0.S[10091]++;result = adaptor.parse(value);
        }
        catch (NumberFormatException ex)
        {
            __CLOVER_453_0.S[10092]++;throw new ValidatorException(
                buildInvalidNumericFormatMessage(field),
                ValidationConstraint.NUMBER_FORMAT);
        }

        __CLOVER_453_0.S[10093]++;if ((((_minimum != null && adaptor.compare(result, _minimum) < 0) && (++__CLOVER_453_0.CT[1721] != 0)) || (++__CLOVER_453_0.CF[1721] == 0))){
            __CLOVER_453_0.S[10094]++;throw new ValidatorException(
                buildNumberTooSmallMessage(field),
                ValidationConstraint.TOO_SMALL);}

        __CLOVER_453_0.S[10095]++;if ((((_maximum != null && adaptor.compare(result, _maximum) > 0) && (++__CLOVER_453_0.CT[1722] != 0)) || (++__CLOVER_453_0.CF[1722] == 0))){
            __CLOVER_453_0.S[10096]++;throw new ValidatorException(
                buildNumberTooLargeMessage(field),
                ValidationConstraint.TOO_LARGE);}

        __CLOVER_453_0.S[10097]++;return result;
    } finally { }}

    public Number getMaximum()
    {try { __CLOVER_453_0.M[2480]++;
        __CLOVER_453_0.S[10098]++;return _maximum;
    } finally { }}

    public boolean getHasMaximum()
    {try { __CLOVER_453_0.M[2481]++;
        __CLOVER_453_0.S[10099]++;return _maximum != null;
    } finally { }}

    public void setMaximum(Number maximum)
    {try { __CLOVER_453_0.M[2482]++;
        __CLOVER_453_0.S[10100]++;_maximum = maximum;
    } finally { }}

    public Number getMinimum()
    {try { __CLOVER_453_0.M[2483]++;
        __CLOVER_453_0.S[10101]++;return _minimum;
    } finally { }}

    public boolean getHasMinimum()
    {try { __CLOVER_453_0.M[2484]++;
        __CLOVER_453_0.S[10102]++;return _minimum != null;
    } finally { }}

    public void setMinimum(Number minimum)
    {try { __CLOVER_453_0.M[2485]++;
        __CLOVER_453_0.S[10103]++;_minimum = minimum;
    } finally { }}

    /**
     *  If true, then when rendering, a zero is treated as a non-value, and null is returned.
     *  If false, the default, then zero is rendered as zero.
     * 
     **/

    public boolean getZeroIsNull()
    {try { __CLOVER_453_0.M[2486]++;
        __CLOVER_453_0.S[10104]++;return _zeroIsNull;
    } finally { }}

    public void setZeroIsNull(boolean zeroIsNull)
    {try { __CLOVER_453_0.M[2487]++;
        __CLOVER_453_0.S[10105]++;_zeroIsNull = zeroIsNull;
    } finally { }}

    /** 
     * 
     *  @since 2.2
     * 
     **/

    public void renderValidatorContribution(
        IFormComponent field,
        IMarkupWriter writer,
        IRequestCycle cycle)
    {try { __CLOVER_453_0.M[2488]++;
        __CLOVER_453_0.S[10106]++;if ((((!isClientScriptingEnabled()) && (++__CLOVER_453_0.CT[1723] != 0)) || (++__CLOVER_453_0.CF[1723] == 0))){
            __CLOVER_453_0.S[10107]++;return;}

        __CLOVER_453_0.S[10108]++;if ((((!(isRequired() || _minimum != null || _maximum != null)) && (++__CLOVER_453_0.CT[1724] != 0)) || (++__CLOVER_453_0.CF[1724] == 0))){
            __CLOVER_453_0.S[10109]++;return;}

        __CLOVER_453_0.S[10110]++;Map symbols = new HashMap();

        __CLOVER_453_0.S[10111]++;if ((((isRequired()) && (++__CLOVER_453_0.CT[1725] != 0)) || (++__CLOVER_453_0.CF[1725] == 0))){
            __CLOVER_453_0.S[10112]++;symbols.put("requiredMessage", buildRequiredMessage(field));}

        __CLOVER_453_0.S[10113]++;if ((((isIntegerNumber()) && (++__CLOVER_453_0.CT[1726] != 0)) || (++__CLOVER_453_0.CF[1726] == 0))){
            __CLOVER_453_0.S[10114]++;symbols.put("formatMessage", buildInvalidIntegerFormatMessage(field));}
        else{
            __CLOVER_453_0.S[10115]++;symbols.put("formatMessage", buildInvalidNumericFormatMessage(field));}

        __CLOVER_453_0.S[10116]++;if ((((_minimum != null || _maximum != null) && (++__CLOVER_453_0.CT[1727] != 0)) || (++__CLOVER_453_0.CF[1727] == 0))){
            __CLOVER_453_0.S[10117]++;symbols.put("rangeMessage", buildRangeMessage(field));}

        __CLOVER_453_0.S[10118]++;processValidatorScript(_scriptPath, cycle, field, symbols);
    } finally { }}

    private String buildRangeMessage(IFormComponent field)
    {try { __CLOVER_453_0.M[2489]++;
        __CLOVER_453_0.S[10119]++;if ((((_minimum != null && _maximum != null) && (++__CLOVER_453_0.CT[1728] != 0)) || (++__CLOVER_453_0.CF[1728] == 0))){
            __CLOVER_453_0.S[10120]++;return buildNumberRangeMessage(field);}

        __CLOVER_453_0.S[10121]++;if ((((_maximum != null) && (++__CLOVER_453_0.CT[1729] != 0)) || (++__CLOVER_453_0.CF[1729] == 0))){
            __CLOVER_453_0.S[10122]++;return buildNumberTooLargeMessage(field);}

        __CLOVER_453_0.S[10123]++;return buildNumberTooSmallMessage(field);
    } finally { }}

    /**
     *  @since 2.2
     * 
     **/

    public String getScriptPath()
    {try { __CLOVER_453_0.M[2490]++;
        __CLOVER_453_0.S[10124]++;return _scriptPath;
    } finally { }}

    /**
     *  Allows a developer to use the existing validation logic with a different client-side
     *  script.  This is often sufficient to allow application-specific error presentation
     *  (perhaps by using DHTML to update the content of a &lt;span&gt; tag, or to use
     *  a more sophisticated pop-up window than <code>window.alert()</code>).
     * 
     *  @since 2.2
     * 
     **/

    public void setScriptPath(String scriptPath)
    {try { __CLOVER_453_0.M[2491]++;
        __CLOVER_453_0.S[10125]++;_scriptPath = scriptPath;
    } finally { }}

    /** Sets the value type from a string type name.  The name may be
     *  a scalar numeric type, a fully qualified class name, or the name
     *  of a numeric wrapper type from java.lang (with the package name omitted).
     * 
     * @since 3.0 
     * 
     **/

    public void setValueType(String typeName)
    {try { __CLOVER_453_0.M[2492]++;
        __CLOVER_453_0.S[10126]++;Class typeClass = (Class) TYPES.get(typeName);

        __CLOVER_453_0.S[10127]++;if ((((typeClass == null) && (++__CLOVER_453_0.CT[1730] != 0)) || (++__CLOVER_453_0.CF[1730] == 0))){
            __CLOVER_453_0.S[10128]++;throw new ApplicationRuntimeException(
                Tapestry.format("NumberValidator.unknown-type", typeName));}

        __CLOVER_453_0.S[10129]++;_valueTypeClass = typeClass;
    } finally { }}

    /** @since 3.0 **/

    public void setValueTypeClass(Class valueTypeClass)
    {try { __CLOVER_453_0.M[2493]++;
        __CLOVER_453_0.S[10130]++;_valueTypeClass = valueTypeClass;
    } finally { }}

    /** 
     *  
     *  Returns the value type to convert strings back into.  The default is int.
     * 
     *  @since 3.0 
     * 
     **/

    public Class getValueTypeClass()
    {try { __CLOVER_453_0.M[2494]++;
        __CLOVER_453_0.S[10131]++;return _valueTypeClass;
    } finally { }}

    /** @since 3.0 */

    public String getInvalidNumericFormatMessage()
    {try { __CLOVER_453_0.M[2495]++;
        __CLOVER_453_0.S[10132]++;return _invalidNumericFormatMessage;
    } finally { }}

    /** @since 3.0 */

    public String getInvalidIntegerFormatMessage()
    {try { __CLOVER_453_0.M[2496]++;
        __CLOVER_453_0.S[10133]++;return _invalidIntegerFormatMessage;
    } finally { }}

    /** @since 3.0 */

    public String getNumberRangeMessage()
    {try { __CLOVER_453_0.M[2497]++;
        __CLOVER_453_0.S[10134]++;return _numberRangeMessage;
    } finally { }}

    /** @since 3.0 */

    public String getNumberTooLargeMessage()
    {try { __CLOVER_453_0.M[2498]++;
        __CLOVER_453_0.S[10135]++;return _numberTooLargeMessage;
    } finally { }}

    /** @since 3.0 */

    public String getNumberTooSmallMessage()
    {try { __CLOVER_453_0.M[2499]++;
        __CLOVER_453_0.S[10136]++;return _numberTooSmallMessage;
    } finally { }}

    /** 
     * Overrides the <code>invalid-numeric-format</code> bundle key.
     * Parameter {0} is the display name of the field.
     * 
     * @since 3.0
     */

    public void setInvalidNumericFormatMessage(String string)
    {try { __CLOVER_453_0.M[2500]++;
        __CLOVER_453_0.S[10137]++;_invalidNumericFormatMessage = string;
    } finally { }}

    /** 
     * Overrides the <code>invalid-int-format</code> bundle key.
     * Parameter {0} is the display name of the field.
     * 
     * @since 3.0
     */

    public void setInvalidIntegerFormatMessage(String string)
    {try { __CLOVER_453_0.M[2501]++;
        __CLOVER_453_0.S[10138]++;_invalidIntegerFormatMessage = string;
    } finally { }}

    /** @since 3.0 */

    protected String buildInvalidNumericFormatMessage(IFormComponent field)
    {try { __CLOVER_453_0.M[2502]++;
        __CLOVER_453_0.S[10139]++;String pattern =
            getPattern(
                getInvalidNumericFormatMessage(),
                "invalid-numeric-format",
                field.getPage().getLocale());

        __CLOVER_453_0.S[10140]++;return formatString(pattern, field.getDisplayName());
    } finally { }}

    /** @since 3.0 */

    protected String buildInvalidIntegerFormatMessage(IFormComponent field)
    {try { __CLOVER_453_0.M[2503]++;
        __CLOVER_453_0.S[10141]++;String pattern =
            getPattern(
                getInvalidIntegerFormatMessage(),
                "invalid-int-format",
                field.getPage().getLocale());

        __CLOVER_453_0.S[10142]++;return formatString(pattern, field.getDisplayName());
    } finally { }}

    /** 
     * Overrides the <code>number-range</code> bundle key.
     * Parameter [0} is the display name of the field.
     * Parameter {1} is the minimum value.
     * Parameter {2} is the maximum value.
     * 
     * @since 3.0
     */

    public void setNumberRangeMessage(String string)
    {try { __CLOVER_453_0.M[2504]++;
        __CLOVER_453_0.S[10143]++;_numberRangeMessage = string;
    } finally { }}

    protected String buildNumberRangeMessage(IFormComponent field)
    {try { __CLOVER_453_0.M[2505]++;
        __CLOVER_453_0.S[10144]++;String pattern =
            getPattern(_numberRangeMessage, "number-range", field.getPage().getLocale());

        __CLOVER_453_0.S[10145]++;return formatString(pattern, new Object[] { field.getDisplayName(), _minimum, _maximum });
    } finally { }}

    /**
     *  Overrides the <code>number-too-large</code> bundle key.
     *  Parameter {0} is the display name of the field.
     *  Parameter {1} is the maximum allowed value.
     *  
     *  @since 3.0
     */

    public void setNumberTooLargeMessage(String string)
    {try { __CLOVER_453_0.M[2506]++;
        __CLOVER_453_0.S[10146]++;_numberTooLargeMessage = string;
    } finally { }}

    /** @since 3.0 */

    protected String buildNumberTooLargeMessage(IFormComponent field)
    {try { __CLOVER_453_0.M[2507]++;
        __CLOVER_453_0.S[10147]++;String pattern =
            getPattern(_numberTooLargeMessage, "number-too-large", field.getPage().getLocale());

        __CLOVER_453_0.S[10148]++;return formatString(pattern, field.getDisplayName(), _maximum);
    } finally { }}

    /**
     *  Overrides the <code>number-too-small</code> bundle key.
     *  Parameter {0} is the display name of the field.
     *  Parameter {1} is the minimum allowed value.
     * 
     *  @since 3.0
     * 
     */

    public void setNumberTooSmallMessage(String string)
    {try { __CLOVER_453_0.M[2508]++;
        __CLOVER_453_0.S[10149]++;_numberTooSmallMessage = string;
    } finally { }}

    /** @since 3.0 */

    protected String buildNumberTooSmallMessage(IFormComponent field)
    {try { __CLOVER_453_0.M[2509]++;
        __CLOVER_453_0.S[10150]++;String pattern =
            getPattern(_numberTooSmallMessage, "number-too-small", field.getPage().getLocale());

        __CLOVER_453_0.S[10151]++;return formatString(pattern, field.getDisplayName(), _minimum);
    } finally { }}

    /** @since 3.0 */

    public boolean isIntegerNumber()
    {try { __CLOVER_453_0.M[2510]++;
        __CLOVER_453_0.S[10152]++;NumberAdaptor result = (NumberAdaptor) _numberAdaptors.getAdaptor(_valueTypeClass);
        __CLOVER_453_0.S[10153]++;if ((((result == null) && (++__CLOVER_453_0.CT[1731] != 0)) || (++__CLOVER_453_0.CF[1731] == 0))){
            __CLOVER_453_0.S[10154]++;return false;}

        __CLOVER_453_0.S[10155]++;return result.getNumberType() == NUMBER_TYPE_INTEGER;
    } finally { }}
}