/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.valid;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.tapestry.IForm;
import org.apache.tapestry.IMarkupWriter;
import org.apache.tapestry.IRender;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.Tapestry;
import org.apache.tapestry.form.IFormComponent;

/**
 *  A base implementation of {@link IValidationDelegate} that can be used
 *  as a helper bean.  This class is often subclassed, typically to override presentation
 *  details.
 *
 *  @author Howard Lewis Ship
 *  @since 1.0.5
 * 
 **/

public class ValidationDelegate implements IValidationDelegate
{public static com.cortexeb.tools.clover.d __CLOVER_461_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private IFormComponent _currentComponent;
    private List _trackings;

    /**
     *  A Map of Maps, keyed on the name of the Form.  Each inner map contains
     *  the trackings for one form, keyed on component name.  Care must
     *  be taken, because the inner Map is not always present.
     * 
     **/

    private Map _trackingMap;

    public void clear()
    {try { __CLOVER_461_0.M[2574]++;
        __CLOVER_461_0.S[10379]++;_currentComponent = null;
        __CLOVER_461_0.S[10380]++;_trackings = null;
        __CLOVER_461_0.S[10381]++;_trackingMap = null;
    } finally { }}

    /**
     *  If the form component is in error, places a &lt;font color="red"&lt; around it.
     *  Note: this will only work on the render phase after a rewind, and will be
     *  confused if components are inside any kind of loop.
     **/

    public void writeLabelPrefix(
        IFormComponent component,
        IMarkupWriter writer,
        IRequestCycle cycle)
    {try { __CLOVER_461_0.M[2575]++;
        __CLOVER_461_0.S[10382]++;if ((((isInError(component)) && (++__CLOVER_461_0.CT[1777] != 0)) || (++__CLOVER_461_0.CF[1777] == 0))){
        {
            __CLOVER_461_0.S[10383]++;writer.begin("font");
            __CLOVER_461_0.S[10384]++;writer.attribute("color", "red");
        }}
    } finally { }}

    /**
     *  Closes the &lt;font&gt; element,started by
     *  {@link #writeLabelPrefix(IFormComponent,IMarkupWriter,IRequestCycle)},
     *  if the form component is in error.
     *
     **/

    public void writeLabelSuffix(
        IFormComponent component,
        IMarkupWriter writer,
        IRequestCycle cycle)
    {try { __CLOVER_461_0.M[2576]++;
        __CLOVER_461_0.S[10385]++;if ((((isInError(component)) && (++__CLOVER_461_0.CT[1778] != 0)) || (++__CLOVER_461_0.CF[1778] == 0))){
        {
            __CLOVER_461_0.S[10386]++;writer.end();
        }}
    } finally { }}

    /**
     *  Returns the {@link IFieldTracking} for the current component, if any.
     *  The {@link IFieldTracking} is usually created in 
     *  {@link #record(String, ValidationConstraint)} or
     *  in {@link #record(IRender, ValidationConstraint)}.
     * 
     *  <p>Components may be rendered multiple times, with multiple names (provided
     *  by the {@link org.apache.tapestry.form.Form}, care must be taken that this method is invoked
     *  <em>after</em> the Form has provided a unique 
     *  {@link IFormComponent#getName()} for the component.
     * 
     *  @see #setFormComponent(IFormComponent)
     * 
     *  @return the {@link FieldTracking}, or null if the field has no tracking.
     * 
     **/

    protected FieldTracking getComponentTracking()
    {try { __CLOVER_461_0.M[2577]++;
        __CLOVER_461_0.S[10387]++;if ((((_trackingMap == null) && (++__CLOVER_461_0.CT[1779] != 0)) || (++__CLOVER_461_0.CF[1779] == 0))){
            __CLOVER_461_0.S[10388]++;return null;}

        __CLOVER_461_0.S[10389]++;String formName = _currentComponent.getForm().getName();

        __CLOVER_461_0.S[10390]++;Map formMap = (Map) _trackingMap.get(formName);

        __CLOVER_461_0.S[10391]++;if ((((formMap == null) && (++__CLOVER_461_0.CT[1780] != 0)) || (++__CLOVER_461_0.CF[1780] == 0))){
            __CLOVER_461_0.S[10392]++;return null;}

        __CLOVER_461_0.S[10393]++;return (FieldTracking) formMap.get(_currentComponent.getName());
    } finally { }}

    public void setFormComponent(IFormComponent component)
    {try { __CLOVER_461_0.M[2578]++;
        __CLOVER_461_0.S[10394]++;_currentComponent = component;
    } finally { }}

    public boolean isInError()
    {try { __CLOVER_461_0.M[2579]++;
        __CLOVER_461_0.S[10395]++;IFieldTracking tracking = getComponentTracking();

        __CLOVER_461_0.S[10396]++;return tracking != null && tracking.isInError();
    } finally { }}

    public String getFieldInputValue()
    {try { __CLOVER_461_0.M[2580]++;
        __CLOVER_461_0.S[10397]++;IFieldTracking tracking = getComponentTracking();

        __CLOVER_461_0.S[10398]++;return (((tracking == null ) && (++__CLOVER_461_0.CT[1781] != 0)) || (++__CLOVER_461_0.CF[1781] == 0))? null : tracking.getInput();
    } finally { }}

    /**
     *  Returns all the field trackings as an unmodifiable List.
     * 
     **/

    public List getFieldTracking()
    {try { __CLOVER_461_0.M[2581]++;
        __CLOVER_461_0.S[10399]++;if ((((Tapestry.size(_trackings) == 0) && (++__CLOVER_461_0.CT[1782] != 0)) || (++__CLOVER_461_0.CF[1782] == 0))){
            __CLOVER_461_0.S[10400]++;return null;}

        __CLOVER_461_0.S[10401]++;return Collections.unmodifiableList(_trackings);
    } finally { }}

    public void reset()
    {try { __CLOVER_461_0.M[2582]++;
        __CLOVER_461_0.S[10402]++;IFieldTracking tracking = getComponentTracking();

        __CLOVER_461_0.S[10403]++;if ((((tracking != null) && (++__CLOVER_461_0.CT[1783] != 0)) || (++__CLOVER_461_0.CF[1783] == 0))){
        {
            __CLOVER_461_0.S[10404]++;_trackings.remove(tracking);

            __CLOVER_461_0.S[10405]++;String formName = tracking.getComponent().getForm().getName();

            __CLOVER_461_0.S[10406]++;Map formMap = (Map) _trackingMap.get(formName);

            __CLOVER_461_0.S[10407]++;if ((((formMap != null) && (++__CLOVER_461_0.CT[1784] != 0)) || (++__CLOVER_461_0.CF[1784] == 0))){
                __CLOVER_461_0.S[10408]++;formMap.remove(tracking.getFieldName());}
        }}
    } finally { }}

    /**
     *  Invokes {@link #record(String, ValidationConstraint)}, or
     *  {@link #record(IRender, ValidationConstraint)} if the 
     *  {@link ValidatorException#getErrorRenderer() error renderer property}
     *  is not null.
     * 
     **/

    public void record(ValidatorException ex)
    {try { __CLOVER_461_0.M[2583]++;
        __CLOVER_461_0.S[10409]++;IRender errorRenderer = ex.getErrorRenderer();

        __CLOVER_461_0.S[10410]++;if ((((errorRenderer == null) && (++__CLOVER_461_0.CT[1785] != 0)) || (++__CLOVER_461_0.CF[1785] == 0))){
            __CLOVER_461_0.S[10411]++;record(ex.getMessage(), ex.getConstraint());}
        else{
            __CLOVER_461_0.S[10412]++;record(errorRenderer, ex.getConstraint());}
    } finally { }}

    /**
     *  Invokes {@link #record(IRender, ValidationConstraint)}, after
     *  wrapping the message parameter in a
     *  {@link RenderString}.
     * 
     **/

    public void record(String message, ValidationConstraint constraint)
    {try { __CLOVER_461_0.M[2584]++;
        __CLOVER_461_0.S[10413]++;record(new RenderString(message), constraint);
    } finally { }}

    /**
     *  Records error information about the currently selected component,
     *  or records unassociated (with any field) errors.
     * 
     *  <p>
     *  Currently, you may have at most one error per <em>field</em>
     *  (note the difference between field and component), but any number of
     *  unassociated errors.
     * 
     *  <p>
     *  Subclasses may override the default error message (based on other
     *  factors, such as the field and constraint) before invoking this
     *  implementation.
     * 
     *  @since 1.0.9
     **/

    public void record(IRender errorRenderer, ValidationConstraint constraint)
    {try { __CLOVER_461_0.M[2585]++;
        __CLOVER_461_0.S[10414]++;FieldTracking tracking = findCurrentTracking();

        // Note that recording two errors for the same field is not advised; the
        // second will override the first.

        __CLOVER_461_0.S[10415]++;tracking.setErrorRenderer(errorRenderer);
        __CLOVER_461_0.S[10416]++;tracking.setConstraint(constraint);
    } finally { }}

    public void recordFieldInputValue(String input)
    {try { __CLOVER_461_0.M[2586]++;
        __CLOVER_461_0.S[10417]++;FieldTracking tracking = findCurrentTracking();

        __CLOVER_461_0.S[10418]++;tracking.setInput(input);
    } finally { }}

    /**
     *  Finds or creates the field tracking for the
     *  {@link #setFormComponent(IFormComponent)} current component.
     *  If no current component, an unassociated error is created
     *  and returned.
     * 
     *  @since 3.0
     * 
     **/

    protected FieldTracking findCurrentTracking()
    {try { __CLOVER_461_0.M[2587]++;
        __CLOVER_461_0.S[10419]++;FieldTracking result = null;

        __CLOVER_461_0.S[10420]++;if ((((_trackings == null) && (++__CLOVER_461_0.CT[1786] != 0)) || (++__CLOVER_461_0.CF[1786] == 0))){
            __CLOVER_461_0.S[10421]++;_trackings = new ArrayList();}

        __CLOVER_461_0.S[10422]++;if ((((_trackingMap == null) && (++__CLOVER_461_0.CT[1787] != 0)) || (++__CLOVER_461_0.CF[1787] == 0))){
            __CLOVER_461_0.S[10423]++;_trackingMap = new HashMap();}

        __CLOVER_461_0.S[10424]++;if ((((_currentComponent == null) && (++__CLOVER_461_0.CT[1788] != 0)) || (++__CLOVER_461_0.CF[1788] == 0))){
        {
            __CLOVER_461_0.S[10425]++;result = new FieldTracking();

            // Add it to the field trackings, but not to the
            // map.

            __CLOVER_461_0.S[10426]++;_trackings.add(result);
        }}
        else{
        {
            __CLOVER_461_0.S[10427]++;result = getComponentTracking();

            __CLOVER_461_0.S[10428]++;if ((((result == null) && (++__CLOVER_461_0.CT[1789] != 0)) || (++__CLOVER_461_0.CF[1789] == 0))){
            {
                __CLOVER_461_0.S[10429]++;String formName = _currentComponent.getForm().getName();

                __CLOVER_461_0.S[10430]++;Map formMap = (Map) _trackingMap.get(formName);

                __CLOVER_461_0.S[10431]++;if ((((formMap == null) && (++__CLOVER_461_0.CT[1790] != 0)) || (++__CLOVER_461_0.CF[1790] == 0))){
                {
                    __CLOVER_461_0.S[10432]++;formMap = new HashMap();
                    __CLOVER_461_0.S[10433]++;_trackingMap.put(formName, formMap);
                }}

                __CLOVER_461_0.S[10434]++;String fieldName = _currentComponent.getName();

                __CLOVER_461_0.S[10435]++;result = new FieldTracking(fieldName, _currentComponent);

                __CLOVER_461_0.S[10436]++;_trackings.add(result);
                __CLOVER_461_0.S[10437]++;formMap.put(fieldName, result);
            }}
        }}

        __CLOVER_461_0.S[10438]++;return result;
    } finally { }}

    /**
     *  Does nothing.  Override in a subclass to decoreate
     *  fields.
     * 
     **/

    public void writePrefix(
        IMarkupWriter writer,
        IRequestCycle cycle,
        IFormComponent component,
        IValidator validator)
    {try { __CLOVER_461_0.M[2588]++;
    } finally { }}

    /**
     *  Does nothing.  Override in a subclass to decorate fields.
     * 
     **/

    public void writeAttributes(
        IMarkupWriter writer,
        IRequestCycle cycle,
        IFormComponent component,
        IValidator validator)
    {try { __CLOVER_461_0.M[2589]++;
    } finally { }}

    /**
     *  Default implementation; if the current field is in error,
     *  then a suffix is written.  The suffix is:
     *  <code>&amp;nbsp;&lt;font color="red"&gt;**&lt;/font&gt;</code>.
     * 
     **/

    public void writeSuffix(
        IMarkupWriter writer,
        IRequestCycle cycle,
        IFormComponent component,
        IValidator validator)
    {try { __CLOVER_461_0.M[2590]++;
        __CLOVER_461_0.S[10439]++;if ((((isInError()) && (++__CLOVER_461_0.CT[1791] != 0)) || (++__CLOVER_461_0.CF[1791] == 0))){
        {
            __CLOVER_461_0.S[10440]++;writer.printRaw("&nbsp;");
            __CLOVER_461_0.S[10441]++;writer.begin("font");
            __CLOVER_461_0.S[10442]++;writer.attribute("color", "red");
            __CLOVER_461_0.S[10443]++;writer.print("**");
            __CLOVER_461_0.S[10444]++;writer.end();
        }}
    } finally { }}

    public boolean getHasErrors()
    {try { __CLOVER_461_0.M[2591]++;
        __CLOVER_461_0.S[10445]++;return getFirstError() != null;
    } finally { }}

    /**
     *  A convienience, as most pages just show the first error on the page.
     * 
     *  <p>As of release 1.0.9, this returns an instance of {@link IRender}, not a {@link String}.
     * 
     **/

    public IRender getFirstError()
    {try { __CLOVER_461_0.M[2592]++;
        __CLOVER_461_0.S[10446]++;if ((((Tapestry.size(_trackings) == 0) && (++__CLOVER_461_0.CT[1792] != 0)) || (++__CLOVER_461_0.CF[1792] == 0))){
            __CLOVER_461_0.S[10447]++;return null;}

        __CLOVER_461_0.S[10448]++;Iterator i = _trackings.iterator();

        __CLOVER_461_0.S[10449]++;while ((((i.hasNext()) && (++__CLOVER_461_0.CT[1793] != 0)) || (++__CLOVER_461_0.CF[1793] == 0))){
        {
            __CLOVER_461_0.S[10450]++;IFieldTracking tracking = (IFieldTracking) i.next();

            __CLOVER_461_0.S[10451]++;if ((((tracking.isInError()) && (++__CLOVER_461_0.CT[1794] != 0)) || (++__CLOVER_461_0.CF[1794] == 0))){
                __CLOVER_461_0.S[10452]++;return tracking.getErrorRenderer();}
        }}

        __CLOVER_461_0.S[10453]++;return null;
    } finally { }}

    /**
     *  Checks to see if the field is in error.  This will <em>not</em> work properly
     *  in a loop, but is only used by {@link FieldLabel}.  Therefore, using {@link FieldLabel}
     *  in a loop (where the {@link IFormComponent} is renderred more than once) will not provide
     *  correct results.
     * 
     **/

    protected boolean isInError(IFormComponent component)
    {try { __CLOVER_461_0.M[2593]++;
        __CLOVER_461_0.S[10454]++;if ((((_trackingMap == null) && (++__CLOVER_461_0.CT[1795] != 0)) || (++__CLOVER_461_0.CF[1795] == 0))){
            __CLOVER_461_0.S[10455]++;return false;}

        __CLOVER_461_0.S[10456]++;IForm form = component.getForm();
        // if there is no form, the component cannot have been rewound or rendered into a form yet
        // so assume it cannot have errors.
        __CLOVER_461_0.S[10457]++;if ((((form == null) && (++__CLOVER_461_0.CT[1796] != 0)) || (++__CLOVER_461_0.CF[1796] == 0))){
            __CLOVER_461_0.S[10458]++;return false;}
        
        __CLOVER_461_0.S[10459]++;String formName = form.getName();
        __CLOVER_461_0.S[10460]++;Map formMap = (Map) _trackingMap.get(formName);

        __CLOVER_461_0.S[10461]++;if ((((formMap == null) && (++__CLOVER_461_0.CT[1797] != 0)) || (++__CLOVER_461_0.CF[1797] == 0))){
            __CLOVER_461_0.S[10462]++;return false;}

        __CLOVER_461_0.S[10463]++;IFieldTracking tracking = (IFieldTracking) formMap.get(component.getName());

        __CLOVER_461_0.S[10464]++;return tracking != null && tracking.isInError();
    } finally { }}

    /**
     *  Returns a {@link List} of {@link IFieldTracking}s.  This is the master list
     *  of trackings, except that it omits and trackings that are not associated
     *  with a particular field.  May return an empty list, or null.
     * 
     *  <p>Order is not determined, though it is likely the order in which components
     *  are laid out on in the template (this is subject to change).
     * 
     **/

    public List getAssociatedTrackings()
    {try { __CLOVER_461_0.M[2594]++;
        __CLOVER_461_0.S[10465]++;int count = Tapestry.size(_trackings);

        __CLOVER_461_0.S[10466]++;if ((((count == 0) && (++__CLOVER_461_0.CT[1798] != 0)) || (++__CLOVER_461_0.CF[1798] == 0))){
            __CLOVER_461_0.S[10467]++;return null;}

        __CLOVER_461_0.S[10468]++;List result = new ArrayList(count);

        __CLOVER_461_0.S[10469]++;for (int i = 0; (((i < count) && (++__CLOVER_461_0.CT[1799] != 0)) || (++__CLOVER_461_0.CF[1799] == 0)); i++){
        {
            __CLOVER_461_0.S[10470]++;IFieldTracking tracking = (IFieldTracking) _trackings.get(i);

            __CLOVER_461_0.S[10471]++;if ((((tracking.getFieldName() == null) && (++__CLOVER_461_0.CT[1800] != 0)) || (++__CLOVER_461_0.CF[1800] == 0))){
                __CLOVER_461_0.S[10472]++;continue;}

            __CLOVER_461_0.S[10473]++;result.add(tracking);
        }}

        __CLOVER_461_0.S[10474]++;return result;
    } finally { }}

    /**
     *  Like {@link #getAssociatedTrackings()}, but returns only the unassociated trackings.
     *  Unassociated trackings are new (in release 1.0.9), and are why
     *  interface {@link IFieldTracking} is not very well named.
     * 
     *  <p>The trackings are returned in an unspecified order, which (for the moment, anyway)
     *  is the order in which they were added (this could change in the future, or become
     *  more concrete).
     * 
     **/

    public List getUnassociatedTrackings()
    {try { __CLOVER_461_0.M[2595]++;
        __CLOVER_461_0.S[10475]++;int count = Tapestry.size(_trackings);

        __CLOVER_461_0.S[10476]++;if ((((count == 0) && (++__CLOVER_461_0.CT[1801] != 0)) || (++__CLOVER_461_0.CF[1801] == 0))){
            __CLOVER_461_0.S[10477]++;return null;}

        __CLOVER_461_0.S[10478]++;List result = new ArrayList(count);

        __CLOVER_461_0.S[10479]++;for (int i = 0; (((i < count) && (++__CLOVER_461_0.CT[1802] != 0)) || (++__CLOVER_461_0.CF[1802] == 0)); i++){
        {
            __CLOVER_461_0.S[10480]++;IFieldTracking tracking = (IFieldTracking) _trackings.get(i);

            __CLOVER_461_0.S[10481]++;if ((((tracking.getFieldName() != null) && (++__CLOVER_461_0.CT[1803] != 0)) || (++__CLOVER_461_0.CF[1803] == 0))){
                __CLOVER_461_0.S[10482]++;continue;}

            __CLOVER_461_0.S[10483]++;result.add(tracking);
        }}

        __CLOVER_461_0.S[10484]++;return result;
    } finally { }}
};