/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.wml;

import java.io.OutputStream;

import org.apache.tapestry.AbstractMarkupWriter;
import org.apache.tapestry.IMarkupWriter;
import org.apache.tapestry.util.text.ICharacterTranslatorSource;

/**
 *  This class is used to create WML output.
 *
 *  <p>The <code>WMLResponseWriter</code> handles the necessary escaping 
 *  of invalid characters.
 *  Specifically, the '$', '&lt;', '&gt;' and '&amp;' characters are properly
 *  converted to their WML entities by the <code>print()</code> methods.
 *  Similar measures are taken by the {@link #attribute(String, String)} method.
 *  Other invalid characters are converted to their numeric entity equivalent.
 *
 *  <p>This class makes it easy to generate trivial and non-trivial WML pages.
 *  It is also useful to generate WML snippets. It's ability to do simple
 *  formatting is very useful. A JSP may create an instance of the class
 *  and use it as an alternative to the simple-minded <b>&lt;%= ... %&gt;</b>
 *  construct, espcially because it can handle null more cleanly.
 *
 *  @author David Solis
 *  @since 0.2.9
 * 
 **/

public class WMLWriter extends AbstractMarkupWriter
{public static com.cortexeb.tools.clover.d __CLOVER_483_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private static final ICharacterTranslatorSource TRANSLATOR_SOURCE = 
    	new WMLCharacterTranslatorSource();
    
    /**
     *  Creates a response writer for content type "text/vnd.wap.wml".
     * 
     **/

    public WMLWriter(OutputStream stream)
    {
        this(stream, "UTF-8");__CLOVER_483_0.S[10679]++;try { __CLOVER_483_0.M[2641]++;
    } finally { }}

    /**
     * 
     * @param stream the output stream where to write the text
     * @param encoding the encoding to be used to generate the output
     * @since 3.0
     * 
     **/
    public WMLWriter(OutputStream stream, String encoding)
    {
        this("text/vnd.wap.wml", encoding, stream);__CLOVER_483_0.S[10680]++;try { __CLOVER_483_0.M[2642]++;
    } finally { }}

    public WMLWriter(String contentType, OutputStream stream)
    {
        super(TRANSLATOR_SOURCE, contentType, stream);__CLOVER_483_0.S[10681]++;try { __CLOVER_483_0.M[2643]++;
    } finally { }}

    /**
     * 
     * @param mimeType the MIME type to be used to generate the content type
     * @param encoding the encoding to be used to generate the output
     * @param stream the output stream where to write the text
     * @since 3.0
     * 
     **/
    public WMLWriter(String mimeType, String encoding, OutputStream stream)
    {
        super(TRANSLATOR_SOURCE, mimeType, encoding, stream);__CLOVER_483_0.S[10682]++;try { __CLOVER_483_0.M[2644]++;
    } finally { }}

    protected WMLWriter(String contentType)
    {
        super(TRANSLATOR_SOURCE, contentType);__CLOVER_483_0.S[10683]++;try { __CLOVER_483_0.M[2645]++;
    } finally { }}

    public IMarkupWriter getNestedWriter()
    {try { __CLOVER_483_0.M[2646]++;
        __CLOVER_483_0.S[10684]++;return new NestedWMLWriter(this);
    } finally { }}
}